/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  ADC ch-0..15 test in ADC ASYNC mode using software data download.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs test for ADC-0..3 channels #0..15 in ADC ASYNC mode 
	with ADC data download via DSP s/w as the following:
	  - DSP on-chip timer #0 is used as sampling frequency generator with simulation
	    of sampling frequency events via DSP s/w. This is the simple way to generate
		sampling frequency inside DSP application, which does not utilize any external
	    h/w resources and uses C6x DSP on-chip timer-0 only.
	  - ADC ASYNC DAQ mode is used
	  - ADC-0..3 are involved in DAQ and ADC IMUX ch-0..15 are acquired in each sampling
		frequency cycle
	  - ADC conversion is started via DSP s/w with ADC ASYNC acquisition enabled
	  - ADC conversion is performed during 0x800 sampling frequency cycles
	  - ADC data is downloaded to DSP environment memory using s/w download
	  - after A/D conversion completes, A/D input channel data can be veiwed via 
		Code Composer signal viewer as 16-bit integers in time domain at the following addresses:
		- ADC ch-0 data are available at 0x1000 address (0x80001000 for C64xx DSP)
		- ADC ch-1 data are available at 0x2000 address (0x80002000 for C64xx DSP)
		- ADC ch-2 data are available at 0x3000 address (0x80003000 for C64xx DSP)
		- ADC ch-3 data are available at 0x4000 address (0x80004000 for C64xx DSP)
		- ADC ch-4 data are available at 0x5000 address (0x80005000 for C64xx DSP)
		- ADC ch-5 data are available at 0x6000 address (0x80006000 for C64xx DSP)
		- ADC ch-6 data are available at 0x7000 address (0x80007000 for C64xx DSP)
		- ADC ch-7 data are available at 0x8000 address (0x80008000 for C64xx DSP)
		- ADC ch-8 data are available at 0x9000 address (0x80009000 for C64xx DSP)
		- ADC ch-9 data are available at 0xa000 address (0x8000a000 for C64xx DSP)
		- ADC ch-10 data are available at 0xb000 address (0x8000b000 for C64xx DSP)
		- ADC ch-11 data are available at 0xc000 address (0x8000c000 for C64xx DSP)
		- ADC ch-12 data are available at 0xd000 address (0x8000d000 for C64xx DSP)
		- ADC ch-13 data are available at 0xe000 address (0x8000e000 for C64xx DSP)
		- ADC ch-14 data are available at 0xf000 address (0x8000f000 for C64xx DSP)
		- ADC ch-15 data are available at 0x10000 address (0x80010000 for C64xx DSP)

	Note, that this ADC sampling/scan method is the most simplest one, it may provide 
	time jitter to input signal sampling time instants at high sampling frequencies 
	and may distort signal spectrum.
	

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file


	// pointers for saving ADC channel data and data length
#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
#define _DATQ_ACQ_BADDR		0x1000			// definition for C620x/C670x DSP
#else
#define _DATQ_ACQ_BADDR		0x80001000		// definition for C64xx DSP
#endif

#define	_DATA_ACQ_LEN	0x800L
#define	_SADDR_ADC_CH0	(_DATQ_ACQ_BADDR + 0*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH1	(_DATQ_ACQ_BADDR + 1*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH2	(_DATQ_ACQ_BADDR + 2*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH3	(_DATQ_ACQ_BADDR + 3*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH4	(_DATQ_ACQ_BADDR + 4*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH5	(_DATQ_ACQ_BADDR + 5*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH6	(_DATQ_ACQ_BADDR + 6*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH7	(_DATQ_ACQ_BADDR + 7*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH8	(_DATQ_ACQ_BADDR + 8*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH9	(_DATQ_ACQ_BADDR + 9*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH10	(_DATQ_ACQ_BADDR + 10*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH11	(_DATQ_ACQ_BADDR + 11*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH12	(_DATQ_ACQ_BADDR + 12*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH13	(_DATQ_ACQ_BADDR + 13*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH14	(_DATQ_ACQ_BADDR + 14*(_DATA_ACQ_LEN*2))
#define	_SADDR_ADC_CH15	(_DATQ_ACQ_BADDR + 15*(_DATA_ACQ_LEN*2))



main()
{ int i;
  int short *mptr0,*mptr1,*mptr2,*mptr3,
			*mptr4,*mptr5,*mptr6,*mptr7,
			*mptr8,*mptr9,*mptr10,*mptr11,
			*mptr12,*mptr13,*mptr14,*mptr15;

	init_cpu();

	// configure ADC DAQ: ASYNC mode, enable ACQ timing, disable Fs sampling
	TPDAS_A1M_SET_ADC_DAQ_MODE_ASYNC;
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_ACQ_EN_ON;
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_FS_EN_OFF;
	TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_ON;
	TPDAS_A1M_RESET_ADC_DAQ;

	// assign ADC data pointers
	mptr0 = (int short *) _SADDR_ADC_CH0;
	mptr1 = (int short *) _SADDR_ADC_CH1;
	mptr2 = (int short *) _SADDR_ADC_CH2;
	mptr3 = (int short *) _SADDR_ADC_CH3;
	mptr4 = (int short *) _SADDR_ADC_CH4;
	mptr5 = (int short *) _SADDR_ADC_CH5;
	mptr6 = (int short *) _SADDR_ADC_CH6;
	mptr7 = (int short *) _SADDR_ADC_CH7;
	mptr8 = (int short *) _SADDR_ADC_CH8;
	mptr9 = (int short *) _SADDR_ADC_CH9;
	mptr10 = (int short *) _SADDR_ADC_CH10;
	mptr11 = (int short *) _SADDR_ADC_CH11;
	mptr12 = (int short *) _SADDR_ADC_CH12;
	mptr13 = (int short *) _SADDR_ADC_CH13;
	mptr14 = (int short *) _SADDR_ADC_CH14;
	mptr15 = (int short *) _SADDR_ADC_CH15;


	// Use DSP on-chip timer #0 as the sampling frequency generator.
	// DSP internal INT14 is used as the timer Fs event indicator via DSP s/w polling mode.

		// configure C6x DSP on-chip timer#0
		// (25 kHz for C6201 DSP, corresponding scaling for other DSP types)
   	*C6X_TM0_CONTROL = 0;
   	*C6X_TM0_PERIOD  = 1000;					
   	*C6X_TM0_CONTROL = 0x03c1;

		// configure INT4 source selector
	C6X_INTMUX_SET(14,C6X_INTSEL_TINT0);
	C6X_INT_CLEAR_ALL;
	C6X_INT_ENABLE(14);

	// execute _DATA_ACQ_LEN sampling frequency cycles
	// each sampling frequency cycle starts on the DSP Timer-0 even and comprises of
	// acquisition of all A/D input channels
	for(i=0;i<_DATA_ACQ_LEN;i++)
	{
		// wait (poll) for valid Timer-0 event (INT14) occurs and clear it afterthat
		while(!(IFR & 0x4000));
		C6X_INT_CLEAR(14);

		// assume SE IMUX inputs will be used
		TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_DIFF_EN_OFF;

		// set ADC IMUX to channels 0-4-8-12 and start sampling/conversion
		TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_0_4_8_12;
		TPDAS_A1M_SET_ASYNC_ADC_START;

		// poll for ADC ready and read/save ADC channels #0-4-8-12 data
		while(TPDAS_A1M_ADC_RDY_IS_OFF);
		*mptr0++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC0_DATA);
		*mptr4++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC1_DATA);
		*mptr8++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC2_DATA);
		*mptr12++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC3_DATA);

		// set ADC IMUX to channels 1-5-9-13 and start sampling/conversion
		TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_1_5_9_13;
		TPDAS_A1M_SET_ASYNC_ADC_START;

		// poll for ADC ready and read/save ADC channels #1-5-9-13 data
		while(TPDAS_A1M_ADC_RDY_IS_OFF);
		*mptr1++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC0_DATA);
		*mptr5++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC1_DATA);
		*mptr9++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC2_DATA);
		*mptr13++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC3_DATA);


		// set ADC IMUX to channels 2-6-10-14 and start sampling/conversion
		TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_2_6_10_14;
		TPDAS_A1M_SET_ASYNC_ADC_START;

		// poll for ADC ready and read/save ADC channels #2-6-10-14 data
		while(TPDAS_A1M_ADC_RDY_IS_OFF);
		*mptr2++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC0_DATA);
		*mptr6++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC1_DATA);
		*mptr10++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC2_DATA);
		*mptr14++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC3_DATA);


		// set ADC IMUX to channels 3-7-11-15 and start sampling/conversion
		TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_3_7_11_15;
		TPDAS_A1M_SET_ASYNC_ADC_START;

		// poll for ADC ready and read/save ADC channels #3-7-11-15 data
		while(TPDAS_A1M_ADC_RDY_IS_OFF);
		*mptr3++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC0_DATA);
		*mptr7++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC1_DATA);
		*mptr11++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC2_DATA);
		*mptr15++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC3_DATA);
	}


	// loop infinitely at the end
 	while(1);	
}


