/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  ADC-0..3 test in ADC ASYNC mode with PFG sampling and IRT data download.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs test for ADC-0..3 and non-muxed channels-0/4/8/12 in ADC ASYNC 
	mode with PFG sampling frequency and ADC data download via ADC IRT:
	  - on-board 1MHz PFG is used as sampling frequency generator
	  - ADC ASYNC DAQ mode is used
	  - ADC-0..3 are used for A/D conversion
	  - ADC IMUX is set to channels 0/4/8/12 and is not updated during A/D acquisition
	  - ADC conversion is performed during 0x800 sampling frequency cycles
	  - ADC-0..3 data is downloaded to DSP environment memory using ADC IRT
		controller and DSP on-chip DMA
	  - after A/D data has been acquired, A/D input channel data can be viewed via 
		Code Composer signal viewer in time domain as 16-bit integers at the 
		following addresses:
		- ADC ch-0 data are available at 0x1000 address (0x80001000 for C64xx DSP)
		- ADC ch-4 data are available at 0x2000 address (0x80002000 for C64xx DSP)
		- ADC ch-8 data are available at 0x3000 address (0x80003000 for C64xx DSP)
		- ADC ch-12 data are available at 0x4000 address (0x80004000 for C64xx DSP)

	Note, that although this ADC sampling/scan method is one of simplest to handle, it 
	still does not provide any time jitter of A/D sampling event and does not distort signal 
	spectrum. Also, ADC ASYNC IRT data download does not utilize virtually none of DSP
	performance.
	

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file



	// set PFG to 1MHz = (25MHz*2*(V=128)/((R=50)*(S=2)*64*(X=1))
#define PFG_V_DATA		128
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV2
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1

#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


	// pointers for saving ADC channel data and data length
#define	_DAQ_LEN			0x800L

#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
#define _DATA_ACQ_BADDR			0x80001000		// acquired/packed data buffer for C620x/C670x DSP (DSP on-chip RAM)
#define _DATA_UNPACKED_BADDR	0x01000			// unpacked data buffer start for C620x/C670x DSP (external SBSRAM)
#else
#define _DATA_ACQ_BADDR			0x18000			// acquired/packed data buffer for C64xx DSP (DSP on-chip RAM)
#define _DATA_UNPACKED_BADDR	0x80008000		// unpacked data buffer start for C64xx DSP (external SBSRAM)
#endif

#define	_SADDR_ADC0				(_DATA_UNPACKED_BADDR + 0*(_DAQ_LEN*2))
#define	_SADDR_ADC1				(_DATA_UNPACKED_BADDR + 1*(_DAQ_LEN*2))
#define	_SADDR_ADC2				(_DATA_UNPACKED_BADDR + 2*(_DAQ_LEN*2))
#define	_SADDR_ADC3				(_DATA_UNPACKED_BADDR + 3*(_DAQ_LEN*2))


		// ADC input MUX channel selector and DIFF mode enable
#define	_ADC_IMUX_CH_SEL	TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_0_4_8_12
#define	_ADC_IMUX_DIFF_EN	TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_OFF



main()
{ int i;
  int short	*mptr0,*mptr1,*mptr2,*mptr3,*mptr_src;

	C6X_GINT_DISABLE;
	init_cpu();

	// reset ADC DAQ, FIFO and IRT
	TPDAS_A1M_RESET_ADC_DAQ;     		

	// configure ADC DAQ: ASYNC mode with Fs enabled, disable ACQ timing, high-speed.
	// ADC signal acquisition can be disabled since ADC IMUX is not 
	// updated between cycles.
	TPDAS_A1M_SET_ADC_DAQ_MODE_ASYNC;
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_ACQ_EN_OFF;
	TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_ON;
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_FS_EN_ON;

	// initially disable ADC-0..3 in order to disable DAQ and IRT.
	// Enable desired ADCs later after IRT and DMA will be configured in order to start
	// DAQ and ADC IRT.
	TPDAS_A1M_SET_ADC_ALL_EN_OFF;

	// configure ADC ASYNC IMUX
	TPDAS_A1M_set_adc_async_daq_imux_ch_sel(_ADC_IMUX_CH_SEL);
	TPDAS_A1M_set_adc_async_daq_imux_diff_en(_ADC_IMUX_DIFF_EN);

	// configure PFG
	TPDAS_A1M_SET_FS_SEL_PFG;
	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	// Use ADC IRT via HIRQ-0 PIOX IRQ line and DMA ch#0 for continuous 
	// ADC data upload. Note, that for TP6x boards, _IRT positive polarity output 
	// must be selected.
	//
	// important note:
	// ---------------
	// 	IRT interrupt at PIOX HIRQ must be enabled as the last step of IRT 
	//  initialization procedure after DMA controller will be configured and 
	//	enabled. This is required in order to exclude false IRT cycles, which
	// 	can occur due to internal DSP pipeline, which results in earlier 
	//	execution of DSP on-chip operations	rather than external peripherals access 
	//  even in case the latter appears	later in the DSP code. This can result in 
	//	DSP on-chip DMA is enabled prior T/PDAS-AD16/16Q/1M on-board peripherals will 
	//	be corectly initialized and	may lead to extra false IRT cycles in the very 
	//	beginning of the IRT process. You must get sure that PIOX HIRQx deselection 
	//	instructions will appear much earlier than DSP on-chip DMA controller will 
	//	be enabled.

		// Deselect any sources at PIOX IRQ-0 line.
		// This is required in order to disable IRT prior DMA will be enabled.
	TPDAS_A1M_SET_HIRQ0_OFF;	
	TPDAS_A1M_SET_HIRQ0_EN_ON;

		// Set HIRQ IRT polarity upon the TORNADO board type
		// note!!!: 
		//	T6x/E62/E67 require negative ADC IRT signal,
		//  whereas TP6x/E6202/E6203/E64xx require positive ADC IRT.
		//  TP6x and E6202/E6203/E64xx also require setting of MIRQ0 selector.
#if(_TP6X_SELECTED || _TP64XX_SELECTED)
		// configure DSP EXT_INT4=PIOX_IRQ-0
	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	
	TPDAS_A1M_SET_HIRQ0_POL_POS;
#else
#if(_E6X_SELECTED  || _E64XX_SELECTED)
	if( TE6202_TE6203_BOARD || TE64XX_BOARD)
	{
		// this is either of E6202/E6203/E64xx boards, which require
		// positive IRT and programmable setting of MIRQ0 selector
		TE6X_set_ext_int_sel(0,TE6X_EXT_INT_SEL_PXIRQ0);	
		TPDAS_A1M_SET_HIRQ0_POL_POS;
	}
	else
	{
		// this is E62/E67 with direct routing of XIRQ0~ to EXT_INT4,
		// which requires negative IRT and no MIRQ0 selector setting
		TPDAS_A1M_SET_HIRQ0_POL_NEG;
	}
#else
		// this is for T6x (always negative IRT, no MIRQ0 selector)
	TPDAS_A1M_SET_HIRQ0_POL_NEG;
#endif
#endif

		// configure C6x on-chip IRQ4 to source from EXT_INT4
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	

// configure DMA controller for C6201/C6202/C6203/C6701 based boards
// (note!!! C64xx EDMA controller requires different initialization)
#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
/*  configure DMA-0 to transfer ADC data via EXT_INT4 DSP interrupt (PIOX IRQ-0)
    without auto-init mode :
	- SRC DIR = no, DST DIR = +1
	- SRC SYNC = EXT_INT4, DST SYNC = no
	- esize = 16-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = no
	- DST RELOAD = NO 
	- CNT RELOAD = no
	- activate RSYNC/WSYNC events in case they have been missed

   PCRG @ DMA-0:   00 00 1 1 0 1 00000 00100 0 0 00 01 01 00 00 00	
*/

	*C6X_DMA0_PCRG = 0x0b010140;	
	*C6X_DMA0_SCRG = 0;
	*C6X_DMA0_SARG = TPDAS_A1M_ADC_IRT_DATA_RG_ADDR;
	*C6X_DMA0_DARG = _DATA_ACQ_BADDR;
	*C6X_DMA0_TCRG = (_DAQ_LEN*4);

	// clear false DMA-0 RSYNC event for ADC IRT data transfer
	*C6X_DMA0_SCRG=0xA000;		

	// perform ADC DAQ, FIFO and IRT reset in order to ensure that
	// IRT will start from the nearest Fs-event. This must be done prior
	// HIRQ0 will be linked to DMA synchro-event and DMA will be initialized.
	TPDAS_A1M_RESET_ADC_DAQ;     		

	// start DMA0 without auto-init mode
	*C6X_DMA0_PCRG|=0x1; 
#endif

	// Now enable ADC IRT by means of activating HIRQ0 source.
	TPDAS_A1M_SET_HIRQ0_SEL_ADC_IRT;

	// Finally, activate ADC DAQ and IRT by enabling desired ADCs in ADC CNF register.
	TPDAS_A1M_set_adc_cnf_rg(		TPDAS_A1M_ADC0_EN_ON 
								| 	TPDAS_A1M_ADC1_EN_ON 
								|	TPDAS_A1M_ADC2_EN_ON 
								|	TPDAS_A1M_ADC3_EN_ON
							);

	// loop here until DMA stops or ADC IRT error occures
	// check for DAC IRT errors. Abort on any error.
	i=0;
	while(1)
    { 
		// check for DMA stops as fast as possible in order to ensure that
		// DMA stop event will be detected earlier than IRT error
		if((*C6X_DMA0_TCRG)==0) break;

		// Perfrom read of ADC IRT error with relatively large period in order to
		// ensure that error status reads do not provide too much traffic to PIOX data bus
		// and DMA IRT has enough time room to complete in each sampling period.
		// Terminate acquisition if ADC IRT error occurs.
		if(++i ==100)
		{
			if(TPDAS_A1M_ADC_IRT_ERR_IS_ON)
				 goto error;
			i=0;
		}
	}

	// get here after ADC DAQ and IRT transfer completes successfully
	// In order to view ADC data via Code Composer viewer, data must be un-packed from the
	// quad-ADC frames to four linear ADC arrays

	// assign ADC data pointers
	mptr0 = (int short *) _SADDR_ADC0;
	mptr1 = (int short *) _SADDR_ADC1;
	mptr2 = (int short *) _SADDR_ADC2;
	mptr3 = (int short *) _SADDR_ADC3;
	mptr_src = (int short *) _DATA_ACQ_BADDR;
	
	for(i=0;i<_DAQ_LEN;i++)
	{
		*mptr0++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr1++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr2++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr3++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
	}

end_OK:
	// loop infinitely here on successful execution
	while(1);


error:          	   
	// trap here on any error
	// stop DMA0
	*C6X_DMA0_PCRG=0; 
	while(1);
	
}


