/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  ADC-0..3 to DAC-0..3 pass-thru test in ADC/DAC ASYNC mode with PFG sampling
  and ADC/DAC software data download/upload.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs continuous ADC->DAC test for ADC-0..3 non-muxed 
	channels-0/4/8/12 in ADC ASYNC DAQ mode and DAC-0..3 in DAC ASYNC DAQ mode 
	with PFG sampling frequency and ADC/DAC data s/w download/upload. 
	ADC-0..3 data are read via DSP software and stored in circular memory buffers 
	in DSP memory for user analysis in case acquisition is terminated via DSP halt. 
	DAC-0..3 are loaded by DSP software with the ADC-0..3 data, that provides 
	continuous ADC#n->DAC#n signal pass-thru demo. 
	  - on-board 250kHz PFG is used as ADC/DAC sampling frequency generator
	  - ADC ASYNC DAQ mode and DAC ASYNC DAQ mode are used
	  - ADC-0..3 and DAC-0..3 are used for A/D and D/A conversions
	  - ADC-0..3 data are continuously downloaded to DSP environment memory via
		DSP s/w and stored in circular data buffers:
		- ADC-0 data are available at 0x80001000 address (0x8000 for C64xx DSP)
		- ADC-1 data are available at 0x80002000 address (0x9000 for C64xx DSP)
		- ADC-2 data are available at 0x80003000 address (0xA000 for C64xx DSP)
		- ADC-3 data are available at 0x80004000 address (0xB000 for C64xx DSP)
	  - DAC-0..3 are continuously uploaded with ADC-0..3 data via DSP s/w
	  - ADC IMUX is set to channels 0/4/8/12 and is not used for A/D channels scan
		(can be change by corresponding modification of demo code). This results
		in the following ADC->DAC data pass-thru scheme:
			ADC IMUX ch-0  -> DAC-0
			ADC IMUX ch-4  -> DAC-1
			ADC IMUX ch-8  -> DAC-2
			ADC IMUX ch-12 -> DAC-3
		User can check for the ADC->DAC pass-thru operation by applying input
		signal at the corresponding ADC channel input.

	Note, that this ADC sampling/scan method is the most simplest one, however it 
	does not provide time jitter to input signal sampling time instants and DAC
	data output. However, is consumes DSP performance for ADC data download and DAC
	data upload.
	

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file



	// set PFG to 0.25MHz = (25MHz*2*(V=32)/((R=50)*(S=2)*64*(X=1))
#define PFG_V_DATA		32
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV2
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1

#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency



	// pointers for saving ADC channel data and data length
#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
#define _DATA_BADDR		0x80001000			// definition for C620x/C670x DSP
#else
#define _DATA_BADDR		0x8000				// definition for C64xx DSP
#endif

#define	_DATA_LEN		0x800L
#define	_ADC0_BADDR		(_DATA_BADDR + 0*(_DATA_LEN*2))
#define	_ADC1_BADDR		(_DATA_BADDR + 1*(_DATA_LEN*2))
#define	_ADC2_BADDR		(_DATA_BADDR + 2*(_DATA_LEN*2))
#define	_ADC3_BADDR		(_DATA_BADDR + 3*(_DATA_LEN*2))



main()
{ 
  	int i;
	int short adc0,adc1,adc2,adc3,*mptr0,*mptr1,*mptr2,*mptr3;


	init_cpu();

	// configure ADC DAQ: ASYNC mode, disable ACQ timing, enable Fs sampling
	TPDAS_A1M_SET_ADC_DAQ_MODE_ASYNC;
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_ACQ_EN_OFF;
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_FS_EN_ON;
	TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_ON;
	TPDAS_A1M_RESET_ADDA_DAQ;

	// assume ADC SE IMUX inputs will be used
	// set ADC IMUX to channels 0-4-8-12
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_0_4_8_12;
	
	// configure DAC DAQC: ASYNC mode, select Fs as DAC output data load strobe
	TPDAS_A1M_SET_DAC_DAQ_MODE_ASYNC;
	TPDAS_A1M_SET_DAC_ASYNC_DAQ_OUTPUT_MODE_FS;

	// select PFG as the sampling frequency source
	TPDAS_A1M_SET_FS_SEL_PFG;
	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	// Loop infinitely in sampling frequency cycles.
	// Each sampling frequency cycle starts on polling the ADC READY and copy ADC data to DACs.
	// Write ADC data to the circular buffer.
	i=0;
	mptr0 = (int short*) _ADC0_BADDR;
	mptr1 = (int short*) _ADC1_BADDR;
	mptr2 = (int short*) _ADC2_BADDR;
	mptr3 = (int short*) _ADC3_BADDR;
	
	while(1)
	{
		// wait (poll) for ADC ready
		while(TPDAS_A1M_ADC_RDY_IS_OFF);

		// read ADC data and copy ADC data to DACs
		TPDAS_A1M_SET_DAC0_DATA(adc0=TPDAS_A1M_GET_ADC0_DATA);
		TPDAS_A1M_SET_DAC1_DATA(adc1=TPDAS_A1M_GET_ADC1_DATA);
		TPDAS_A1M_SET_DAC2_DATA(adc2=TPDAS_A1M_GET_ADC2_DATA);
		TPDAS_A1M_SET_DAC3_DATA(adc3=TPDAS_A1M_GET_ADC3_DATA);

		// save ADC data to circular memory buffers
		*mptr0++ = adc0;
		*mptr1++ = adc1;
		*mptr2++ = adc2;
		*mptr3++ = adc3;
		
		// check for end of circular buffer
		if (++i == _DATA_LEN)
		{
			mptr0 = (int short*) _ADC0_BADDR;
			mptr1 = (int short*) _ADC1_BADDR;
			mptr2 = (int short*) _ADC2_BADDR;
			mptr3 = (int short*) _ADC3_BADDR;
			i=0;
		}

		// reset ADC DAQ in order to clear ADC READY		
		TPDAS_A1M_RESET_ADC_DAQ;
	}
}


