/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  DAC-0..3 test in DAC ASYNC mode using software data upload and DAC 
  TRANSPARENT Load mode.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs continuous test for DAC-0..3 in DAC ASYNC mode with 
	DAC data software data upload as the following:
	  - DSP on-chip timer #0 is used as sampling frequency generator with simulation
	    of sampling frequency events via DSP s/w. This is the simple way to generate
		sampling frequency inside DSP application, which does not utilize any external
	    h/w resources and uses C6x DSP on-chip timer-0 only.
	  - DAC-0 data array is filled in with the SINE data. DAC-1 data array is filled in 
	    with the rising ramp data. DAC-2 data array is filled in with the falling
	    ramp data. DAC-3 data array is filled in with the square pulse data.
	  - DAC ASYNC mode and DAC ASYNC SW OUTPUT Load mode are used.
	  - DAC-0..3 are uploaded continuously using via host DSP software without 
	    IRT and DMA.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file


	// ADC/DAC DAQ data length
#define	_DAQ_LEN		0x0400

		// DAC data array pointers
		// T6x/TP6x/E6x: DSP on-chip DATA RAM
		// TP64xx/E64xx: external SBSRAM
#define	_DAC0_DATA_PTR		(0x80008000)	
#define	_DAC1_DATA_PTR		(0x80008000+1*(_DAQ_LEN*2))	
#define	_DAC2_DATA_PTR		(0x80008000+2*(_DAQ_LEN*2))	
#define	_DAC3_DATA_PTR		(0x80008000+3*(_DAQ_LEN*2))	


main()
{ int i;
  int short *mptr0, *mptr1, *mptr2, *mptr3;

	C6X_GINT_DISABLE;
	init_cpu();


	// configure DAC DAQ mode and DAC Load mode
	TPDAS_A1M_SET_DAC_DAQ_MODE_ASYNC;
	TPDAS_A1M_SET_DAC_ASYNC_DAQ_OUTPUT_MODE_SFT;

	// enable DAC-0..3
	// (this is actually not required for ASYNC mode unless ASYNC DAC IRT is used)
	TPDAS_A1M_set_dac_cnf_rg(		TPDAS_A1M_DAC0_EN_ON 
								| 	TPDAS_A1M_DAC1_EN_ON 
								|	TPDAS_A1M_DAC2_EN_ON 
								| 	TPDAS_A1M_DAC3_EN_ON
							);


	// now fill-in DAC-0..3 data arrays for further continuous transfer to DACs
	mptr0 = (int short *) _DAC0_DATA_PTR;
	mptr1 = (int short *) _DAC1_DATA_PTR;
	mptr2 = (int short *) _DAC2_DATA_PTR;
	mptr3 = (int short *) _DAC3_DATA_PTR;

	for(i=0;i<_DAQ_LEN;i++)
	{ 		
		//-- fill-in DAC-0 as SINE signal
		*mptr0++ = (int short)((float)  (32000.0 * sin(6.28 *((float)(i%256))/256.0))) ;
		
		//-- fill-in DAC-1 as the rising RAMP signal
		*mptr1++ = (int short)((float)  (32000.0 * ((float)((i%256)-128))/128.0)) ;

		//-- fill-in DAC-2 as the falling RAMP signal
		*mptr2++ = (int short)((float)  (32000.0 * ((float)(128-(i%256)))/128.0)) ;

		//-- fill-in DAC-3 as the square pulse signal
		*mptr3++ = (int short) (((i%256)<128) ? 32000:-32000);
	  }


	// Use DSP on-chip timer #0 as the sampling frequency generator (approximately
	// 100 kHz for C6201 DSP, corrsponding scaling for other DSP types).
	// Use DSP internal INT14 as the timer Fs event indicator via DSP s/w polling mode.

		// configure C6x DSP on-chip timer#0
   	*C6X_TM0_CONTROL = 0;
   	*C6X_TM0_PERIOD  = 250;		// this will provide 100kHz@C6201
   	*C6X_TM0_CONTROL = 0x03c1;

		// configure INT4 source selector
	C6X_INTMUX_SET(14,C6X_INTSEL_TINT0);
	C6X_INT_CLEAR_ALL;
	C6X_INT_ENABLE(14);


	// set DAC data array pointers
	mptr0 = (int short *) _DAC0_DATA_PTR;
	mptr1 = (int short *) _DAC1_DATA_PTR;
	mptr2 = (int short *) _DAC2_DATA_PTR;
	mptr3 = (int short *) _DAC3_DATA_PTR;
	i = 0;

	// perform continuous DAC data upload at Timer-0 events
	while(1)
	{
		// wait (poll) for valid Timer-0 event (INT14) occurs and clear it afterthat
		while(!(IFR & 0x4000));
		C6X_INT_CLEAR(14);

		// upload DAC-0..3 data to DAC data registers
		TPDAS_A1M_SET_DAC0_DATA(*mptr0++);
		TPDAS_A1M_SET_DAC1_DATA(*mptr1++);
		TPDAS_A1M_SET_DAC2_DATA(*mptr2++);
		TPDAS_A1M_SET_DAC3_DATA(*mptr3++);

		// issue common DAC load signal in order to load all DACs outputs synchronously
		TPDAS_A1M_SET_ASYNC_DAC_OUTPUT;

		// now check whether DAC array pointers need to be reset for circular data output
		if(++i >= _DAQ_LEN)
		{
			mptr0 = (int short *) _DAC0_DATA_PTR;
			mptr1 = (int short *) _DAC1_DATA_PTR;
			mptr2 = (int short *) _DAC2_DATA_PTR;
			mptr3 = (int short *) _DAC3_DATA_PTR;
			i = 0;
		}
	}
}


