/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  ADC ch-0..15 test in ADC SYNC-FIFO_OPM mode with PFG sampling and 
  ADC FIFO software data download/unpacking.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs ADC test in ADC SYNC-FIFO_OPM mode with ADC FIFO software
	data download and unpacking:
	  - on-board 250kHz PFG is used as sampling frequency generator
	  - ADC SYNC-FIFO_OPM DAQ mode is used
	  - A/D conversion is started by DSP s/w without external start-up 
		synchronization
	  - ADC-0..3 are used for A/D conversion
	  - ADC IMUX is configured to scan all ADC ch-0..15 input channels via
		four ADC scan cycles within each sampling frequency cycle
	  - A/D conversion is automatically terminated after FIFO is full
	  - ADC FIFO data is downloaded after DAQ terminates to DSP environment memory 
		via DSP software and 0x400 samples of each channel are stored in
		on-board DSP memory
	  - after ADC DAQ terminates, ADC data can be viewed via Code Composer signal 
		viewer in time domain at the following addresses:
		- ADC ch-0 data are available at 0x1000 address (0x80001000 for C64xx DSP)
		- ADC ch-1 data are available at 0x1800 address (0x80001800 for C64xx DSP)
		- ADC ch-2 data are available at 0x2000 address (0x80002000 for C64xx DSP)
		- ADC ch-3 data are available at 0x2800 address (0x80002800 for C64xx DSP)
		- ADC ch-4 data are available at 0x3000 address (0x80003000 for C64xx DSP)
		- ADC ch-5 data are available at 0x3800 address (0x80003800 for C64xx DSP)
		- ADC ch-6 data are available at 0x4000 address (0x80004000 for C64xx DSP)
		- ADC ch-7 data are available at 0x4800 address (0x80004800 for C64xx DSP)
		- ADC ch-8 data are available at 0x5000 address (0x80005000 for C64xx DSP)
		- ADC ch-9 data are available at 0x5800 address (0x80005800 for C64xx DSP)
		- ADC ch-10 data are available at 0x6000 address (0x80006000 for C64xx DSP)
		- ADC ch-11 data are available at 0x6800 address (0x80006800 for C64xx DSP)
		- ADC ch-12 data are available at 0x7000 address (0x80007000 for C64xx DSP)
		- ADC ch-13 data are available at 0x7800 address (0x80007800 for C64xx DSP)
		- ADC ch-14 data are available at 0x8000 address (0x80008000 for C64xx DSP)
		- ADC ch-15 data are available at 0x8800 address (0x80008800 for C64xx DSP)


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file


	// set PFG to 0.25MHz = (25*2*(V=64)/((R=50)*(S=4)*64*(X=1))
#define PFG_V_DATA		64
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV4
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1

#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


	// pointers for saving ADC channel data and data length
#define	_DAQ_LEN			0x400L

#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
#define _DATA_ACQ_BADDR			0x80001000		// acquired/packed data buffer for C620x/C670x DSP (DSP on-chip RAM)
#define _DATA_UNPACKED_BADDR	0x01000			// unpacked data buffer start for C620x/C670x DSP (external SBSRAM)
#else
#define _DATA_ACQ_BADDR			0x18000			// acquired/packed data buffer for C64xx DSP (DSP on-chip RAM)
#define _DATA_UNPACKED_BADDR	0x80008000		// unpacked data buffer start for C64xx DSP (external SBSRAM)
#endif

#define	_ADC_CH0_DATA			(_DATA_UNPACKED_BADDR + 0*(_DAQ_LEN*2))
#define	_ADC_CH1_DATA			(_DATA_UNPACKED_BADDR + 1*(_DAQ_LEN*2))
#define	_ADC_CH2_DATA			(_DATA_UNPACKED_BADDR + 2*(_DAQ_LEN*2))
#define	_ADC_CH3_DATA			(_DATA_UNPACKED_BADDR + 3*(_DAQ_LEN*2))
#define	_ADC_CH4_DATA			(_DATA_UNPACKED_BADDR + 4*(_DAQ_LEN*2))
#define	_ADC_CH5_DATA			(_DATA_UNPACKED_BADDR + 5*(_DAQ_LEN*2))
#define	_ADC_CH6_DATA			(_DATA_UNPACKED_BADDR + 6*(_DAQ_LEN*2))
#define	_ADC_CH7_DATA			(_DATA_UNPACKED_BADDR + 7*(_DAQ_LEN*2))
#define	_ADC_CH8_DATA			(_DATA_UNPACKED_BADDR + 8*(_DAQ_LEN*2))
#define	_ADC_CH9_DATA			(_DATA_UNPACKED_BADDR + 9*(_DAQ_LEN*2))
#define	_ADC_CH10_DATA			(_DATA_UNPACKED_BADDR + 10*(_DAQ_LEN*2))
#define	_ADC_CH11_DATA			(_DATA_UNPACKED_BADDR + 11*(_DAQ_LEN*2))
#define	_ADC_CH12_DATA			(_DATA_UNPACKED_BADDR + 12*(_DAQ_LEN*2))
#define	_ADC_CH13_DATA			(_DATA_UNPACKED_BADDR + 13*(_DAQ_LEN*2))
#define	_ADC_CH14_DATA			(_DATA_UNPACKED_BADDR + 14*(_DAQ_LEN*2))
#define	_ADC_CH15_DATA			(_DATA_UNPACKED_BADDR + 15*(_DAQ_LEN*2))


	// ADC FIFO PAF/PAE offsets, which are used to test ADC FIFO flags
#define	_adc_pae_offset		0x1000
#define	_adc_paf_offset		0x2000



main()
{ int i,j;
  int short adc_fifo_data;
  int short	*mptr0,*mptr1,*mptr2,*mptr3,
			*mptr4,*mptr5,*mptr6,*mptr7,
			*mptr8,*mptr9,*mptr10,*mptr11,
			*mptr12,*mptr13,*mptr14,*mptr15;


	C6X_GINT_DISABLE;
	init_cpu();

	// reset ADC DAQ, FIFO and IRT
	TPDAS_A1M_RESET_ADDA_DAQ;     		

	// configure ADC DAQ mode: SYNC-FIFO_OPM with s/w start-up synchronization,
	// ADC high-speed mode, extended inter-scan_cycle timing is disabled
	TPDAS_A1M_SET_ADC_DAQ_MODE_SYNC_FIFO_OPM;
	TPDAS_A1M_SET_ADC_SYNC_DAQ_SSYNC_SEL_SFT;
	TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_ON;
	TPDAS_A1M_SET_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_OFF;

	// set ADC DAQ termination flag to FF
	TPDAS_A1M_SET_ADC_SYNC_FIFO_DAQ_TF_SEL_FF;

	// configure PFG
	TPDAS_A1M_SET_FS_SEL_PFG;
	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	// enable desired ADCs in ADC CNF register, which is required for ADC DAQ and IRT
	TPDAS_A1M_set_adc_cnf_rg(		TPDAS_A1M_ADC0_EN_ON 
								| 	TPDAS_A1M_ADC1_EN_ON 
								|	TPDAS_A1M_ADC2_EN_ON 
								|	TPDAS_A1M_ADC3_EN_ON
							);

	// configure ADC IMUX for each scan cycle
	// set EOS flag for the 3rd cycle only (is set always via hardware)
	TPDAS_A1M_SET_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12;
	TPDAS_A1M_SET_ADC_SCCL0_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_SCCL0_EOS_EN_OFF;

	TPDAS_A1M_SET_ADC_SCCL1_IMUX_CH_SEL_1_5_9_13;
	TPDAS_A1M_SET_ADC_SCCL1_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_SCCL1_EOS_EN_OFF;

	TPDAS_A1M_SET_ADC_SCCL2_IMUX_CH_SEL_2_6_10_14;
	TPDAS_A1M_SET_ADC_SCCL2_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_SCCL2_EOS_EN_OFF;

	TPDAS_A1M_SET_ADC_SCCL3_IMUX_CH_SEL_3_7_11_15;
	TPDAS_A1M_SET_ADC_SCCL3_IMUX_DIFF_EN_OFF;

	// configure ADC FIFO PAF/PAE flags
	if(   TPDAS_A1M_set_adc_fifo_offset(_adc_pae_offset*16,_adc_paf_offset*16)!=0
	  )
            goto error;

	// again, reset ADC DAQ, FIFO, IRT and ADC errors to ensure no errors
	TPDAS_A1M_RESET_ADC_DAQ;     		

	// check for ADC FIFO flags and errors
	if(		TPDAS_A1M_ADC_FIFO_FF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_PAF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_PAE_IS_OFF
		||	TPDAS_A1M_ADC_FIFO_EF_IS_OFF
		||	TPDAS_A1M_ADC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_UNF_IS_ON
	  )
		goto error;

	// enable ADC SYNC DAQ and check for errors
	TPDAS_A1M_SET_ADC_SYNC_DAQ_RUN_ON;

	if(		TPDAS_A1M_ADC_SYNC_DAQ_RUN_IS_OFF
		||	TPDAS_A1M_ADC_SYNC_DAQ_END_IS_ON
	  )
		goto error;

	// wait until ADC DAQ terminates
	while(TPDAS_A1M_ADC_SYNC_DAQ_RUN_IS_ON);

	// check FIFO flags and ADC DAQ END flags
	if(		TPDAS_A1M_ADC_FIFO_FF_IS_OFF
		||	TPDAS_A1M_ADC_FIFO_PAF_IS_OFF
		||	TPDAS_A1M_ADC_FIFO_PAE_IS_ON
		||	TPDAS_A1M_ADC_FIFO_EF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_UNF_IS_ON
		||	TPDAS_A1M_ADC_SYNC_DAQ_END_IS_OFF
	  )
		goto error;


	// Now read and unpack ADC FIFO data, and verify ADC FIFO flags procurement
	// during ADC FIFO read.
	// Only first 0x200 samples of each A/D channel is unpacked into DSP memory
	// and can be further viewed via Code COmpser signal viewer, whereas the 
	// following A/D data in FIFO are just read and lost in order to test ADC FIFO 
	// PAF/PAE flags.

		// set data array pointers
	mptr0 = (int short *) _ADC_CH0_DATA;
	mptr1 = (int short *) _ADC_CH1_DATA;
	mptr2 = (int short *) _ADC_CH2_DATA;
	mptr3 = (int short *) _ADC_CH3_DATA;
	mptr4 = (int short *) _ADC_CH4_DATA;
	mptr5 = (int short *) _ADC_CH5_DATA;
	mptr6 = (int short *) _ADC_CH6_DATA;
	mptr7 = (int short *) _ADC_CH7_DATA;
	mptr8 = (int short *) _ADC_CH8_DATA;
	mptr9 = (int short *) _ADC_CH9_DATA;
	mptr10 = (int short *) _ADC_CH10_DATA;
	mptr11 = (int short *) _ADC_CH11_DATA;
	mptr12 = (int short *) _ADC_CH12_DATA;
	mptr13 = (int short *) _ADC_CH13_DATA;
	mptr14 = (int short *) _ADC_CH14_DATA;
	mptr15 = (int short *) _ADC_CH15_DATA;

	for(i=0;i<(TPDAS_A1M_FIFO_LEN/16);i++)
	{

		if(i<_DAQ_LEN)
		{
			// for first 0x200 samples, unpack data from quad-ADC data frames
			// each data frame contains data from four ADCs
			*mptr0++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr4++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr8++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr12++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);

			*mptr1++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr5++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr9++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr13++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);

			*mptr2++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr6++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr10++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr14++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);

			*mptr3++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr7++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr11++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
			*mptr15++ = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
		}
		else
		{
			// otherwise just read ADC FIFO and forget data
			for(j=0;j<16;j++)
				adc_fifo_data = TPDAS_A1M_CONV_ADC_TO_INT16(TPDAS_A1M_GET_ADC_FIFO_DATA);
		}


		// check for ADC FIFO errors
		if(	TPDAS_A1M_GET_ADC_FIFO_ERR_ALL )
			goto error;

		// verify ADC FIFO EF/FF/PAF/PAE flag procurement
	  	if( 	(i!=((TPDAS_A1M_FIFO_LEN/16)-1) &&  TPDAS_A1M_ADC_FIFO_EF_IS_ON)
			||  ((i+1)<((TPDAS_A1M_FIFO_LEN/16)-_adc_pae_offset) && TPDAS_A1M_ADC_FIFO_PAE_IS_ON)
			||  ((i+1)>=((TPDAS_A1M_FIFO_LEN/16)-_adc_pae_offset)  && TPDAS_A1M_ADC_FIFO_PAE_IS_OFF)
			||  ((i+1)<=((TPDAS_A1M_FIFO_LEN/16)-_adc_paf_offset)  && TPDAS_A1M_ADC_FIFO_PAF_IS_OFF)
			||  ((i+1)>((TPDAS_A1M_FIFO_LEN/16)-_adc_paf_offset) && TPDAS_A1M_ADC_FIFO_PAF_IS_ON)
			||	TPDAS_A1M_ADC_FIFO_FF_IS_ON
	      ) 
			goto error;
	}
	

end_OK:
	// loop infinitely here on successful execution
	while(1);


error:          	   
	// trap here on any error
	// stop DMA0 and terminate ADC DAQ
	*C6X_DMA0_PCRG=0; 
	TPDAS_A1M_SET_ADC_SYNC_DAQ_ABORT_ON;
	while(1);
	
}


