/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  ADC-to-DAC pass-thru test in SYNC-FIFO_PTM DAQ mode mode with IRT data 
  download/upload.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs continuous ADC-to-DAC pass-thru test in ADC/DAC SYNC-FIFO_PTM
	DAQ mode using ADC/DAC IRT controllers for ADC/DAC data download/upload.

	ADC non-muxed channels #0/4/8/12 and ADC-0..3 are involved into the demo on the
	A/D side and DAC-0..3 are involved on the D/A side. ADC#n data are downloaded 
	to the DSP memory via ADC IRT controller and then uploaded to DAC#n via DAC 
	IRT controller. This provides continuous ADC#n->DAC#n signal pass-thru demo.:
	  - on-board 0.5MHz PFG is used as ADC/DAC sampling frequency generator
		(higher sampling frequency may not run perfectly due to C6x DSP featured 
		 delays in initializing DSP on-chip DMA controllers)
	  - ADC SYNC-FIFO_PTM DAQ mode is used 
	  - DAC SYNC-FIFO_PTM DAQ mode is used 
	  - ADC-0..3 and DAC-0..3 are used for A/D and D/A conversions
	  - ADC IMUX is set statically to channels 0/4/8/12 and is not updated
		dynamically via scan cycles. This results in the following ADC->DAC data 
		pass-thru scheme:
			ADC IMUX ch-0  -> DAC-0
			ADC IMUX ch-4  -> DAC-1
			ADC IMUX ch-8  -> DAC-2
			ADC IMUX ch-12 -> DAC-3
		User can check for the ADC->DAC pass-thru operation by applying input
		signal at the corresponding ADC channel input. Involved A/D channels can be 
		modified by user.
	  - ADC-0..3 data are continuously downloaded to shared circular memory buffer in DSP 
		on-chip memory using ADC IRT controller and DSP on-chip DMA-0 channel, 
		whereas DAC-0..3 data are continuously uploaded from the same shared memory
		buffer using DAC IRT controller and DSP on-chip DMA-1 channel. 



   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file


	// set PFG to 0.5MHz = (25MHz*2*(V=64)/((R=50)*(S=2)*64*(X=1))
#define PFG_V_DATA		64
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV2
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1

#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency



	// pointers for saving ADC channel data and data length
#define	_DAQ_LEN				0x400L

#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
#define _DATA_ACQ_BADDR			0x80001000		// shared ADC/DAC data buffer for C620x/C670x DSP (DSP on-chip RAM)
#else
#define _DATA_ACQ_BADDR			0x00080000		// shared ADC/DAC data buffer for C64xx DSP (DSP on-chip RAM)
#endif


main()
{ int i;

	C6X_GINT_DISABLE;
	init_cpu();

	// reset ADC/DAC DAQ, FIFO, IRT and errors
	TPDAS_A1M_RESET_ADDA_DAQ;     		

	// configure ADC DAQ mode: SYNC-noFIFO with s/w start-up synchronization,
	// ADC high-speed mode, extended inter-scan_cycle timing is disabled
	TPDAS_A1M_SET_ADC_DAQ_MODE_SYNC_FIFO_PTM;
	TPDAS_A1M_SET_ADC_SYNC_DAQ_SSYNC_SEL_SFT;
	TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_ON;

	// configure DAC DAQ mode: SYNC-noFIFO with s/w start-up synchronization 
	TPDAS_A1M_SET_DAC_DAQ_MODE_SYNC_FIFO_PTM;
	TPDAS_A1M_SET_DAC_SYNC_DAQ_SSYNC_SEL_SFT;

	// set ADC/DAC IRT start-up and termination flags to PAE/PAF flag
	TPDAS_A1M_SET_ADC_SYNC_FIFO_DAQ_TF_SEL_PAF;
	TPDAS_A1M_SET_ADC_SYNC_FIFO_IRT_TF_SEL_PAE;
	TPDAS_A1M_SET_DAC_SYNC_FIFO_DAQ_TF_SEL_PAE;
	TPDAS_A1M_SET_DAC_SYNC_FIFO_IRT_TF_SEL_PAF;

	// configure PFG
	TPDAS_A1M_SET_FS_SEL_PFG;
	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	// enable ADC-0..3 and DAC-0..3 (required for DAQ and IRT)
	TPDAS_A1M_set_adc_cnf_rg(		TPDAS_A1M_ADC0_EN_ON 
								| 	TPDAS_A1M_ADC1_EN_ON 
								|	TPDAS_A1M_ADC2_EN_ON 
								| 	TPDAS_A1M_ADC3_EN_ON
							);

	TPDAS_A1M_set_dac_cnf_rg(		TPDAS_A1M_DAC0_EN_ON 
								| 	TPDAS_A1M_DAC1_EN_ON 
								|	TPDAS_A1M_DAC2_EN_ON 
								| 	TPDAS_A1M_DAC3_EN_ON
							);

	// configure ADC IMUX to use scan cycle #0 only and convert channels #0/4/8/12 in SE-mode
	TPDAS_A1M_SET_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12;
	TPDAS_A1M_SET_ADC_SCCL0_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_SCCL0_EOS_EN_ON;

	// configure ADC/DAC FIFO PAF/PAE flags
	if(   	TPDAS_A1M_set_dac_fifo_offset(_DAQ_LEN*4/2,_DAQ_LEN*4)!=0
		||	TPDAS_A1M_set_adc_fifo_offset(_DAQ_LEN*4/2,_DAQ_LEN*4)!=0
	  )
            goto error;

	// again, reset ADC/DAC DAQ, FIFO, IRT and ADC/DAC errors to ensure no errors
	TPDAS_A1M_RESET_ADDA_DAQ;     		

	// check for ADC/DAC FIFO flags and errors
	if(		TPDAS_A1M_DAC_FIFO_FF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAE_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_EF_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_FF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_PAF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_PAE_IS_OFF
		||	TPDAS_A1M_ADC_FIFO_EF_IS_OFF
		||	TPDAS_A1M_ADC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_ADC_FIFO_UNF_IS_ON
		||	TPDAS_A1M_ADC_IRT_ERR_IS_ON
		||	TPDAS_A1M_DAC_IRT_ERR_IS_ON
	  )
		goto error;


	// Use ADC IRT via HIRQ-0 PIOX IRQ line and DMA ch#0 for continuous 
	// ADC data upload. 
	// Use DAC IRT via HIRQ-1 PIOX IRQ line and DMA ch#1 for continuous 
	// DAC data upload. 
	//
	// important note:
	// ---------------
	// 	IRT interrupt at PIOX HIRQ must be enabled as the last step of IRT 
	//  initialization procedure after DMA controller will be configured and 
	//	enabled. 

		// Deselect any sources at PIOX IRQ-0/1 lines.
		// This is required in order to disable IRT prior DMA will be enabled.
	TPDAS_A1M_SET_HIRQ0_OFF;	
	TPDAS_A1M_SET_HIRQ0_EN_ON;
	TPDAS_A1M_SET_HIRQ1_OFF;	
	TPDAS_A1M_SET_HIRQ1_EN_ON;

		// Set HIRQ IRT polarity upon the TORNADO board type
		// note!!!: 
		//	T6x/E62/E67 require negative ADC/DAC IRT signal,
		//  whereas TP6x/E6202/E6203/E64xx require positive ADC/DAC IRT.
		//  TP6x and E6202/E6203/E64xx also require setting of MIRQ0/1 selectors.
#if(_TP6X_SELECTED || _TP64XX_SELECTED)
		// configure DSP EXT_INT4=PIOX_IRQ-0 and EXT_INT5=PIOX_IRQ-1
	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	
	TP6X_set_mirq_sel(1,TP6X_MIRQ_SEL_XIRQ3);	
	TPDAS_A1M_SET_HIRQ0_POL_POS;
	TPDAS_A1M_SET_HIRQ1_POL_POS;
#else
#if(_E6X_SELECTED  || _E64XX_SELECTED)
	if( TE6202_TE6203_BOARD || TE64XX_BOARD)
	{
		// this is either of E6202/E6203/E64xx boards, which require
		// positive IRT and programmable setting of MIRQ0/1 selectors
		TE6X_set_ext_int_sel(0,TE6X_EXT_INT_SEL_PXIRQ0);	
		TE6X_set_ext_int_sel(1,TE6X_EXT_INT_SEL_PXIRQ1);	
		TPDAS_A1M_SET_HIRQ0_POL_POS;
		TPDAS_A1M_SET_HIRQ1_POL_POS;
	}
	else
	{
		// this is E62/E67 with direct routing of XIRQ0~/XIRQ1~ to EXT_INT4/EXT_INT5,
		// which requires negative IRT and no MIRQ0/1 selectors settings
		TPDAS_A1M_SET_HIRQ0_POL_NEG;
		TPDAS_A1M_SET_HIRQ1_POL_NEG;
	}
#else
		// this is for T6x (always negative IRT, no MIRQ0 selector)
	TPDAS_A1M_SET_HIRQ0_POL_NEG;
	TPDAS_A1M_SET_HIRQ1_POL_NEG;
#endif
#endif

		// configure C6x on-chip IRQ4/5 to source from EXT_INT4/5
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	
	C6X_INTMUX_SET(5,C6X_INTSEL_XINT7);	


// configure DMA controller for C6201/C6202/C6203/C6701 based boards
// (note!!! C64xx EDMA controller requires different initialization)
#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
/*  configure DMA-0 to transfer ADC data via EXT_INT4 DSP interrupt (PIOX IRQ-0)
    in auto-init mode :
	- SRC DIR = no, DST DIR = +1
	- SRC SYNC = EXT_INT4, DST SYNC = no
	- esize = 16-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = no
	- DST RELOAD = from global address B
	- CNT RELOAD = from global counter rg A
	- activate RSYNC/WSYNC events in case they have been missed

   PCRG @ DMA-0:   01 00 1 1 0 1 00000 00100 0 0 00 01 01 00 00 00	
*/
	*C6X_DMA0_PCRG = 0x4b010140;	
	*C6X_DMA0_SCRG = 0;
	*C6X_DMA0_SARG = TPDAS_A1M_ADC_FIFO_DATA_RG_ADDR;
	*C6X_DMA0_DARG = _DATA_ACQ_BADDR;
	*C6X_DMA0_TCRG = (_DAQ_LEN*4);


/*	configure DMA-1 to transfer DAC data via EXT_INT5 DSP interrupt (PIOX IRQ-1)
    in auto-init mode:
	- SRC DIR = +1, DST DIR = no
	- SRC SYNC = no, DST SYNC = EXT_INT5
	- esize = 16-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = from global address B
	- DST RELOAD = NO 
	- CNT RELOAD = from global counter rg A
	- activate RSYNC/WSYNC events in case they have been missed

   PCRG @ DMA-1:   00 01 1 0 0 1 00101 00000 0 0 00 01 00 01 00 00	
*/
	*C6X_DMA1_PCRG = 0x19280110;	
	*C6X_DMA1_SCRG = 0;
			// configure first DAC IRT DMA address as 0x100 data frames ahead (i.e. behind)
			// of ADC DMA IRT address in order to ensure that DAC data are already
			// consistent and acquired ADC data and uploaded DAC data are from 
			// different data frames. Otherwise, DAC IRT, running asynchronously 
			// re ADC IRT, can upload data to DAC FIFO, which are not yet valid, i.e
			// from the previous FIFO data frame, which was many periods before.
			// Inter IRT synchronization for ADC and DAC IRTs is the important issue
			// for ADC/DAC SYNC-FIFO modes.
	*C6X_DMA1_SARG = (_DATA_ACQ_BADDR+0x100*4*2);	
	*C6X_DMA1_DARG = TPDAS_A1M_DAC_FIFO_DATA_RG_ADDR;
	*C6X_DMA1_TCRG = ((_DAQ_LEN-0x100)*4);

		// global ADC/DAC DMA registers for auto-init reload:
	*C6X_DMA_GCRARG= (_DAQ_LEN*4);			// counter reload (DMA-0/1)
	*C6X_DMA_GABRG = _DATA_ACQ_BADDR;		// source address DMA-0/1

	// clear false DMA-0 RSYNC event for ADC IRT data transfer
	*C6X_DMA0_SCRG=0xA000;		

	// start DMA0/1 in auto-init mode
	*C6X_DMA0_PCRG|=0x3; 
	*C6X_DMA1_PCRG|=0x3; 
#endif


	// Now enable ADC/DAC IRT by means of activating HIRQ0/1 sources.
	TPDAS_A1M_SET_HIRQ0_SEL_ADC_IRT;
	TPDAS_A1M_SET_HIRQ1_SEL_DAC_IRT;

	// DAC IRT controller has been initialized. Wait until DAC FIFO will be 
	// filled in.
	while(TPDAS_A1M_DAC_FIFO_PAF_IS_OFF)
    { 
		if(		TPDAS_A1M_DAC_IRT_ERR_IS_ON
			||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
			||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		  )
          	goto error;
	}    

	// check for DAC FIFO flags again when FIFO must be full
	if(		TPDAS_A1M_DAC_FIFO_FF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAF_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_PAE_IS_ON
		||	TPDAS_A1M_DAC_FIFO_EF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		||	TPDAS_A1M_DAC_IRT_ERR_IS_ON
	  )
		goto error;


	// Enable ADC/DAC SYNC DAQ
	TPDAS_A1M_SET_ADDA_SYNC_DAQ_RUN_ON;

	// Loop infinitely while cyclic ADC->DAC data are transferred via IRT & DMA.
	// Abort on any ADC/DAC IRT error occurs.
	i=0;
	while(1)
    { 
		// Perfrom read of ADC/DAC IRT error with relatively large period in order to
		// ensure that error status reads do not provide too much traffic to PIOX data bus
		// and DMA IRT has enough time room to complete in each sampling period.
		// Terminate acquisition if ADC/DAC IRT error occurs.
		if(++i ==100)
		{
			if(		TPDAS_A1M_GET_ADDA_IRT_ERR_ALL
				||	TPDAS_A1M_GET_ADC_FIFO_ERR_ALL
				||	TPDAS_A1M_GET_DAC_FIFO_ERR_ALL
				||	TPDAS_A1M_DAC_SYNC_DAQ_RUN_IS_OFF
				||	TPDAS_A1M_DAC_SYNC_DAQ_END_IS_ON
				||	TPDAS_A1M_ADC_SYNC_DAQ_RUN_IS_OFF
				||	TPDAS_A1M_ADC_SYNC_DAQ_END_IS_ON
			  )
				 goto error;
			i=0;
		}
	}

error:          	   
	// trap here on IRT errors
	// stop DMA0/1, loop infinitely
	*C6X_DMA0_PCRG=0; 
	*C6X_DMA1_PCRG=0; 
	TPDAS_A1M_RESET_ADDA_DAQ;
	while(1);	
}


