/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  DAC-0..3 test in DAC SYNC-FIFO_PTM mode using DAC FIFO software data upload.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs test for DAC-0..3 in DAC SYNC-FIFO_PTM DAQ mode with DAC data upload 
	via DSP software:
	  - PFG 0.5MHz is used as sampling frequency generator. 
	  - DAC-0 data array is filled in with the SINE data. DAC-1 data array is filled in 
	    with the rising ramp data. DAC-2 data array is filled in with the falling
	    ramp data. DAC-3 data array is filled in with the square pulse data.
	  - DAC SYNC-FIFO_PTM continuous DAQ mode is used without start-up synchronization
	  - DAC FIFO is uploaded continuously via DSP software using DAC FIFO PAF/PAE
		flags. 


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file



	// set PFG to 0.5MHz = (25*2*(V=64)/((R=50)*(S=2)*64*(X=1))
#define PFG_V_DATA		64
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV2
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1

#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


	// ADC/DAC DAQ data length
#define	_DAC_ARRAY_LEN		0x800

		// DAC data array pointers
		// T6x/TP6x/E6x: DSP on-chip DATA RAM
		// TP64xx/E64xx: external SBSRAM
#define	_DAC_DATA_PTR		(0x80008000)	


	// DAC FIFO PAF/PAE offsets
#define	_dac_pae_offset		(_DAC_ARRAY_LEN/2)
#define	_dac_paf_offset		_DAC_ARRAY_LEN


main()
{ int i;
  int short *mptr, *mptr_end, *mptr_start;
  int dac_upload_flag;


	C6X_GINT_DISABLE;
	init_cpu();

	// reset DAC DAQ, FIFO, IRT and DAC errors
	TPDAS_A1M_RESET_DAC_DAQ;     		

	// configure DAC DAQ mode: SYNC-FIFO_PTM without start-up synchronization 
	TPDAS_A1M_SET_DAC_DAQ_MODE_SYNC_FIFO_PTM;
	TPDAS_A1M_SET_DAC_SYNC_DAQ_SSYNC_SEL_SFT;

	// configure PFG
	TPDAS_A1M_SET_FS_SEL_PFG;
	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	// enable desired DACs in DAC CNF register, which is required for DAC DAQ and IRT
	TPDAS_A1M_set_dac_cnf_rg(		TPDAS_A1M_DAC0_EN_ON 
								| 	TPDAS_A1M_DAC1_EN_ON 
								|	TPDAS_A1M_DAC2_EN_ON 
								| 	TPDAS_A1M_DAC3_EN_ON
							);

	// now fill-in DAC-0..3 data arrays for further continuous transfer to DACs
	// DAC arrays shall be organized as continuous set of 4xDAC data, since
	// one DSP DMA channel will extract data continuously for all DACs during 
	// each sampling period.
	mptr = (int short *) _DAC_DATA_PTR;

	for(i=0;i<_DAC_ARRAY_LEN;i++)
	{ 		
		//-- fill-in DAC-0 as SINE signal
		*mptr++ = (int short)((float)  (32000.0 * sin(6.28 *((float)(i%256))/256.0))) ;
		
		//-- fill-in DAC-1 as the rising RAMP signal
		*mptr++ = (int short)((float)  (32000.0 * ((float)((i%256)-128))/128.0)) ;

		//-- fill-in DAC-2 as the falling RAMP signal
		*mptr++ = (int short)((float)  (32000.0 * ((float)(128-(i%256)))/128.0)) ;

		//-- fill-in DAC-3 as the square pulse signal
		*mptr++ = (int short)(((i%256)<128) ? 32000:-32000);
	}

	// configure DAC FIFO PAF/PAE flags
	if(   TPDAS_A1M_set_dac_fifo_offset(_dac_pae_offset*4,_dac_paf_offset*4)!=0
	  )
            goto error;


	// again, reset DAC DAQ, FIFO, IRT and DAC errors to ensure no errors
	TPDAS_A1M_RESET_DAC_DAQ;     		

	// check for DAC FIFO flags
	if(		TPDAS_A1M_DAC_FIFO_FF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAE_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_EF_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
	  )
		goto error;

	// now fill-in DAC FIFO with DAC data frames in order DAQ can start with valid data
	mptr_start = (int short *) _DAC_DATA_PTR;
	mptr_end = & (mptr_start[_DAC_ARRAY_LEN*4]);
	mptr = mptr_start;

	for(i=0;i<(TPDAS_A1M_FIFO_LEN/4);i++)
	{
		// write data frame to DAC FIFO
		TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);
		TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);
		TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);
		TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);

		// check for source data array end
		if(mptr==mptr_end)
			mptr=mptr_start;		

		// check for DAC FIFO errors
		if(		TPDAS_A1M_DAC_FIFO_OVF_IS_ON
			||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		  )
			goto error;

		// verify DAC FIFO EF/FF/PAF/PAE flag procurement
	  	if( 	((i+1)<=_dac_pae_offset && TPDAS_A1M_DAC_FIFO_PAE_IS_OFF)
            ||  ((i+1)>_dac_pae_offset  && TPDAS_A1M_DAC_FIFO_PAE_IS_ON)
            ||  ((i+1)<_dac_paf_offset  && TPDAS_A1M_DAC_FIFO_PAF_IS_ON)
			||  ((i+1)>=_dac_paf_offset && TPDAS_A1M_DAC_FIFO_PAF_IS_OFF)
			||	(i!=((TPDAS_A1M_FIFO_LEN/4)-1) && TPDAS_A1M_DAC_FIFO_FF_IS_ON)
			||	TPDAS_A1M_DAC_FIFO_EF_IS_ON
	      ) 
			goto error;
	}
	
	// check for DAC FIFO flags again when FIFO must be full
	if(		TPDAS_A1M_DAC_FIFO_FF_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_PAF_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_PAE_IS_ON
		||	TPDAS_A1M_DAC_FIFO_EF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
	  )
		goto error;


	// enable DAC SYNC DAQ and check for errors
	TPDAS_A1M_SET_DAC_SYNC_DAQ_RUN_ON;
	if(		TPDAS_A1M_DAC_SYNC_DAQ_RUN_IS_OFF
		||	TPDAS_A1M_DAC_SYNC_DAQ_END_IS_ON
	  )
		goto error;


	// 	Loop here infinitely with DAC FIFO s/w upload.
	// 	This upload method is awaiting for the DAC FIFO PAE flag coming active, 
	//	and	afterthat uploads data to DAC FIFO until DAC PAF flag will come active.
	//	This method is the software simulation of the DAC IRT procedure with 
 	//	the PAE/PAF termination flags. This method saves DSP performance, since DSP can be activated only
	//	when PAE flag comes active.
	// 	Check for DAC FIFO errors. Abort on any error.

	// 	configure GPIO-0 as output and use it as DAC-FIFO upload indicator
	TPDAS_A1M_SET_GPIO0_DIR_OUTPUT;
	TPDAS_A1M_SET_GPIO0_DATA_OFF;

	dac_upload_flag = 0;

	while(1)
    { 
		// check DAC FIFO errors
		if(		TPDAS_A1M_DAC_FIFO_OVF_IS_ON
			||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		  )
			goto error;

		// Wait for DAC FIFO PAE flag comes active and set upload enable flag.
		// Upload enable flag will be reset as soon as DAC FIFO PAF flag comes active.
		if(TPDAS_A1M_DAC_FIFO_PAE_IS_ON) 
		{ 
			dac_upload_flag = 1;
			TPDAS_A1M_SET_GPIO0_DATA_ON;
		}

		// check for upload is enabled
		if(dac_upload_flag==1)
		{
			// fill-in DAC FIFO
			TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);
			TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);
			TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);
			TPDAS_A1M_SET_DAC_FIFO_DATA(*mptr++);

			// check for source data array end
			if(mptr==mptr_end)
				mptr=mptr_start;		

			// check for DAC FIFO PAF flag comes active in order to clear upload flag
			if(TPDAS_A1M_DAC_FIFO_PAF_IS_ON)
			{
				dac_upload_flag = 0;
				TPDAS_A1M_SET_GPIO0_DATA_OFF;
			}
		}
	}

error:          	   
	// trap here on any error
	TPDAS_A1M_SET_DAC_SYNC_DAQ_ABORT_ON;
	while(1);	
}


