/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  DAC-0..3 test in DAC SYNC-FIFO_PTM mode using DAC FIFO IRT upload.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs test for DAC-0..3 in DAC SYNC-FIFO_PTM DAQ mode with DAC data upload 
	via DAC IRT controller and DSP DMA:
	  - PFG 1MHz is used as sampling frequency generator. 
	  - DAC-0 data array is filled in with the SINE data. DAC-1 data array is filled in 
	    with the rising ramp data. DAC-2 data array is filled in with the falling
	    ramp data. DAC-3 data array is filled in with the square pulse data.
	  - DAC SYNC-FIFO_PTM continuous DAQ mode is used without start-up synchronization
	  - DAC FIFO is uploaded continuously via DAC IRT controller and DSP on-chip
		DMA-0 controller. 


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file



	// set PFG to 1MHz = (25*2*(V=128)/((R=50)*(S=2)*64*(X=1))
#define PFG_V_DATA		128
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV2
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1

#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


	// ADC/DAC DAQ data length
#define	_DAC_ARRAY_LEN		0x800

		// DAC data array pointers
		// T6x/TP6x/E6x: DSP on-chip DATA RAM
		// TP64xx/E64xx: external SBSRAM
#define	_DAC_DATA_PTR		(0x80008000)	


	// DAC FIFO PAF/PAE offsets
#define	_dac_pae_offset		0x1000
#define	_dac_paf_offset		0x8000


main()
{ int i;
  int short *mptr;


	C6X_GINT_DISABLE;
	init_cpu();

	// reset DAC DAQ, FIFO, IRT and DAC errors
	TPDAS_A1M_RESET_DAC_DAQ;     		

	// configure DAC DAQ mode: SYNC-FIFO_PTM without start-up synchronization 
	TPDAS_A1M_SET_DAC_DAQ_MODE_SYNC_FIFO_PTM;
	TPDAS_A1M_SET_DAC_SYNC_DAQ_SSYNC_SEL_SFT;

	// set DAC IRT start-up and termination flags to PAE/PAF flag
	TPDAS_A1M_SET_DAC_SYNC_FIFO_DAQ_TF_SEL_PAE;
	TPDAS_A1M_SET_DAC_SYNC_FIFO_IRT_TF_SEL_PAF;

	// configure PFG
	TPDAS_A1M_SET_FS_SEL_PFG;
	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	// enable desired DACs in DAC CNF register, which is required for DAC DAQ and IRT
	TPDAS_A1M_set_dac_cnf_rg(		TPDAS_A1M_DAC0_EN_ON 
								| 	TPDAS_A1M_DAC1_EN_ON 
								|	TPDAS_A1M_DAC2_EN_ON 
								| 	TPDAS_A1M_DAC3_EN_ON
							);

	// now fill-in DAC-0..3 data arrays for further continuous transfer to DACs
	// DAC arrays shall be organized as continuous set of 4xDAC data, since
	// one DSP DMA channel will extract data continuously for all DACs during 
	// each sampling period.
	mptr = (int short *) _DAC_DATA_PTR;

	for(i=0;i<_DAC_ARRAY_LEN;i++)
	{ 		
		//-- fill-in DAC-0 as SINE signal
		*mptr++ = (int short)((float)  (32000.0 * sin(6.28 *((float)(i%256))/256.0))) ;
		
		//-- fill-in DAC-1 as the rising RAMP signal
		*mptr++ = (int short)((float)  (32000.0 * ((float)((i%256)-128))/128.0)) ;

		//-- fill-in DAC-2 as the falling RAMP signal
		*mptr++ = (int short)((float)  (32000.0 * ((float)(128-(i%256)))/128.0)) ;

		//-- fill-in DAC-3 as the square pulse signal
		*mptr++ = (int short)(((i%256)<128) ? 32000:-32000);
	}

	// configure DAC FIFO PAF/PAE flags
	if(   TPDAS_A1M_set_dac_fifo_offset(_dac_pae_offset*4,_dac_paf_offset*4)!=0
	  )
            goto error;


	// again, reset DAC DAQ, FIFO, IRT and DAC errors to ensure no errors
	TPDAS_A1M_RESET_DAC_DAQ;     		

	// check for DAC FIFO flags and errors
	if(		TPDAS_A1M_DAC_FIFO_FF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAE_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_EF_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		||	TPDAS_A1M_DAC_IRT_ERR_IS_ON
	  )
		goto error;

	// Use DAC IRT via HIRQ-0 PIOX IRQ line and DMA ch#0 for continuous 
	// DAC data upload. 
	// Initially, DAC IRT will immediately start after IRT and DMA controllers
	// have been configured. This will fill-in DAC FIFO initially. After DAC DAQ
	// will be enabled, IRT will be restarted as soon as DAC DAQ TF will come active
	// and will proceed until DAC IRT TF flag will be reached. This will repeat
	// continuosly.
	//
	// important note:
	// ---------------
	// 	IRT interrupt at PIOX HIRQ must be enabled as the last step of IRT 
	//  initialization procedure after DMA controller will be configured and 
	//	enabled. 

		// Deselect any sources at PIOX IRQ-0 line.
		// This is required in order to disable IRT prior DMA will be enabled.
	TPDAS_A1M_SET_HIRQ0_OFF;	
	TPDAS_A1M_SET_HIRQ0_EN_ON;

		// Set HIRQ IRT polarity upon the TORNADO board type
		// note!!!: 
		//	T6x/E62/E67 require negative DAC IRT signal,
		//  whereas TP6x/E6202/E6203/E64xx require positive DAC IRT.
		//  TP6x and E6202/E6203/E64xx also require setting of MIRQ0 selector.
#if(_TP6X_SELECTED || _TP64XX_SELECTED)
		// configure DSP EXT_INT4=PIOX_IRQ-0 and EXT_INT5=PIOX_IRQ-1
	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	
	TPDAS_A1M_SET_HIRQ0_POL_POS;
#else
#if(_E6X_SELECTED  || _E64XX_SELECTED)
	if( TE6202_TE6203_BOARD || TE64XX_BOARD)
	{
		// this is either of E6202/E6203/E64xx boards, which require
		// positive IRT and programmable setting of MIRQ0 selector
		TE6X_set_ext_int_sel(0,TE6X_EXT_INT_SEL_PXIRQ0);	
		TPDAS_A1M_SET_HIRQ0_POL_POS;
	}
	else
	{
		// this is E62/E67 with direct routing of XIRQ0~ to EXT_INT4,
		// which requires negative IRT and no MIRQ0 selectors settings
		TPDAS_A1M_SET_HIRQ0_POL_NEG;
	}
#else
		// this is for T6x (always negative IRT, no MIRQ0 selector)
	TPDAS_A1M_SET_HIRQ0_POL_NEG;
#endif
#endif


		// configure C6x on-chip IRQ4 to source from EXT_INT4
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	


// configure DMA controller for C6201/C6202/C6203/C6701 based boards
// (note!!! C64xx EDMA controller requires different initialization)
#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
/*  configure DMA-0 to transfer DAC data via EXT_INT4 DSP interrupt (PIOX IRQ-0)
    in auto-init mode in oder to exclude successive reloads of DMA configuration during
    continuous DAC PTM mode:
	- SRC DIR = +1, DST DIR = no
	- SRC SYNC = no, DST SYNC = EXT_INT4
	- esize = 16-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = from global address B/C
	- DST RELOAD = NO 
	- CNT RELOAD = from global counter rg A
	- activate RSYNC/WSYNC events in case they have been missed

   PCRG @ DMA-0:   00 01 1 0 0 1 00100 00000 0 0 00 01 00 01 00 00	
*/

	*C6X_DMA0_PCRG = 0x19200110;	
	*C6X_DMA0_SCRG = 0x30000;
	*C6X_DMA0_SARG = _DAC_DATA_PTR;
	*C6X_DMA0_DARG = TPDAS_A1M_DAC_FIFO_DATA_RG_ADDR;
	*C6X_DMA0_TCRG = (_DAC_ARRAY_LEN*4);

		// global DMA register for auto-init reload:
	*C6X_DMA_GCRARG= (_DAC_ARRAY_LEN*4);	// counter reload (DMA-0)
	*C6X_DMA_GABRG = _DAC_DATA_PTR;			// source address DMA-0

		// start DMA0 in auto-init mode
	*C6X_DMA0_PCRG|=0x3; 
#endif

	// Enable DAC IRT by means of activating HIRQ0 source.
	TPDAS_A1M_SET_HIRQ0_SEL_DAC_IRT;

	// DAC IRT controller has been initialized. Wait until FIFO will be 
	// filled in.
	while(TPDAS_A1M_DAC_FIFO_PAF_IS_OFF)
    { 
		if(		TPDAS_A1M_DAC_IRT_ERR_IS_ON
			||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
			||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		  )
          	goto error;
	}    

	// check for DAC FIFO flags again when FIFO must be full
	if(		TPDAS_A1M_DAC_FIFO_FF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_PAF_IS_OFF
		||	TPDAS_A1M_DAC_FIFO_PAE_IS_ON
		||	TPDAS_A1M_DAC_FIFO_EF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_OVF_IS_ON
		||	TPDAS_A1M_DAC_FIFO_UNF_IS_ON
		||	TPDAS_A1M_DAC_IRT_ERR_IS_ON
	  )
		goto error;

	// now enable DAC DAQ
	TPDAS_A1M_SET_DAC_SYNC_DAQ_RUN_ON;

	// loop here infinitely with DMA auto-init
	// check for DAC FIFO and IRT errors. Abort on any error.
	i=0;
	while(1)
    { 
		// Perfrom read of DAC FIFO/IRT errors with relatively large period in order to
		// ensure that error status reads do not provide too much traffic to PIOX data bus
		// and DMA IRT has enough time room to complete.
		// Terminate acquisition if DAC IRT error occurs.
		if(++i ==100)
		{
			if(		TPDAS_A1M_GET_DAC_FIFO_IRT_ERR_ALL
				||	TPDAS_A1M_DAC_SYNC_DAQ_RUN_IS_OFF
				||	TPDAS_A1M_DAC_SYNC_DAQ_END_IS_ON
			  )
				goto error;
			i=0;
		}
	}

error:          	   
	// trap here on any error
	TPDAS_A1M_SET_DAC_SYNC_DAQ_ABORT_ON;
	while(1);	
}


