/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  For host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  ADC ch-0..15 test in ADC SYNC-noFIFO mode with PFG sampling and 
  IRT data download.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs ADC test in ADC SYNC-noFIFO	mode with ADC data download 
	via ADC IRT:
	  - on-board 200kHz PFG is used as sampling frequency generator
	  - ADC SYNC-noFIFO DAQ mode is used
	  - A/D conversion is started by DSP s/w without external start-up 
		synchronization
	  - ADC-0..3 are used for A/D conversion
	  - ADC IMUX is configured to scan all ADC ch-0..15 input channels via
		four ADC scan cycles within each sampling frequency cycle
	  - A/D conversion is performed during 0x400 sampling frequency cycles and	
		terminates afterthat
	  - real-time ADC data is downloaded to DSP environment memory using ADC IRT 
		and DSP	on-chip DMA-0 controller, 
	  - after ADC DAQ terminates, ADC data can be viewed via Code Composer signal 
		viewer in time domain as 16-bit integers at the following addresses:
		- ADC ch-0 data are available at 0x1000 address (0x80001000 for C64xx DSP)
		- ADC ch-1 data are available at 0x1800 address (0x80001800 for C64xx DSP)
		- ADC ch-2 data are available at 0x2000 address (0x80002000 for C64xx DSP)
		- ADC ch-3 data are available at 0x2800 address (0x80002800 for C64xx DSP)
		- ADC ch-4 data are available at 0x3000 address (0x80003000 for C64xx DSP)
		- ADC ch-5 data are available at 0x3800 address (0x80003800 for C64xx DSP)
		- ADC ch-6 data are available at 0x4000 address (0x80004000 for C64xx DSP)
		- ADC ch-7 data are available at 0x4800 address (0x80004800 for C64xx DSP)
		- ADC ch-8 data are available at 0x5000 address (0x80005000 for C64xx DSP)
		- ADC ch-9 data are available at 0x5800 address (0x80005800 for C64xx DSP)
		- ADC ch-10 data are available at 0x6000 address (0x80006000 for C64xx DSP)
		- ADC ch-11 data are available at 0x6800 address (0x80006800 for C64xx DSP)
		- ADC ch-12 data are available at 0x7000 address (0x80007000 for C64xx DSP)
		- ADC ch-13 data are available at 0x7800 address (0x80007800 for C64xx DSP)
		- ADC ch-14 data are available at 0x8000 address (0x80008000 for C64xx DSP)
		- ADC ch-15 data are available at 0x8800 address (0x80008800 for C64xx DSP)


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file



	// set PFG to 0.2MHz = (25*2*(V=64)/((R=50)*(S=5)*64*(X=1))
#define PFG_V_DATA		64
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV10
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1

#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


	// pointers for saving ADC channel data and data length
#define	_DAQ_LEN			0x400L

#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
#define _DATA_ACQ_BADDR			0x80001000		// acquired/packed data buffer for C620x/C670x DSP (DSP on-chip RAM)
#define _DATA_UNPACKED_BADDR	0x01000			// unpacked data buffer start for C620x/C670x DSP (external SBSRAM)
#else
#define _DATA_ACQ_BADDR			0x18000			// acquired/packed data buffer for C64xx DSP (DSP on-chip RAM)
#define _DATA_UNPACKED_BADDR	0x80008000		// unpacked data buffer start for C64xx DSP (external SBSRAM)
#endif

#define	_ADC_CH0_DATA			(_DATA_UNPACKED_BADDR + 0*(_DAQ_LEN*2))
#define	_ADC_CH1_DATA			(_DATA_UNPACKED_BADDR + 1*(_DAQ_LEN*2))
#define	_ADC_CH2_DATA			(_DATA_UNPACKED_BADDR + 2*(_DAQ_LEN*2))
#define	_ADC_CH3_DATA			(_DATA_UNPACKED_BADDR + 3*(_DAQ_LEN*2))
#define	_ADC_CH4_DATA			(_DATA_UNPACKED_BADDR + 4*(_DAQ_LEN*2))
#define	_ADC_CH5_DATA			(_DATA_UNPACKED_BADDR + 5*(_DAQ_LEN*2))
#define	_ADC_CH6_DATA			(_DATA_UNPACKED_BADDR + 6*(_DAQ_LEN*2))
#define	_ADC_CH7_DATA			(_DATA_UNPACKED_BADDR + 7*(_DAQ_LEN*2))
#define	_ADC_CH8_DATA			(_DATA_UNPACKED_BADDR + 8*(_DAQ_LEN*2))
#define	_ADC_CH9_DATA			(_DATA_UNPACKED_BADDR + 9*(_DAQ_LEN*2))
#define	_ADC_CH10_DATA			(_DATA_UNPACKED_BADDR + 10*(_DAQ_LEN*2))
#define	_ADC_CH11_DATA			(_DATA_UNPACKED_BADDR + 11*(_DAQ_LEN*2))
#define	_ADC_CH12_DATA			(_DATA_UNPACKED_BADDR + 12*(_DAQ_LEN*2))
#define	_ADC_CH13_DATA			(_DATA_UNPACKED_BADDR + 13*(_DAQ_LEN*2))
#define	_ADC_CH14_DATA			(_DATA_UNPACKED_BADDR + 14*(_DAQ_LEN*2))
#define	_ADC_CH15_DATA			(_DATA_UNPACKED_BADDR + 15*(_DAQ_LEN*2))



main()
{ int i;
  int short	*mptr_src;
  int short	*mptr0,*mptr1,*mptr2,*mptr3,
			*mptr4,*mptr5,*mptr6,*mptr7,
			*mptr8,*mptr9,*mptr10,*mptr11,
			*mptr12,*mptr13,*mptr14,*mptr15;


	C6X_GINT_DISABLE;
	init_cpu();

	// reset ADC DAQ, FIFO and IRT
	TPDAS_A1M_RESET_ADDA_DAQ;     		

	// configure ADC DAQ mode: SYNC-noFIFO with s/w start-up synchronization,
	// ADC high-speed mode, extended inter-scan_cycle timing is disabled
	TPDAS_A1M_SET_ADC_DAQ_MODE_SYNC_NO_FIFO;
	TPDAS_A1M_SET_ADC_SYNC_DAQ_SSYNC_SEL_SFT;
	TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_ON;
	TPDAS_A1M_SET_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_OFF;

	// configure PFG
	TPDAS_A1M_SET_FS_SEL_PFG;
	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	// enable desired ADCs in ADC CNF register, which is required for ADC DAQ and IRT
	TPDAS_A1M_set_adc_cnf_rg(		TPDAS_A1M_ADC0_EN_ON 
								| 	TPDAS_A1M_ADC1_EN_ON 
								|	TPDAS_A1M_ADC2_EN_ON 
								|	TPDAS_A1M_ADC3_EN_ON
							);

	// configure ADC IMUX for each scan cycle
	// set EOS flag for the 3rd cycle only (is set always via hardware)
	TPDAS_A1M_SET_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12;
	TPDAS_A1M_SET_ADC_SCCL0_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_SCCL0_EOS_EN_OFF;

	TPDAS_A1M_SET_ADC_SCCL1_IMUX_CH_SEL_1_5_9_13;
	TPDAS_A1M_SET_ADC_SCCL1_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_SCCL1_EOS_EN_OFF;

	TPDAS_A1M_SET_ADC_SCCL2_IMUX_CH_SEL_2_6_10_14;
	TPDAS_A1M_SET_ADC_SCCL2_IMUX_DIFF_EN_OFF;
	TPDAS_A1M_SET_ADC_SCCL2_EOS_EN_OFF;

	TPDAS_A1M_SET_ADC_SCCL3_IMUX_CH_SEL_3_7_11_15;
	TPDAS_A1M_SET_ADC_SCCL3_IMUX_DIFF_EN_OFF;


	// Use ADC IRT via HIRQ-0 PIOX IRQ line and DMA ch#0 for ADC data upload. 
	// Note, that for TP6x boards, _IRT_POS output must be selected.
	//
	// important note:
	// ---------------
	// 	IRT interrupt at PIOX HIRQ must be enabled as the last step of IRT 
	//  initialization procedure after DMA controller will be configured and 
	//	enabled. 

		// Deselect any sources at PIOX IRQ-0 line.
		// This is required in order to disable IRT prior DMA will be enabled.
	TPDAS_A1M_SET_HIRQ0_OFF;	
	TPDAS_A1M_SET_HIRQ0_EN_ON;

		// Set HIRQ IRT polarity upon the TORNADO board type
		// note!!!: 
		//	T6x/E62/E67 require negative ADC IRT signal,
		//  whereas TP6x/E6202/E6203/E64xx require positive ADC IRT.
		//  TP6x and E6202/E6203/E64xx also require setting of MIRQ0 selector.
#if(_TP6X_SELECTED || _TP64XX_SELECTED)
		// configure DSP EXT_INT4=PIOX_IRQ-0
	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	
	TPDAS_A1M_SET_HIRQ0_POL_POS;
#else
#if(_E6X_SELECTED  || _E64XX_SELECTED)
	if( TE6202_TE6203_BOARD || TE64XX_BOARD)
	{
		// this is either of E6202/E6203/E64xx boards, which require
		// positive IRT and programmable setting of MIRQ0 selector
		TE6X_set_ext_int_sel(0,TE6X_EXT_INT_SEL_PXIRQ0);	
		TPDAS_A1M_SET_HIRQ0_POL_POS;
	}
	else
	{
		// this is E62/E67 with direct routing of XIRQ0~ to EXT_INT4,
		// which requires negative IRT and no MIRQ0 selector setting
		TPDAS_A1M_SET_HIRQ0_POL_NEG;
	}
#else
		// this is for T6x (always negative IRT, no MIRQ0 selector)
	TPDAS_A1M_SET_HIRQ0_POL_NEG;
#endif
#endif

		// configure C6x on-chip IRQ4 to source from EXT_INT4
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	

// configure DMA controller for C6201/C6202/C6203/C6701 based boards
// (note!!! C64xx EDMA controller requires different initialization)
#if(_TP6X_SELECTED || _T6X_SELECTED || _E6X_SELECTED)
/*  configure DMA-0 to transfer ADC data via EXT_INT4 DSP interrupt (PIOX IRQ-0)
    without auto-init mode :
	- SRC DIR = no, DST DIR = +1
	- SRC SYNC = EXT_INT4, DST SYNC = no
	- esize = 16-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = no
	- DST RELOAD = NO 
	- CNT RELOAD = no
	- activate RSYNC/WSYNC events in case they have been missed

   PCRG @ DMA-0:   00 00 1 1 0 1 00000 00100 0 0 00 01 01 00 00 00	
*/

	*C6X_DMA0_PCRG = 0x0b010140;	
	*C6X_DMA0_SCRG = 0;
	*C6X_DMA0_SARG = TPDAS_A1M_ADC_IRT_DATA_RG_ADDR;
	*C6X_DMA0_DARG = _DATA_ACQ_BADDR;
	*C6X_DMA0_TCRG = (_DAQ_LEN*16);

	// clear false DMA-0 RSYNC event for ADC IRT data transfer
	*C6X_DMA0_SCRG=0xA000;		

	// start DMA0 without auto-init mode
	*C6X_DMA0_PCRG|=0x1; 
#endif

	// Now enable ADC IRT by means of activating HIRQ0 source.
	TPDAS_A1M_SET_HIRQ0_SEL_ADC_IRT;

	// Finally, enable ADC SYNC DAQ
	TPDAS_A1M_SET_ADC_SYNC_DAQ_RUN_ON;

	// loop here until DMA stops or ADC IRT error occures
	// check for ADC IRT errors. Abort on any error.
	i=0;
	while(1)
    { 
		// check for DMA stops as fast as possible in order to ensure that
		// DMA stop event will be detected earlier than IRT error
		if((*C6X_DMA0_TCRG)==0) break;

		// Perfrom read of ADC IRT error with relatively large period in order to
		// ensure that error status reads do not provide too much traffic to PIOX data bus
		// and DMA IRT has enough time room to complete in each sampling period.
		// Terminate acquisition if ADC IRT error occurs.
		if(++i ==100)
		{
			if(TPDAS_A1M_ADC_IRT_ERR_IS_ON || TPDAS_A1M_ADC_SYNC_DAQ_RUN_IS_OFF)
				 goto error;
			i=0;
		}
	}

	// get here after ADC DAQ and IRT transfer completes successfully.
	// Abort ADC SYNC-noFIFO DAQ and re-construct ADC channels data arrays in order
	// they can be viewed via Code COmposer viewer.
	// In order to view ADC data via Code Composer viewer, data must be unpacked from the
	// four quad-ADC data frames to sixteen linear ADC arrays

		// terminate ADC DAQ
	TPDAS_A1M_SET_ADC_SYNC_DAQ_ABORT_ON;

		// unpack ADC channels data
	mptr0 = (int short *) _ADC_CH0_DATA;
	mptr1 = (int short *) _ADC_CH1_DATA;
	mptr2 = (int short *) _ADC_CH2_DATA;
	mptr3 = (int short *) _ADC_CH3_DATA;
	mptr4 = (int short *) _ADC_CH4_DATA;
	mptr5 = (int short *) _ADC_CH5_DATA;
	mptr6 = (int short *) _ADC_CH6_DATA;
	mptr7 = (int short *) _ADC_CH7_DATA;
	mptr8 = (int short *) _ADC_CH8_DATA;
	mptr9 = (int short *) _ADC_CH9_DATA;
	mptr10 = (int short *) _ADC_CH10_DATA;
	mptr11 = (int short *) _ADC_CH11_DATA;
	mptr12 = (int short *) _ADC_CH12_DATA;
	mptr13 = (int short *) _ADC_CH13_DATA;
	mptr14 = (int short *) _ADC_CH14_DATA;
	mptr15 = (int short *) _ADC_CH15_DATA;

	mptr_src = (int short *) _DATA_ACQ_BADDR;
	
	for(i=0;i<_DAQ_LEN;i++)
	{
		// unpack data from quad-ADC data frames
		// each data frame contains data from four ADCs
		*mptr0++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr4++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr8++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr12++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);

		*mptr1++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr5++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr9++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr13++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);

		*mptr2++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr6++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr10++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr14++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);

		*mptr3++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr7++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr11++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
		*mptr15++ = TPDAS_A1M_CONV_ADC_TO_INT16(*mptr_src++);
	}

end_OK:
	// loop infinitely here on successful execution
	while(1);


error:          	   
	// trap here on any error
	// stop DMA0 and terminate ADC DAQ
	*C6X_DMA0_PCRG=0; 
	TPDAS_A1M_SET_ADC_SYNC_DAQ_ABORT_ON;
	while(1);
	
}


