/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  Sampling Frequency Selector Test for TORNADO-6x/P6x/E6x.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo dynamically switches between different sources for
	DAQ sampling frequency:
	  - on-board PFG (1MHz)
	  - external sampling frequency input (XFS)
	  - PIOX Timer-0 (100kHz@C6201)
	  - PIOX Timer-1 (50kHz@C6201)
	Selection is performed dynamically with approximately 1sec interval.
	Output sampling frequency can be observed at the <FS_OUT> pin of 
	output connector of the XIO/SYNC cable.


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
	  	- TORNADO on-board TM0/TM1-OUT ENBALE jumpers/switches shall be installed/ON
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file


	// set PFG to 1MHz = (25MHz*2*(V=128)/((R=50)*(S=2)*64*(X=1))
#define PFG_V_DATA		128
#define PFG_R_DATA      50
#define PFG_S_DATA		TPDAS_A1M_PFG_S_DIV2
#define PFG_X_DATA		TPDAS_A1M_PFG_X_DIV1



// generates pause approx 1sec	
pause()
{ unsigned long i;

	for(i=0;i<10000000L;i++);
}



main()
{ 
   	init_cpu();

	//-- configure C6x DSP on-chip timers
   	*C6X_TM0_CONTROL = 0;
   	*C6X_TM0_PERIOD  = 250;
   	*C6X_TM0_CONTROL = 0x03c1;

   	*C6X_TM1_CONTROL = 0;
   	*C6X_TM1_PERIOD  = 500;
   	*C6X_TM1_CONTROL = 0x03c1;

	// program PFG
   	TPDAS_A1M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);


 	while(1)
 	{
		TPDAS_A1M_SET_FS_SEL_PFG;
		pause();	

		TPDAS_A1M_SET_FS_SEL_XFS;
		pause();	

		TPDAS_A1M_SET_FS_SEL_PXTM0;
		pause();	

		TPDAS_A1M_SET_FS_SEL_PXTM1;
		pause();	
 	}


}


