/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  Host I/F registers test for TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs host I/F registers test via host PIOX-16 I/F of
	TORNADO mainboard.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file



main()
{ int i;
  unsigned d;

   init_cpu();

 	while(1)
 	{
		// first test that all D0..D7 databits are connected perfectly
		// (use two registers with different data in order to exclude PIOX latched data read)
   		for(i=0,d=1;i<8;i++)
     	{
       		TPDAS_A1M_set_pfg_cntr1_rg(d);
       		TPDAS_A1M_set_pfg_cntr2_rg(d>>1);

       		if(   	TPDAS_A1M_get_pfg_cntr1_rg()!=d
          		|| 	TPDAS_A1M_get_pfg_cntr2_rg()!=(d>>1)
	 		  ) 
				goto error;
       		d <<= 1;
     	}

		// now test address lines 
		// (write/verify data to different registers fm each different address area)
       	TPDAS_A1M_set_daq_cntr1_rg(0x01);
       	TPDAS_A1M_set_daq_cntr3_rg(0x02);
       	TPDAS_A1M_set_daq_sync_rg(0x83);
       	TPDAS_A1M_set_adc_imux_cnf1_rg(0x21);
       	TPDAS_A1M_set_adc_imux_cnf2_rg(0xA3);
       	TPDAS_A1M_set_adc_cnf_rg(0x5);
       	TPDAS_A1M_set_dac_cnf_rg(0xA);
       	TPDAS_A1M_set_pfg_cntr1_rg(0xaa);
       	TPDAS_A1M_set_pfg_cntr2_rg(0x55);
       	TPDAS_A1M_set_gpio_dir_rg(0x0a);
       	TPDAS_A1M_set_hirq0_sel_rg(0x4c);
       	TPDAS_A1M_set_hirq1_sel_rg(0x9d);
       	TPDAS_A1M_set_hirq2_sel_rg(0x8e);
       	TPDAS_A1M_set_hirq3_sel_rg(0x4f);
       	TPDAS_A1M_set_xim_err_rg(0x2b);

		// insert FIFO reset in order to ensure that
		// write to a different area has been performed
		TPDAS_A1M_RESET_ADC_DAQ;
		TPDAS_A1M_RESET_DAC_DAQ;
		TPDAS_A1M_RESET_ADDA_DAQ;


       	if(   	TPDAS_A1M_get_daq_cntr1_rg()!=0x01
          	|| 	TPDAS_A1M_get_daq_cntr3_rg()!=0x02
          	|| 	TPDAS_A1M_get_daq_sync_rg()!=0x83
          	|| 	TPDAS_A1M_get_adc_imux_cnf1_rg()!=0x21
			||	TPDAS_A1M_get_adc_imux_cnf2_rg()!=0xA3
			||	(TPDAS_A1M_get_adc_cnf_rg() & (TPDAS_A1M_ADC0_EN_BITMASK | TPDAS_A1M_ADC1_EN_BITMASK | TPDAS_A1M_ADC2_EN_BITMASK | TPDAS_A1M_ADC3_EN_BITMASK))!=0x5
			||	(TPDAS_A1M_get_dac_cnf_rg() & (TPDAS_A1M_DAC0_EN_BITMASK | TPDAS_A1M_DAC1_EN_BITMASK | TPDAS_A1M_DAC2_EN_BITMASK | TPDAS_A1M_DAC3_EN_BITMASK))!=0xA
          	|| 	TPDAS_A1M_get_pfg_cntr1_rg()!=0xaa
          	|| 	TPDAS_A1M_get_pfg_cntr2_rg()!=0x55
          	|| 	TPDAS_A1M_get_gpio_dir_rg()!=0x0A
          	|| 	TPDAS_A1M_get_hirq0_sel_rg()!=0x4c
          	|| 	TPDAS_A1M_get_hirq1_sel_rg()!=0x9d
          	|| 	TPDAS_A1M_get_hirq2_sel_rg()!=0x8e
          	|| 	TPDAS_A1M_get_hirq3_sel_rg()!=0x4f
	  		|| 	TPDAS_A1M_get_xim_err_rg()!=0x2b
	 	  ) 
			goto error;


		// clear all regs
       	TPDAS_A1M_set_daq_cntr1_rg(0);
       	TPDAS_A1M_set_daq_cntr3_rg(0);
       	TPDAS_A1M_set_daq_sync_rg(0);
       	TPDAS_A1M_set_adc_imux_cnf1_rg(0);
       	TPDAS_A1M_set_adc_imux_cnf2_rg(0);
       	TPDAS_A1M_set_adc_cnf_rg(0);
       	TPDAS_A1M_set_dac_cnf_rg(0);
       	TPDAS_A1M_set_pfg_cntr1_rg(0);
       	TPDAS_A1M_set_pfg_cntr2_rg(0);
       	TPDAS_A1M_set_gpio_dir_rg(0);
       	TPDAS_A1M_set_hirq0_sel_rg(0);
       	TPDAS_A1M_set_hirq1_sel_rg(0);
       	TPDAS_A1M_set_hirq2_sel_rg(0);
       	TPDAS_A1M_set_hirq3_sel_rg(0);
       	TPDAS_A1M_set_xim_err_rg(0);

 	}

error:
	// clear all regs and idle forever
	TPDAS_A1M_set_daq_cntr1_rg(0);
    TPDAS_A1M_set_daq_cntr3_rg(0);
    TPDAS_A1M_set_daq_sync_rg(0);
   	TPDAS_A1M_set_adc_imux_cnf1_rg(0);
   	TPDAS_A1M_set_adc_imux_cnf2_rg(0);
   	TPDAS_A1M_set_adc_cnf_rg(0);
   	TPDAS_A1M_set_dac_cnf_rg(0);
    TPDAS_A1M_set_pfg_cntr1_rg(0);
    TPDAS_A1M_set_pfg_cntr2_rg(0);
    TPDAS_A1M_set_gpio_dir_rg(0);
    TPDAS_A1M_set_hirq0_sel_rg(0);
    TPDAS_A1M_set_hirq1_sel_rg(0);
    TPDAS_A1M_set_hirq2_sel_rg(0);
    TPDAS_A1M_set_hirq3_sel_rg(0);
    TPDAS_A1M_set_xim_err_rg(0);
	while(1) ;
}


