/*************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  PFG test for TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs PFG control test by writing a 'running '1'' to 
	the bits of V/R/S fields of on-board PFG. The 'running '1'' can be
	be observed at the pins of the PFG chip in order to ensure that
	the PFG chip is perfecty controlled by host DSP.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload code to host 
	TORNADO board. TI CCS tools are required to recompile this demo.

  
   How to compile this demo:
   -------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular TORNADO platform, the corresponding run-time
	compiler definition must be set in TPDAS_A1M_C6XXX.CFG file prior compiling
	the demo. Also, the corresonding linker command file must be used to link
	demo for C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  TPDAS_A1M_C6XXX.CFG 
	file for more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/


#include	"math.h"
#include	"tpdas_a1m_c6xxx.cfg"	// include demo configuration file




main()
{ int i;
  unsigned v,r,s,x;

   init_cpu();

 	while(1)
 	{
		// first test the PFG V-field
   		r=0;
   		s=0;
		x=TPDAS_A1M_PFG_X_DIV1;
   		for(i=0,v=1;i<9;i++)
     	{
       		TPDAS_A1M_SET_PFG(v,r,s,x);

       		if(   	TPDAS_A1M_GET_PFG_V!=v
          		|| 	TPDAS_A1M_GET_PFG_R!=r
          		|| 	TPDAS_A1M_GET_PFG_S!=s
          		|| 	TPDAS_A1M_GET_PFG_X!=x
	 		  ) 
				goto error;

       		v <<= 1;
     	}

		// then test the PFG R-field
   		v=0;
   		s=0;
		x=TPDAS_A1M_PFG_X_DIV1;
   		for(i=0,r=1;i<7;i++)
     	{
       		TPDAS_A1M_SET_PFG(v,r,s,x);

       		if(   	TPDAS_A1M_GET_PFG_V!=v
          		|| 	TPDAS_A1M_GET_PFG_R!=r
          		|| 	TPDAS_A1M_GET_PFG_S!=s
          		|| 	TPDAS_A1M_GET_PFG_X!=x
	 		  ) 
				goto error;
       		r <<= 1;
     	}

		// finally test the PFG S-field
   		v=0;
   		r=0;
		x=TPDAS_A1M_PFG_X_DIV1;
   		for(i=0,s=1;i<3;i++)
     	{
       		TPDAS_A1M_SET_PFG(v,r,s,x);

       		if(    TPDAS_A1M_GET_PFG_V!=v
          		|| TPDAS_A1M_GET_PFG_R!=r
          		|| TPDAS_A1M_GET_PFG_S!=s
          		|| TPDAS_A1M_GET_PFG_X!=x
	 		  ) 
				goto error;
       		s <<= 1;
     	}
 	}

error:
    while(1) ;

}


