/*******************************************************************************************************
  T/PDAS-AD16/16Q/1M-DA4/16Q/1M and T/PDAS-AD16/16D/1M-DA2/16D/1M PIOX-16 AD/DA DCM Utilities. Rev.1B
  Host TORNADO DSP defs for TORNADO DSP systems/controllers with C3x/C6xxx DSP.

  (C) MicroLAB Systems Ltd, 2002

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


  Revision history:
  -----------------
	rev.1A	- Jan-2002, initial release
	rev.1B  - Sept-2002, added support for C64xx DSP

  Notes:
  ------
    1. This file must be used as .H header file for C3x/C6x DSP environment of 
	   host TORNADO-3x/P3x/E3x/6x/P6x/E6x DSP systems and controllers.

    2. All commands are adjusted for 32-bit data words and
       occupy least significant 16-bit word.

	3.	This header file is externally controlled from user C-code by run-time 
		compiler definitions (keys) in order to match DSP and TORNADO board types,
		to select desired C-code data type, to define PIOX/PIOX-16 base address, and to 
		include/exclude extended API functions.

			__HOST_C6XXX_DSP_PXW32_USHORT__    (** default)
				- if defined to any value in host DSP .C file for, then this applications
				  is assumed to be compiled for C6xxx DSP with PIOX-16 data word allocation
				  as LSW of 32-bit words and for 16-bit unsigned data type for C-code. 
				  This definition can be used for TORNADO-P62/P6202/P6203/P67/P64xx DSP systems
				  and TORNADO-E62/E6202/E6203/E67 DSP controllers. In case __PIOX16_SADDR__ 
				  definition is omitted in host DSP .C file, then it is defined in accordance 
				  with default setting for TORNADO-P62/P6202/P6203/P67 DSP systems. This is
				  also the recommended setting for TORNADO-P64xx DSP systems, since it will
				  access LSW only and will not result in one extra access cycle via EMIF-B 
				  to PIOX data buffers. 
				  This is default setting if none of __HOST_C6XXX_DSP_PXW32_ULONG__, 
				  __HOST_C6XXX_DSP_PXW32_USHORT__, __HOST_C6XXX_DSP_PXW16_USHORT__ and 
				  __HOST_C3X_DSP__ definitions is provided in host DSP .C file.

			__HOST_C6XXX_DSP_PXW32_ULONG__
				- if defined to any value in host DSP .C file for, then this applications
				  is assumed to be compiled for C6xxx DSP with PIOX-16 data word allocation
				  as LSW of 32-bit words and for 32-bit unsigned long data type for C-code. 
				  This definition can be used for TORNADO-P62/P6202/P6203/P67/P64xx DSP systems
				  and TORNADO-E62/E6202/E6203/E67 DSP controllers. In case __PIOX16_SADDR__ 
				  definition is omitted in host DSP .C file, then it is defined in accordance 
				  with default setting for TORNADO-P62/P6202/P6203/P67 DSP systems. For
				  TORNADO-P64xx DSP systems this is not recommended setting, since it will
				  result in one extra access cycle via EMIF-B to PIOX data buffers.

			__HOST_C6XXX_DSP_PXW16_USHORT__
				- if defined to any value in host DSP .C file for, then this applications
				  is assumed to be compiled for C6xxx DSP with PIOX-16 data word allocation
				  as as series of 16-bit words and for 16-bit unsigned data type for C-code. 
				  This definition can be used for TORNADO-E64xx DSP controllers only. If
				  __PIOX16_SADDR__ definition is omitted in host DSP .C file, then it will be
				  set to default setting for TORNADO-E64xx DSP controllers. This is the
				  recommended setting for TORNADO-E64xx DSP controllers.

			__HOST_C3X_DSP__ 
				- if defined to any value in host DSP .C file for, then this applications
				  is assumed to be compiled for C3x DSP. If __PIOX16_SADDR__ definition 
				  is omitted in host .C file, then it is set to default 0xff0000 value, 
				  which is common for all TORNADO-3x/TP3x/E3x DSP systems/controllers.

			__PIOX16_SADDR__ 
				- must be defined along with __HOST_C6XXX_DSP_PXW32_ULONG__, 
				  __HOST_C6XXX_DSP_PXW32_USHORT__, __HOST_C6XXX_DSP_PXW16_USHORT__ and 
				  __HOST_C3X_DSP__ definitions in order to set PIOX-16 I/O base address in DSP 
				  memory map. For some of TORNADO DSP systems and controllers, the 
				  __PIOX16_SADDR__ definition can be omitted, since in this case it will be 
				  set to default value (refer to the above description for 
				  __HOST_C6XXX_DSP_PXW32_ULONG__, __HOST_C6XXX_DSP_PXW32_USHORT__, 
				  __HOST_C6XXX_DSP_PXW16_USHORT__ and __HOST_C3X_DSP__ definitions.

			__TPDAS_A1M_EXCLUDE_FIFO_CONTROL_API__
				- if defined in user code prior inclusion of this header file, 
				  then the bodies for ADC/DAC FIFO OFFSET programming functions will be 
				  excluded from the user source code. Default is INCLUDED. Note, that in case 
				  user application comprises of several .C linked together modules, then this 
				  run-time compiler key must be included in all modules except for the one module in
				  order to exclude multiple function body appearance.
****************************************************************************************************/
#ifndef __TPDAS_A1M__
#define __TPDAS_A1M__		1

#include	"stdlib.h"


//================ DSP platform selection and host PIOX-16 address defs ===================
//	Host TORNADO DSP platform and C-code data tyoe definition via either of the 
//  following setting:
//		__HOST_C6XXX_DSP_PXW32_USHORT__ (default if none is defined)
//		__HOST_C6XXX_DSP_PXW32_ULONG__
//		__HOST_C6XXX_DSP_PXW16_USHORT__
//		__HOST_C3X_DSP__
//
//	PIOX-16 start address (__PIOX16_SADDR__) shall be defined in host TORNADO
//	DSP application (host DSP .C file) prior inclusion of this PA1M_HDSP.H file 
//  in order to provide correct operation of T/PDAS-AD16/16Q/1M DCM utilities. 
//
//	Refer to .H	files and documentation for your TORNADO-6x/P6x/E6x DSP 
//	system/controller in order to set __PIOX16_SADDR__ compiler run-time definition 
//  to the PIOX-16 base address of your TORNADO DSP system/controller. 
//  Host TORNADO-3x/P3x/E3x DSP application do not need to set __PIOX16_SADDR__ compiler
//	run-time definition.


	// check for host C6x DSP platform with PIOX-16 data words allocated as
	// LSW at 32-bit words boundaries and 16-bit unsigned data type for C-code
#ifdef __HOST_C6XXX_DSP_PXW32_USHORT__
		// treat all other defs as erroneous defs
#ifdef __HOST_C6XXX_DSP_PXW32_ULONG__		
#undef __HOST_C6XXX_DSP_PXW32_ULONG__
#endif
#ifdef __HOST_C6XXX_DSP_PXW16_USHORT__		
#undef __HOST_C6XXX_DSP_PXW16_USHORT__
#endif
#ifdef __HOST_C3X_DSP__		
#undef __HOST_C3X_DSP__
#endif
		// word address step definition
#define	_WAS_						4		

		// define C-code data type
typedef	unsigned short				__TPDAS_A1M_DATA_TYPE__;

		// check for __PIOX16_SADDR__ definition. 
		// If not defined, set to PIOX-16 base address for TORNADO-P6x.
#ifndef __PIOX16_SADDR__
#define	__PIOX16_SADDR__				0x00200000
#endif
#endif


	// check for host C6x DSP platform with PIOX-16 data words allocated as
	// LSW at 32-bit words boundaries and 32-bit unsigned data type for C-code
#ifdef __HOST_C6XXX_DSP_PXW32_ULONG__
		// treat all other defs as erroneous defs
#ifdef __HOST_C6XXX_DSP_PXW32_USHORT__		
#undef __HOST_C6XXX_DSP_PXW32_USHORT__
#endif
#ifdef __HOST_C6XXX_DSP_PXW16_USHORT__		
#undef __HOST_C6XXX_DSP_PXW16_USHORT__
#endif
#ifdef __HOST_C3X_DSP__		
#undef __HOST_C3X_DSP__
#endif
		// word address step definition
#define	_WAS_						4		

		// define C-code data type
typedef	unsigned 					__TPDAS_A1M_DATA_TYPE__;

		// check for __PIOX16_SADDR__ definition. 
		// If not defined, set to PIOX-16 base address for TORNADO-P6x.
#ifndef __PIOX16_SADDR__
#define	__PIOX16_SADDR__				0x00200000
#endif
#endif


	// check for host C6x DSP platform with PIOX-16 data words allocated at
	// 16-bit word boundaries 16-bit unsigned data type for C-code
#ifdef __HOST_C6XXX_DSP_PXW16_USHORT__
		// treat all other defs as erroneous defs
#ifdef __HOST_C6XXX_DSP_PXW32_USHORT__		
#undef __HOST_C6XXX_DSP_PXW32_USHORT__
#endif
#ifdef __HOST_C6XXX_DSP_PXW32_ULONG__		
#undef __HOST_C6XXX_DSP_PXW32_ULONG__
#endif
#ifdef __HOST_C3X_DSP__		
#undef __HOST_C3X_DSP__
#endif
		// word address step definition
#define	_WAS_						2

		// define C-code data type
typedef	unsigned short				__TPDAS_A1M_DATA_TYPE__;

		// check for __PIOX16_SADDR__ definition. 
		// If not defined, set to PIOX-16 base address for TORNADO-E64xx.
#ifndef __PIOX16_SADDR__
#define	__PIOX16_SADDR__				0x60100000
#endif
#endif


	// check for host C3x DSP platform
	// (allocated as LSW of 32-bit words and is always accessed as 32-bit unsigned)
#ifdef __HOST_C3X_DSP__
		// treat all other defs as erroneous defs
#ifdef __HOST_C6XXX_DSP_PXW32_USHORT__		
#undef __HOST_C6XXX_DSP_PXW32_USHORT__
#endif
#ifdef __HOST_C6XXX_DSP_PXW32_ULONG__		
#undef __HOST_C6XXX_DSP_PXW32_ULONG__
#endif
#ifdef __HOST_C6XXX_DSP_PXW16_USHORT__		
#undef __HOST_C6XXX_DSP_PXW16_USHORT__
#endif
		// word address step definition
#define	_WAS_						1		

		// define C-code data type
typedef	unsigned					__TPDAS_A1M_DATA_TYPE__;

		// if non defined, define PIOX-16 address to common default value for T3x/TP3x/E3x
#ifndef  __PIOX16_SADDR__
#define	__PIOX16_SADDR__				0xff0000	
#endif
#endif

	// check for neither def is undefined, and assume default host C6x DSP platform 
	// with PIOX-16 data words allocated as LSW at 32-bit words boundaries and 16-bit 
	// unsigned data type for C-code
#ifndef __HOST_C6XXX_DSP_PXW32_USHORT__
#ifndef __HOST_C6XXX_DSP_PXW32_ULONG__		
#ifndef __HOST_C6XXX_DSP_PXW16_USHORT__		
#ifndef __HOST_C3X_DSP__		
		// word address step definition
#define	_WAS_						4		

		// define C-code data type
typedef	unsigned short				__TPDAS_A1M_DATA_TYPE__;

		// check for __PIOX16_SADDR__ definition. 
		// If not defined, set to PIOX-16 base address for TORNADO-P6x.
#ifndef __PIOX16_SADDR__
#define	__PIOX16_SADDR__				0x00200000
#endif
#endif
#endif
#endif
#endif
//==========================================================================================



//=========================== general defs ==============================

#define	TPDAS_A1M_OFF			0
#define	TPDAS_A1M_ON			1
	
//===============================================================================


//============== host PIOX I/O ports address definitions ========================

#define TPDAS_A1M_DAQ_CNTR1_RG_ADDR			(__PIOX16_SADDR__+(0x0000*_WAS_))	// r/w: CONTROL #1 register
#define TPDAS_A1M_DAQ_CNTR2_RG_ADDR			(__PIOX16_SADDR__+(0x0001*_WAS_))	// r/w: CONTROL #2 register
#define TPDAS_A1M_DAQ_CNTR3_RG_ADDR			(__PIOX16_SADDR__+(0x0002*_WAS_))	// r/w: CONTROL #3 register
#define TPDAS_A1M_DAQ_SYNC_RG_ADDR			(__PIOX16_SADDR__+(0x0003*_WAS_))	// r/w: CLOCK/SYNC selector register

#define TPDAS_A1M_ADC_CNF_RG_ADDR			(__PIOX16_SADDR__+(0x0004*_WAS_))	// r/w: ADC enable flags (for IRT/FIFO), 
																				//      and ADC READY and ADC EOS indicators

#define TPDAS_A1M_ADC_IMUX_CNF1_RG_ADDR		(__PIOX16_SADDR__+(0x0006*_WAS_))	// r/w: ADC IMUX/SCCL CNF#1 register (ASYNC/SYNC)
#define TPDAS_A1M_ADC_IMUX_CNF2_RG_ADDR		(__PIOX16_SADDR__+(0x0007*_WAS_))	// r/w: ADC IMUX/SCCL CNF#2 register (SYNC)

#define TPDAS_A1M_DAC_CNF_RG_ADDR			(__PIOX16_SADDR__+(0x0008*_WAS_))	// r/w: DAC enable flags (for IRT/FIFO)

#define TPDAS_A1M_FIFO_STAT_RG_ADDR			(__PIOX16_SADDR__+(0x0009*_WAS_))	// r:   ADC/DAC FIFO status register

#define TPDAS_A1M_ERR_STAT_RG_ADDR			(__PIOX16_SADDR__+(0x000A*_WAS_))	// r:   error status register
#define TPDAS_A1M_ERR_CLR_RG_ADDR			(__PIOX16_SADDR__+(0x000A*_WAS_))	// w:   error clear register

#define TPDAS_A1M_HIRQ0_SEL_RG_ADDR			(__PIOX16_SADDR__+(0x0010*_WAS_))	// r/w: PIOX-16 HIRQ-0 selector register
#define TPDAS_A1M_HIRQ1_SEL_RG_ADDR			(__PIOX16_SADDR__+(0x0011*_WAS_))	// r/w: PIOX-16 HIRQ-1 selector register
#define TPDAS_A1M_HIRQ2_SEL_RG_ADDR			(__PIOX16_SADDR__+(0x0012*_WAS_))	// r/w: PIOX-16 HIRQ-2 selector register
#define TPDAS_A1M_HIRQ3_SEL_RG_ADDR			(__PIOX16_SADDR__+(0x0013*_WAS_))	// r/w: PIOX-16 HIRQ-3 selector register

#define TPDAS_A1M_XIM_ERR_RG_ADDR			(__PIOX16_SADDR__+(0x0016*_WAS_))	// r/w: PIOX-16 ERROR Aux IM register for HIRQ

#define TPDAS_A1M_PFG_CNTR1_RG_ADDR			(__PIOX16_SADDR__+(0x0018*_WAS_))	// r/w: PFG CONTROL #1 register
#define TPDAS_A1M_PFG_CNTR2_RG_ADDR			(__PIOX16_SADDR__+(0x0019*_WAS_))	// r/w: PFG CONTROL #2 register
#define TPDAS_A1M_PFG_CNTR3_RG_ADDR			(__PIOX16_SADDR__+(0x001A*_WAS_))	// r/w: PFG CONTROL #3 register

#define TPDAS_A1M_GPIO_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x001C*_WAS_))	// r/w: GPIO data register
#define TPDAS_A1M_GPIO_DIR_RG_ADDR			(__PIOX16_SADDR__+(0x001D*_WAS_))	// r/w: GPIO direction register


#define TPDAS_A1M_ADC_FIFO_DATA_RG_ADDR		(__PIOX16_SADDR__+(0x8000*_WAS_))	// r:   FIFO/ADC data register (ADC-SYNC mode)
#define TPDAS_A1M_ADC_FIFO_DATA_MSW_RG_ADDR	(__PIOX16_SADDR__+(0x000C*_WAS_))	// r:   D16/D17 bits of ADC FIFO 

#define TPDAS_A1M_DAC_FIFO_DATA_RG_ADDR		(__PIOX16_SADDR__+(0x8001*_WAS_))	// w:   FIFO/DAC data register (ADC-SYNC mode)

#define TPDAS_A1M_ADC_FIFO_SIN_RG_ADDR		(__PIOX16_SADDR__+(0x8002*_WAS_))	// w:   FIFO/ADC Serial-IN register for CNF mode
#define TPDAS_A1M_DAC_FIFO_SIN_RG_ADDR		(__PIOX16_SADDR__+(0x8003*_WAS_))	// w:   FIFO/DAC Serial-IN register for CNF mode

#define TPDAS_A1M_ADC_DAQ_RESET_RG_ADDR		(__PIOX16_SADDR__+(0x8004*_WAS_))	// w:   reset for ADC, ADC-DAQ (ASYNC/SYNC) and ADC-FIFO
#define TPDAS_A1M_DAC_DAQ_RESET_RG_ADDR		(__PIOX16_SADDR__+(0x8005*_WAS_))	// w:   reset for ADC, ADC-DAQ (ASYNC/SYNC) and ADC-FIFO
#define TPDAS_A1M_ADDA_DAQ_RESET_RG_ADDR	(__PIOX16_SADDR__+(0x8006*_WAS_))	// w:   simulateneous reset for ADC/DAC, ADC/DAC-DAQ (ASYNC/SYNC) and ADC/DAC-FIFO

#define TPDAS_A1M_ADC0_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8010*_WAS_))	// r:   ADC-0 data register (ADC ASYNC and SYNC-FIFO mode)
#define TPDAS_A1M_ADC1_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8011*_WAS_))	// r:   ADC-1 data register (ADC ASYNC and SYNC-FIFO mode)
#define TPDAS_A1M_ADC2_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8012*_WAS_))	// r:   ADC-2 data register (ADC ASYNC and SYNC-FIFO mode)
#define TPDAS_A1M_ADC3_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8013*_WAS_))	// r:   ADC-3 data register (ADC ASYNC and SYNC-FIFO mode)

#define TPDAS_A1M_DAC0_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8014*_WAS_))	// w:   DAC-0 data register (DAC ASYNC and SYNC-FIFO mode)
#define TPDAS_A1M_DAC1_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8015*_WAS_))	// w:   DAC-1 data register (DAC ASYNC and SYNC-FIFO mode)
#define TPDAS_A1M_DAC2_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8016*_WAS_))	// w:   DAC-2 data register (DAC ASYNC and SYNC-FIFO mode)
#define TPDAS_A1M_DAC3_DATA_RG_ADDR			(__PIOX16_SADDR__+(0x8017*_WAS_))	// w:   DAC-3 data register (DAC ASYNC and SYNC-FIFO mode)

#define TPDAS_A1M_ADC_IRT_DATA_RG_ADDR		(__PIOX16_SADDR__+(0x8018*_WAS_))	// r:   common ADC data read port for ASYNC ADC IRT controller
#define TPDAS_A1M_DAC_IRT_DATA_RG_ADDR		(__PIOX16_SADDR__+(0x8019*_WAS_))	// w:   common DAC data read port for ASYNC DAC IRT controller

#define TPDAS_A1M_ADC_START_RG_ADDR			(__PIOX16_SADDR__+(0x800c*_WAS_))	// w:   ADC conversion start command for ASYNC-mode
#define TPDAS_A1M_DAC_OUTPUT_RG_ADDR		(__PIOX16_SADDR__+(0x800d*_WAS_))	// w:   DAC output data load command for ASYNC-mode
#define TPDAS_A1M_ADC_START_DAC_OUTPUT_RG_ADDR	(__PIOX16_SADDR__+(0x800e*_WAS_))	// w:   simultaneous ADC conversion start and DAC load command for ASYNC-mode

//================================================================================


//========================== common system parameters ============================

#define	TPDAS_A1M_FIFO_LEN				0x40000L	// ADC/DAC FIFO depth

#define TPDAS_A1M_PFG_REF_CLK_FREQ		25.0e+6		// PFG source clk (Hz)

#define TPDAS_A1M_FS_MAX_FREQ			1.0e+6		// maximum sampling frequency (Hz)

#define TPDAS_A1M_AD_MAX_CHANNELS		16			// maximum number of A/D channels
#define TPDAS_A1M_DA_MAX_CHANNELS		4			// maximum D/A channels

//================================================================================

//============== control register bit/fields definitions =========================

#define TPDAS_A1M_CNTR_RG_DATA_BITMASK				0xff	// D0..D7 only are valid for all CONTROL registers (not FIFO/ADC/DAC data registers)

	//-- Control #1 Register bits
		// read-back bits bitmasks
#define TPDAS_A1M_ADC_DAQ_MODE_BITMASK				0x3		
#define TPDAS_A1M_DAC_DAQ_MODE_BITMASK				0xc
#define TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_BITMASK		0x10
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_BITMASK	0xc0

		// ADC/DAC DAQ modes
#define TPDAS_A1M_ADC_DAQ_MODE_ASYNC				0x0		// (default) ADC ASYNC mode
#define TPDAS_A1M_ADC_DAQ_MODE_SYNC_NO_FIFO			0x1		// ADC SYNC mode with PIOX-16 access (no FIFO)
#define TPDAS_A1M_ADC_DAQ_MODE_SYNC_FIFO_OPM		0x2		// ADC SYNC FIFO one pass mode
#define TPDAS_A1M_ADC_DAQ_MODE_SYNC_FIFO_PTM		0x3		// ADC SYNC FIFO pass-thru mode
#define TPDAS_A1M_DAC_DAQ_MODE_ASYNC				0x0		// (default) DAC ASYNC mode
#define TPDAS_A1M_DAC_DAQ_MODE_SYNC_NO_FIFO			0x4		// DAC SYNC mode with PIOX-16 access (no FIFO)
#define TPDAS_A1M_DAC_DAQ_MODE_SYNC_FIFO_OPM		0x8		// DAC SYNC FIFO one pass mode
#define TPDAS_A1M_DAC_DAQ_MODE_SYNC_FIFO_PTM		0xc		// DAC SYNC FIFO pass-thru mode

		// ADC ASYNC mode options
#define TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_OFF			0x0		// (default) disable Fs-event to start ADC ASYNC cycle (disable Fs selector)
#define TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_ON			0x10	// enable Fs-event to start ADC ASYNC cycle (enable Fs selector)

		// DAC output load mode options for ASYNC DAC mode 
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_TRANSPARENT	0x0	// (default) DAC Output Load is performed on data write to any of DACs for immediate DAC output update
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_SFT			0x40// DAC Output Load is performed on DSP s/w write to PIOX DAC_LD port
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_FS			0x80// DAC Output Load is performed on Fs event, which is selected by Fs selector
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_IRT_END		0xc0// DAC Output Load is performed immediately after the end of DAC ASYNC IRT cycle

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC_DAQ_MODE_BITSHIFT				0
#define TPDAS_A1M_DAC_DAQ_MODE_BITSHIFT				2
#define TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_BITSHIFT		4

		// universal defs if used via shift bits and universal macros (ADC/DAC DAQ modes)
#define TPDAS_A1M_ADDA_DAQ_MODE_ASYNC				0x0		// (default) ADC/DAC ASYNC mode
#define TPDAS_A1M_ADDA_DAQ_MODE_SYNC_NO_FIFO		0x1		// ADC/DAC SYNC mode with PIOX-16 access (no FIFO)
#define TPDAS_A1M_ADDA_DAQ_MODE_SYNC_FIFO_OPM		0x2		// ADC/DAC SYNC FIFO one pass mode
#define TPDAS_A1M_ADDA_DAQ_MODE_SYNC_FIFO_PTM		0x3		// ADC/DAC SYNC FIFO pass-thru mode
		

	//-- Control #2 Register bits
		// read-back bits bitmasks
#define TPDAS_A1M_ADC_SYNC_DAQ_RUN_BITMASK			0x1	
#define TPDAS_A1M_DAC_SYNC_DAQ_RUN_BITMASK			0x2
#define TPDAS_A1M_ADC_RDY_BITMASK					0x4
#define TPDAS_A1M_ADC_SYNC_DAQ_EOS_BITMASK			0x8
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_BITMASK		0x10
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_BITMASK		0x20
#define TPDAS_A1M_ADC_SYNC_DAQ_END_BITMASK			0x40
#define TPDAS_A1M_DAC_SYNC_DAQ_END_BITMASK			0x80

#define TPDAS_A1M_ADC_SYNC_DAQ_ABORT_BITMASK		0x40
#define TPDAS_A1M_DAC_SYNC_DAQ_ABORT_BITMASK		0x80

		// bit value defs
#define TPDAS_A1M_ADC_SYNC_DAQ_RUN_ON				0x1		// w1/r: ADC SYNC DAQ mode start (write '1') and status  (write of '0' is ignored)
#define TPDAS_A1M_ADC_SYNC_DAQ_RUN_OFF				0

#define TPDAS_A1M_DAC_SYNC_DAQ_RUN_ON				0x2		// w1/r: DAC SYNC DAQ mode start (write '1') and status (write of '0' is ignored)
#define TPDAS_A1M_DAC_SYNC_DAQ_RUN_OFF				0

#define TPDAS_A1M_ADC_RDY_OFF						0x0		// r: all ADC ready flag
#define TPDAS_A1M_ADC_RDY_ON						0x4

#define TPDAS_A1M_ADC_SYNC_DAQ_EOS_OFF				0x0		// r: EOS flag for ADC SYNC DAQ modes
#define TPDAS_A1M_ADC_SYNC_DAQ_EOS_ON				0x8

#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_ON				0x10	// r: ADC SYNC DAQ start sync status bit
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_OFF			0
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_OK				TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_ON				

#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_ON				0x20	// r: DAC SYNC DAQ start sync status bit
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_OFF			0
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_OK				TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_ON				

#define TPDAS_A1M_ADC_SYNC_DAQ_END_ON				0x40	// r: ADC SYNC DAQ end status
#define TPDAS_A1M_ADC_SYNC_DAQ_END_OFF				0

#define TPDAS_A1M_DAC_SYNC_DAQ_END_ON				0x80	// r: DAC SYNC DAQ end status
#define TPDAS_A1M_DAC_SYNC_DAQ_END_OFF				0

#define TPDAS_A1M_ADC_SYNC_DAQ_ABORT_ON				0x40	// w1: ABORT command bit (write of '0' value is ignored)	
#define TPDAS_A1M_ADC_SYNC_DAQ_ABORT_OFF			0
#define TPDAS_A1M_DAC_SYNC_DAQ_ABORT_ON				0x80
#define TPDAS_A1M_DAC_SYNC_DAQ_ABORT_OFF			0

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC_SYNC_DAQ_RUN_BITSHIFT			0
#define TPDAS_A1M_DAC_SYNC_DAQ_RUN_BITSHIFT			1
#define TPDAS_A1M_ADC_RDY_BITSHIFT					2
#define TPDAS_A1M_ADC_SYNC_DAQ_EOS_BITSHIFT			3
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_BITSHIFT		4
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_BITSHIFT		5
#define TPDAS_A1M_ADC_SYNC_DAQ_END_BITSHIFT			6
#define TPDAS_A1M_DAC_SYNC_DAQ_END_BITSHIFT			7

		// universal defs if used via shift bits and universal macros
		// These defs meet universal 'ON/OFF' defs.
#define TPDAS_A1M_ADDA_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_ON


	//-- Control #3 Register bits
		// read-back bits bitmasks
#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_BITMASK			0x1	
#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_BITMASK			0x2	
#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK		0x4	
#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK		0x8	

		// bit values defs
#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_FF				0x0	// ADC SYNC/FIFO DAQ termination flags
#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_PAF				0x1	

#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_EF				0x0	// DAC SYNC/FIFO DAQ termination flags
#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_PAE				0x2	

#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_EF			0x0	// ADC SYNC/FIFO DAQ mode IRT termination flags
#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_PAE			0x4	

#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_FF			0x0	// DAC SYNC/FIFO DAQ mode IRT termination flags
#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_PAF			0x8	


	//-- AD/DA DAQ SYNC Register bits
		// bits bitmasks
#define TPDAS_A1M_FS_SEL_BITMASK							0x3	
#define TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_BITMASK				0x4
#define TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_BITMASK	0x8
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_BITMASK			0x10	
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_BITMASK			0x20	
#define TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_BITMASK			0x40	
#define TPDAS_A1M_ADC_HIGH_SPEED_EN_BITMASK					0x80	

		// bit values defs
#define TPDAS_A1M_FS_SEL_PFG								0x0		// Sampling frequency (Fs) selector
#define TPDAS_A1M_FS_SEL_XFS								0x1
#define TPDAS_A1M_FS_SEL_PXTM0								0x2	
#define TPDAS_A1M_FS_SEL_PXTM1								0x3

#define TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_OFF					0x0		// enable signal acquisition time prior A/D conversion for ADC ASYNC DAQ mode
#define TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_ON					0x4

#define TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_OFF		0x0		// enable inter-scan_cycles delay for ADC SYNC NO-FIFO DAQ mode
#define TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_ON		0x8	

#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_SFT				0x0		// ADC DAQ start synch selector
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_XSYNC				0x10	

#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_SFT				0x0		// DAC DAQ start synch selector
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_XSYNC				0x20	

#define TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_LEVEL			0x0		// ADC/DAC XSYNC edge/level mode selector for SYNC modes
#define TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_EDGE				0x40	

#define TPDAS_A1M_ADC_HIGH_SPEED_EN_OFF						0		// ADC high-speed mode enable (0.75uS A/D conversion against 1uS A/D conversion for NORMAL mode)
#define TPDAS_A1M_ADC_HIGH_SPEED_EN_ON						0x80	
			// aliases
#define TPDAS_A1M_ADC_HIGH_SPEED_MODE						TPDAS_A1M_ADC_HIGH_SPEED_EN_OFF						
#define TPDAS_A1M_ADC_NORMAL_SPEED_MODE						TPDAS_A1M_ADC_HIGH_SPEED_EN_ON


		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_BITSHIFT				2
#define TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_BITSHIFT	3
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_BITSHIFT			4
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_BITSHIFT			5
#define TPDAS_A1M_ADC_HIGH_SPEED_EN_BITSHIFT				7

		// universal defs if used via shift bits and universal macros
#define TPDAS_A1M_ADDA_SYNC_DAQ_SSYNC_SEL_SFT				0
#define TPDAS_A1M_ADDA_SYNC_DAQ_SSYNC_SEL_XSYNC				1



	//-- ADC CNF Register bits
		// bits bitmasks
#define TPDAS_A1M_ADC0_EN_BITMASK					0x1
#define TPDAS_A1M_ADC1_EN_BITMASK					0x2
#define TPDAS_A1M_ADC2_EN_BITMASK					0x4
#define TPDAS_A1M_ADC3_EN_BITMASK					0x8
#define TPDAS_A1M_ADC_POOL_ID_BITMASK				0x80

		// bit values defs
#define TPDAS_A1M_ADC0_EN_OFF						0x0		// r/w: ADC-0 enable/disable defs
#define TPDAS_A1M_ADC0_EN_ON						0x1

#define TPDAS_A1M_ADC1_EN_OFF						0x0		// r/w: ADC-1 enable/disable defs
#define TPDAS_A1M_ADC1_EN_ON						0x2

#define TPDAS_A1M_ADC2_EN_OFF						0x0		// r/w: ADC-2 enable/disable defs (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM only)
#define TPDAS_A1M_ADC2_EN_ON						0x4

#define TPDAS_A1M_ADC3_EN_OFF						0x0		// r/w: ADC-3 enable/disable defs (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM only)
#define TPDAS_A1M_ADC3_EN_ON						0x8

			// ID for number of ADC chips in the ADC pool
#define TPDAS_A1M_ADC_POOL_ID_X2					0x0		// r: 2x ADC chips in ADC pool (T/PDAS-AD16/16D/1M-DA2/16D/1M DCM)
#define TPDAS_A1M_ADC_POOL_ID_X4					0x80	// r: 4x ADC chips in ADC pool (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM)

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC0_EN_BITSHIFT					0
#define TPDAS_A1M_ADC1_EN_BITSHIFT					1
#define TPDAS_A1M_ADC2_EN_BITSHIFT					2
#define TPDAS_A1M_ADC3_EN_BITSHIFT					3
#define TPDAS_A1M_ADC_POOL_ID_BITSHIFT				7

			// universal ID for number of ADC/DAC chips in the ADC/DAC pool
#define TPDAS_A1M_ADDA_POOL_ID_X2					0x0		// r: 2x ADC/DAC chips in ADC/DAC pools (T/PDAS-AD16/16D/1M-DA2/16D/1M DCM)
#define TPDAS_A1M_ADDA_POOL_ID_X4					0x80	// r: 4x ADC/DAC chips in ADC/DAC pools (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM)



	//-- DAC CNF Register bits
		// bits bitmasks
#define TPDAS_A1M_DAC0_EN_BITMASK					0x1
#define TPDAS_A1M_DAC1_EN_BITMASK					0x2
#define TPDAS_A1M_DAC2_EN_BITMASK					0x4
#define TPDAS_A1M_DAC3_EN_BITMASK					0x8
#define TPDAS_A1M_DAC_POOL_ID_BITMASK				0x80

		// bit values defs
#define TPDAS_A1M_DAC0_EN_OFF						0x0		// r/w: DAC-0 enable/disable defs
#define TPDAS_A1M_DAC0_EN_ON						0x1

#define TPDAS_A1M_DAC1_EN_OFF						0x0		// r/w: DAC-1 enable/disable defs
#define TPDAS_A1M_DAC1_EN_ON						0x2

#define TPDAS_A1M_DAC2_EN_OFF						0x0		// r/w: DAC-2 enable/disable defs (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM only)
#define TPDAS_A1M_DAC2_EN_ON						0x4

#define TPDAS_A1M_DAC3_EN_OFF						0x0		// r/w: DAC-3 enable/disable defs (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM only)
#define TPDAS_A1M_DAC3_EN_ON						0x8

			// ID for number of DAC chips in the DAC pool
#define TPDAS_A1M_DAC_POOL_ID_X2					0x0		// r: 2x DAC chips in DAC pool (T/PDAS-AD16/16D/1M-DA2/16D/1M DCM)
#define TPDAS_A1M_DAC_POOL_ID_X4					0x80	// r: 4x DAC chips in DAC pool (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM)

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_DAC0_EN_BITSHIFT					0
#define TPDAS_A1M_DAC1_EN_BITSHIFT					1
#define TPDAS_A1M_DAC2_EN_BITSHIFT					2
#define TPDAS_A1M_DAC3_EN_BITSHIFT					3
#define TPDAS_A1M_DAC_POOL_ID_BITSHIFT				7



	//-- ADC IMUX/SCCL CNF-#1/#2 Registers bits (T/PDAS-AD16/16Q/1M-DA4/16Q/1M DCM only)
		// fields bitmasks for ADC_IMUX_CNF1 register
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_BITMASK		0x3	
#define TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITMASK	0x4
#define TPDAS_A1M_ADC_SCCL0_EOS_EN_BITMASK			0x8
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_BITMASK		0x30	
#define TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_BITMASK	0x40
#define TPDAS_A1M_ADC_SCCL1_EOS_EN_BITMASK			0x80

		// fields bitmasks for ADC_IMUX_CNF1 register
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_BITMASK		0x3	
#define TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_BITMASK	0x4
#define TPDAS_A1M_ADC_SCCL2_EOS_EN_BITMASK			0x8
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_BITMASK		0x30	
#define TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_BITMASK	0x40
#define TPDAS_A1M_ADC_SCCL3_EOS_EN_BITMASK			0x80

		// bit values defs
			// ADC IMUX SE/DIFF channel selector
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12	0x0		// SE: ch-0/4/8/12, DIFF: 0-2/4-6/8-10/12-14
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_1_5_9_13	0x1		// SE: ch-1/5/9/13, DIFF: 1-3/5-7/9-11/13-15
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_2_6_10_14	0x2		// SE-only: ch-2/6/10/14
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_3_7_11_15	0x3		// SE-only: ch-3/7/11/15

#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_0_4_8_12	0x00	// SE: ch-0/4/8/12, DIFF: 0-2/4-6/8-10/12-14
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_1_5_9_13	0x10	// SE: ch-1/5/9/13, DIFF: 1-3/5-7/9-11/13-15
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_2_6_10_14	0x20	// SE-only: ch-2/6/10/14
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_3_7_11_15	0x30	// SE-only: ch-3/7/11/15

#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_0_4_8_12	0x0		// SE: ch-0/4/8/12, DIFF: 0-2/4-6/8-10/12-14
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_1_5_9_13	0x1		// SE: ch-1/5/9/13, DIFF: 1-3/5-7/9-11/13-15
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_2_6_10_14	0x2		// SE-only: ch-2/6/10/14
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_3_7_11_15	0x3		// SE-only: ch-3/7/11/15

#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_0_4_8_12	0x00	// SE: ch-0/4/8/12, DIFF: 0-2/4-6/8-10/12-14
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_1_5_9_13	0x10	// SE: ch-1/5/9/13, DIFF: 1-3/5-7/9-11/13-15
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_2_6_10_14	0x20	// SE-only: ch-2/6/10/14
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_3_7_11_15	0x30	// SE-only: ch-3/7/11/15

			// DIFF input enable
#define TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_OFF		0
#define TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_ON			0x4

#define TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_OFF		0
#define TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_ON			0x40

#define TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_OFF		0
#define TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_ON			0x4

#define TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_OFF		0
#define TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_ON			0x40

			// END-of-scan enable for particular scan cycle for ADC SYNC modes
#define TPDAS_A1M_ADC_SCCL0_EOS_EN_OFF				0
#define TPDAS_A1M_ADC_SCCL0_EOS_EN_ON				0x8

#define TPDAS_A1M_ADC_SCCL1_EOS_EN_OFF				0
#define TPDAS_A1M_ADC_SCCL1_EOS_EN_ON				0x80

#define TPDAS_A1M_ADC_SCCL2_EOS_EN_OFF				0
#define TPDAS_A1M_ADC_SCCL2_EOS_EN_ON				0x8

#define TPDAS_A1M_ADC_SCCL3_EOS_EN_OFF				0
#define TPDAS_A1M_ADC_SCCL3_EOS_EN_ON				0x80

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_BITSHIFT	0
#define TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITSHIFT	2
#define TPDAS_A1M_ADC_SCCL0_EOS_EN_BITSHIFT			3
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_BITSHIFT	4
#define TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_BITSHIFT	6
#define TPDAS_A1M_ADC_SCCL1_EOS_EN_BITSHIFT			7

#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_BITSHIFT	0
#define TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_BITSHIFT	2
#define TPDAS_A1M_ADC_SCCL2_EOS_EN_BITSHIFT			3
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_BITSHIFT	4
#define TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_BITSHIFT	6
#define TPDAS_A1M_ADC_SCCL3_EOS_EN_BITSHIFT			7

			// universal values for AMUX AIN channel selector
#define TPDAS_A1M_ADC_SCCL_IMUX_CH_SEL_0_4_8_12		0x0		// SE: ch-0/4/8/12, DIFF: 0-2/4-6/8-10/12-14
#define TPDAS_A1M_ADC_SCCL_IMUX_CH_SEL_1_5_9_13		0x1		// SE: ch-1/5/9/13, DIFF: 1-3/5-7/9-11/13-15
#define TPDAS_A1M_ADC_SCCL_IMUX_CH_SEL_2_6_10_14	0x2		// SE-only: ch-2/6/10/14
#define TPDAS_A1M_ADC_SCCL_IMUX_CH_SEL_3_7_11_15	0x3		// SE-only: ch-3/7/11/15

		// ADC ASYNC DAQ uses SCCL#0 control bits for ADC SYNC DAQ modes in order to control IMUX
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_BITMASK		TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_BITMASK	
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_BITMASK	TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITMASK

#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_0_4_8_12	TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_1_5_9_13	TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_1_5_9_13
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_2_6_10_14	TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_2_6_10_14
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_3_7_11_15	TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_3_7_11_15

#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_OFF		TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_OFF
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_ON			TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_ON

#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_BITSHIFT	TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_BITSHIFT
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_BITSHIFT	TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITSHIFT



	//-- ADC/DAC FIFO status Register bits (read-only)
#define TPDAS_A1M_ADC_FIFO_FF_BITMASK				0x01	// ADC FIFO status flags
#define TPDAS_A1M_ADC_FIFO_PAF_BITMASK				0x02
#define TPDAS_A1M_ADC_FIFO_EF_BITMASK				0x04
#define TPDAS_A1M_ADC_FIFO_PAE_BITMASK				0x08
#define TPDAS_A1M_DAC_FIFO_FF_BITMASK				0x10	// DAC FIFO status flags
#define TPDAS_A1M_DAC_FIFO_PAF_BITMASK				0x20
#define TPDAS_A1M_DAC_FIFO_EF_BITMASK				0x40
#define TPDAS_A1M_DAC_FIFO_PAE_BITMASK				0x80

		// bit values defs
#define TPDAS_A1M_ADC_FIFO_FF_OFF					0
#define TPDAS_A1M_ADC_FIFO_FF_ON					0x01	

#define TPDAS_A1M_ADC_FIFO_PAF_OFF					0
#define TPDAS_A1M_ADC_FIFO_PAF_ON					0x02	

#define TPDAS_A1M_ADC_FIFO_EF_OFF					0
#define TPDAS_A1M_ADC_FIFO_EF_ON					0x04

#define TPDAS_A1M_ADC_FIFO_PAE_OFF					0
#define TPDAS_A1M_ADC_FIFO_PAE_ON					0x08

#define TPDAS_A1M_DAC_FIFO_FF_OFF					0
#define TPDAS_A1M_DAC_FIFO_FF_ON					0x10

#define TPDAS_A1M_DAC_FIFO_PAF_OFF					0
#define TPDAS_A1M_DAC_FIFO_PAF_ON					0x20

#define TPDAS_A1M_DAC_FIFO_EF_OFF					0
#define TPDAS_A1M_DAC_FIFO_EF_ON					0x40

#define TPDAS_A1M_DAC_FIFO_PAE_OFF					0
#define TPDAS_A1M_DAC_FIFO_PAE_ON					0x80

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC_FIFO_FF_SHIFT					0
#define TPDAS_A1M_ADC_FIFO_PAF_SHIFT				1
#define TPDAS_A1M_ADC_FIFO_EF_SHIFT					2
#define TPDAS_A1M_ADC_FIFO_PAE_SHIFT				3
#define TPDAS_A1M_DAC_FIFO_FF_SHIFT					4
#define TPDAS_A1M_DAC_FIFO_PAF_SHIFT				5
#define TPDAS_A1M_DAC_FIFO_EF_SHIFT					6
#define TPDAS_A1M_DAC_FIFO_PAE_SHIFT				7


	//-- ERROR status Register bits (r-only)
#define TPDAS_A1M_ADC_FIFO_OVF_BITMASK				0x01
#define TPDAS_A1M_ADC_FIFO_UNF_BITMASK				0x02
#define TPDAS_A1M_DAC_FIFO_OVF_BITMASK				0x04
#define TPDAS_A1M_DAC_FIFO_UNF_BITMASK				0x08
#define TPDAS_A1M_ADC_IRT_ERR_BITMASK				0x10
#define TPDAS_A1M_DAC_IRT_ERR_BITMASK				0x20

		// bit values defs
#define TPDAS_A1M_ADC_FIFO_OVF_OFF					0
#define TPDAS_A1M_ADC_FIFO_OVF_ON					0x01

#define TPDAS_A1M_ADC_FIFO_UNF_OFF					0
#define TPDAS_A1M_ADC_FIFO_UNF_ON					0x02

#define TPDAS_A1M_DAC_FIFO_OVF_OFF					0
#define TPDAS_A1M_DAC_FIFO_OVF_ON					0x04

#define TPDAS_A1M_DAC_FIFO_UNF_OFF					0
#define TPDAS_A1M_DAC_FIFO_UNF_ON					0x08

#define TPDAS_A1M_ADC_IRT_ERR_OFF					0
#define TPDAS_A1M_ADC_IRT_ERR_ON					0x10

#define TPDAS_A1M_DAC_IRT_ERR_OFF					0
#define TPDAS_A1M_DAC_IRT_ERR_ON					0x20

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC_FIFO_OVF_SHIFT				0
#define TPDAS_A1M_ADC_FIFO_UNF_SHIFT				1
#define TPDAS_A1M_DAC_FIFO_OVF_SHIFT				2
#define TPDAS_A1M_DAC_FIFO_UNF_SHIFT				3
#define TPDAS_A1M_ADC_IRT_ERR_SHIFT					4	
#define TPDAS_A1M_DAC_IRT_ERR_SHIFT					5


	//-- ERROR CLEAR Register bits (w-only, '0' bit values are ignored)
#define TPDAS_A1M_ADC_FIFO_OVF_CLR					0x01
#define TPDAS_A1M_ADC_FIFO_UNF_CLR					0x02
#define TPDAS_A1M_DAC_FIFO_OVF_CLR					0x04
#define TPDAS_A1M_DAC_FIFO_UNF_CLR					0x08
#define TPDAS_A1M_ADC_IRT_ERR_CLR					0x10
#define TPDAS_A1M_DAC_IRT_ERR_CLR					0x20



	//-- HIRQ-0..3 Interrupt control Registers bits
			// bitmasks
#define TPDAS_A1M_HIRQ_SEL_BITMASK					0x1f
#define TPDAS_A1M_HIRQ_EN_BITMASK					0x80
#define TPDAS_A1M_HIRQ_POL_BITMASK					0x40

			// HIRQ source selector settings
			// Note, that HIRQ-0/1 support 21 HORQ sources, whereas HIRQ-2/3 support 11 HIRQ sources.
#define TPDAS_A1M_HIRQ_SEL_OFF						0x00	
#define TPDAS_A1M_HIRQ_SEL_NO						TPDAS_A1M_HIRQ_SEL_OFF
#define TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END			0x01
#define TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END			0x02
#define TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK	0x03
#define TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK	0x04
#define TPDAS_A1M_HIRQ_SEL_ADC_IRT					0x05
#define TPDAS_A1M_HIRQ_SEL_DAC_IRT					0x06
#define TPDAS_A1M_HIRQ_SEL_GPIO0					0x07
#define TPDAS_A1M_HIRQ_SEL_GPIO1					0x08
#define TPDAS_A1M_HIRQ_SEL_ERR						0x09
#define TPDAS_A1M_HIRQ_SEL_FS						0x0A
#define TPDAS_A1M_HIRQ_SEL_ADC_RDY					0x0B

				// HIRQ sources for HIRQ-0/1 only
#define TPDAS_A1M_HIRQ_SEL_ADC_IRT_END				0x0E
#define TPDAS_A1M_HIRQ_SEL_DAC_IRT_END				0x0F
#define TPDAS_A1M_HIRQ_SEL_ADC_FIFO_FF				0x10	
#define TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAF				0x11
#define TPDAS_A1M_HIRQ_SEL_ADC_FIFO_EF				0x12
#define TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAE				0x13
#define TPDAS_A1M_HIRQ_SEL_DAC_FIFO_FF				0x14
#define TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAF				0x15
#define TPDAS_A1M_HIRQ_SEL_DAC_FIFO_EF				0x16
#define TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAE				0x17

#define TPDAS_A1M_HIRQ_EN_OFF						0x00	// HIRQ line OE-enable control
#define TPDAS_A1M_HIRQ_EN_ON						0x80

#define TPDAS_A1M_HIRQ_POL_POS						0x00	// HIRQ line polarity control
#define TPDAS_A1M_HIRQ_POL_NEG						0x40	

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_HIRQ_EN_SHIFT						0x7
#define TPDAS_A1M_HIRQ_POL_SHIFT					0x6


	//-- HIRQ Aux Interrupt Mask (XIM) Registers bits, which 
	//   generates TPDAS_A1M_HIRQ_SEL_ERR HRQ source
			// bitmasks
#define TPDAS_A1M_XIM_ADC_FIFO_OVF_BITMASK			0x01
#define TPDAS_A1M_XIM_ADC_FIFO_UNF_BITMASK			0x02
#define TPDAS_A1M_XIM_DAC_FIFO_OVF_BITMASK			0x04
#define TPDAS_A1M_XIM_DAC_FIFO_UNF_BITMASK			0x08
#define TPDAS_A1M_XIM_ADC_IRT_ERR_BITMASK			0x10
#define TPDAS_A1M_XIM_DAC_IRT_ERR_BITMASK			0x20

		// bit values defs
#define TPDAS_A1M_XIM_ADC_FIFO_OVF_OFF				0
#define TPDAS_A1M_XIM_ADC_FIFO_OVF_ON				0x01

#define TPDAS_A1M_XIM_ADC_FIFO_UNF_OFF				0
#define TPDAS_A1M_XIM_ADC_FIFO_UNF_ON				0x02

#define TPDAS_A1M_XIM_DAC_FIFO_OVF_OFF				0
#define TPDAS_A1M_XIM_DAC_FIFO_OVF_ON				0x04

#define TPDAS_A1M_XIM_DAC_FIFO_UNF_OFF				0
#define TPDAS_A1M_XIM_DAC_FIFO_UNF_ON				0x08

#define TPDAS_A1M_XIM_ADC_IRT_ERR_OFF				0
#define TPDAS_A1M_XIM_ADC_IRT_ERR_ON				0x10

#define TPDAS_A1M_XIM_DAC_IRT_ERR_OFF				0
#define TPDAS_A1M_XIM_DAC_IRT_ERR_ON				0x20

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_XIM_ADC_FIFO_OVF_BITSHIFT			0
#define TPDAS_A1M_XIM_ADC_FIFO_UNF_BITSHIFT			1
#define TPDAS_A1M_XIM_DAC_FIFO_OVF_BITSHIFT			2
#define TPDAS_A1M_XIM_DAC_FIFO_UNF_BITSHIFT			3
#define TPDAS_A1M_XIM_ADC_IRT_ERR_BITSHIFT			4
#define TPDAS_A1M_XIM_DAC_IRT_ERR_BITSHIFT			5


	//--- GPIO-DIR/DATA registers for GPIO-0..3 control
			// bitmasks
#define TPDAS_A1M_GPIO0_DIR_BITMASK					1
#define TPDAS_A1M_GPIO1_DIR_BITMASK					2
#define TPDAS_A1M_GPIO2_DIR_BITMASK					4
#define TPDAS_A1M_GPIO3_DIR_BITMASK					8
#define TPDAS_A1M_GPIO0_DATA_BITMASK				1
#define TPDAS_A1M_GPIO1_DATA_BITMASK				2
#define TPDAS_A1M_GPIO2_DATA_BITMASK				4
#define TPDAS_A1M_GPIO3_DATA_BITMASK				8

			// bit values defs
#define TPDAS_A1M_GPIO0_DIR_INPUT					0
#define TPDAS_A1M_GPIO0_DIR_OUTPUT					1
#define TPDAS_A1M_GPIO1_DIR_INPUT					0
#define TPDAS_A1M_GPIO1_DIR_OUTPUT					2
#define TPDAS_A1M_GPIO2_DIR_INPUT					0
#define TPDAS_A1M_GPIO2_DIR_OUTPUT					4
#define TPDAS_A1M_GPIO3_DIR_INPUT					0
#define TPDAS_A1M_GPIO3_DIR_OUTPUT					8

#define TPDAS_A1M_GPIO0_DATA_OFF					0
#define TPDAS_A1M_GPIO0_DATA_ON						1
#define TPDAS_A1M_GPIO1_DATA_OFF					0
#define TPDAS_A1M_GPIO1_DATA_ON						2
#define TPDAS_A1M_GPIO2_DATA_OFF					0
#define TPDAS_A1M_GPIO2_DATA_ON						4
#define TPDAS_A1M_GPIO3_DATA_OFF					0
#define TPDAS_A1M_GPIO3_DATA_ON						8

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_GPIO0_DIR_BITSHIFT				0	
#define TPDAS_A1M_GPIO1_DIR_BITSHIFT				1
#define TPDAS_A1M_GPIO2_DIR_BITSHIFT				2
#define TPDAS_A1M_GPIO3_DIR_BITSHIFT				3
#define TPDAS_A1M_GPIO0_DATA_BITSHIFT				0
#define TPDAS_A1M_GPIO1_DATA_BITSHIFT				1
#define TPDAS_A1M_GPIO2_DATA_BITSHIFT				2
#define TPDAS_A1M_GPIO3_DATA_BITSHIFT				3

		// universal defs if used via shift bits and universal macros
#define TPDAS_A1M_GPIO_DIR_INPUT					0
#define TPDAS_A1M_GPIO_DIR_OUTPUT					1
#define TPDAS_A1M_GPIO_DATA_OFF						0
#define TPDAS_A1M_GPIO_DATA_ON						1


	//--- PFG CNTR1..3 registers defs
#define TPDAS_A1M_PFG_V07_BITMASK					0xff	// PFG CNTR1 register
#define TPDAS_A1M_PFG_V8_BITMASK					0x01	// PFG CNTR2 register
#define TPDAS_A1M_PFG_R_BITMASK						0xfe	// PFG CNTR2 register
#define TPDAS_A1M_PFG_S_BITMASK						0x07	// PFG CNTR3 register
#define TPDAS_A1M_PFG_X_BITMASK						0x30	// PFG CNTR3 register

		// S-divider encoder-field values and the corresponding S-divider values
#define TPDAS_A1M_PFG_S_DIV2						1		// :2
#define TPDAS_A1M_PFG_S_DIV4						3		// :4
#define TPDAS_A1M_PFG_S_DIV5						4		// :5
#define TPDAS_A1M_PFG_S_DIV6						7		// :6
#define TPDAS_A1M_PFG_S_DIV7						5		// :7
#define TPDAS_A1M_PFG_S_DIV8						2		// :8
#define TPDAS_A1M_PFG_S_DIV9						6		// :9
#define TPDAS_A1M_PFG_S_DIV10						0		// :10

		// X-divider encoder-field values and the corresponding X-divider values
#define TPDAS_A1M_PFG_X_DIV1						0		// :1
#define TPDAS_A1M_PFG_X_DIV32						0x10	// :32
#define TPDAS_A1M_PFG_X_DIV1024						0x20	// :1024

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_PFG_R_BITSHIFT					1
//========================================================================


//============== ADC/DAC direct data and FIFO bit/fields definitions =====
	// ADC/DAC FIFO data bitmask
#define TPDAS_A1M_ADDA_FIFO_DATA_BITMASK			0xffff	// D0..D15 data bits for ADC/DAC FIFO data	

	// ADC/DAC direct PIOX-16 data bitmask
#define TPDAS_A1M_ADDA_DATA_BITMASK					0xffff	// D0..D15 data bits for direct ADC/DAC PIOX access


	//-- ADC-FIFO MSW Register bits (read-only)
	//   Returns FIFO D16/D17 bits of ADC-FIFO, which are not available during FIFO-read via PIOX-16.
	//   ADC-FIFO D16 bit is the EOS flag. D17 is not used.
#define TPDAS_A1M_ADC_FIFO_MSW_DATA_BITMASK			0x03	// ADC FIFO MSW data bits, available as a separate PIOX-16 word
#define TPDAS_A1M_ADC_FIFO_MSW_EOS_BITMASK			0x01

		// bit values defs
#define TPDAS_A1M_ADC_FIFO_MSW_EOS_OFF				0		// ADC FIFO D16-bit is EOS
#define TPDAS_A1M_ADC_FIFO_MSW_EOS_ON				0x01

		// bitshift data for universal macros using unified vars and ON/OFF defs
#define TPDAS_A1M_ADC_FIFO_MSW_EOS_BITSHIFT			0


	//-- ADC/DAC FIFO serial-IN CNF port
#define TPDAS_A1M_ADDA_FIFO_SIN_BITMASK				0x1		// D0 is only valid for ADC/DAC FIFO Serial-IN data	in ADC/DAC ASYNC DAQ mode

//========================================================================



//================ macros for accessing HIF registers ====================
// T/PDAS-AD16/16Q/1M DCM host DSP utilities offer the unified utility functions synopsis:
//  - TPDAS_A1M_set_..._rg(..)    - sets the control register contents
//	- TPDAS_A1M_get_..._rg()      - returns contents of control register
//  - TPDAS_A1M_set_...(..)       - sets bit of control register (using particular bit set value)
//  - TPDAS_A1M_get_...()         - returns bit state of control register (using particular bit set value)
//  - TPDAS_A1M_uset_...(ON/OFF)  - sets bit of control register using unified ON/OFF values
//  - TPDAS_A1M_uget_...()        - returns bit state of control register using unified ON/OFF values


	//~~~ ADC/DAC DAQ controllers reset macros (both ASYNC/SYNC modes)
#define TPDAS_A1M_reset_adc_daq()					(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_DAQ_RESET_RG_ADDR) = 0
#define TPDAS_A1M_reset_dac_daq()					(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC_DAQ_RESET_RG_ADDR) = 0
#define TPDAS_A1M_reset_adda_daq()					(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADDA_DAQ_RESET_RG_ADDR) = 0

#define TPDAS_A1M_RESET_ADC_DAQ						TPDAS_A1M_reset_adc_daq()
#define TPDAS_A1M_RESET_DAC_DAQ						TPDAS_A1M_reset_dac_daq()
#define TPDAS_A1M_RESET_ADDA_DAQ					TPDAS_A1M_reset_adda_daq()

		// aliases
#define TPDAS_A1M_RESET_ADC_DAQ_FIFO				TPDAS_A1M_RESET_ADC_DAQ
#define TPDAS_A1M_RESET_ADC_FIFO					TPDAS_A1M_RESET_ADC_DAQ
#define TPDAS_A1M_RESET_DAC_DAQ_FIFO				TPDAS_A1M_RESET_DAC_DAQ
#define TPDAS_A1M_RESET_DAC_FIFO					TPDAS_A1M_RESET_DAC_DAQ
#define TPDAS_A1M_RESET_DAQ_ALL						TPDAS_A1M_RESET_ADDA_DAQ
#define TPDAS_A1M_RESET_DAQ_FIFO_ALL				TPDAS_A1M_RESET_ADDA_DAQ
#define TPDAS_A1M_RESET_FIFO_ALL					TPDAS_A1M_RESET_ADDA_DAQ


	//~~~ CONTROL-1 register
#define TPDAS_A1M_get_daq_cntr1_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_CNTR1_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_daq_cntr1_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_CNTR1_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc_daq_mode()				(TPDAS_A1M_get_daq_cntr1_rg() & TPDAS_A1M_ADC_DAQ_MODE_BITMASK)
#define TPDAS_A1M_set_adc_daq_mode(d)				TPDAS_A1M_set_daq_cntr1_rg((TPDAS_A1M_get_daq_cntr1_rg() & ~TPDAS_A1M_ADC_DAQ_MODE_BITMASK) | (d & TPDAS_A1M_ADC_DAQ_MODE_BITMASK))

#define TPDAS_A1M_get_dac_daq_mode()				(TPDAS_A1M_get_daq_cntr1_rg() & TPDAS_A1M_DAC_DAQ_MODE_BITMASK)
#define TPDAS_A1M_set_dac_daq_mode(d)				TPDAS_A1M_set_daq_cntr1_rg((TPDAS_A1M_get_daq_cntr1_rg() & ~TPDAS_A1M_DAC_DAQ_MODE_BITMASK) | (d & TPDAS_A1M_DAC_DAQ_MODE_BITMASK))

#define TPDAS_A1M_get_adc_async_daq_fs_en()			(TPDAS_A1M_get_daq_cntr1_rg() & TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_BITMASK)
#define TPDAS_A1M_set_adc_async_daq_fs_en(d)		TPDAS_A1M_set_daq_cntr1_rg((TPDAS_A1M_get_daq_cntr1_rg() & ~TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_BITMASK) | (d & TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_BITMASK))

#define TPDAS_A1M_get_dac_async_daq_output_mode()	(TPDAS_A1M_get_daq_cntr1_rg() & TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_BITMASK)
#define TPDAS_A1M_set_dac_async_daq_output_mode(d)	TPDAS_A1M_set_daq_cntr1_rg((TPDAS_A1M_get_daq_cntr1_rg() & ~TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_BITMASK) | (d & TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_BITMASK))

		// unified macros with TPDAS_A1M_ADDA_DAQ_MODE_ASYNC/_SYNC_NO_FIFO/_SYNC_FIFO_OPM/_SYNC_FIFO_PTM
#define TPDAS_A1M_uget_adc_daq_mode()				(TPDAS_A1M_get_adc_daq_mode()>>TPDAS_A1M_ADC_DAQ_MODE_BITSHIFT) 
#define TPDAS_A1M_uset_adc_daq_mode(d)				TPDAS_A1M_set_adc_daq_mode((d)<<TPDAS_A1M_ADC_DAQ_MODE_BITSHIFT)

#define TPDAS_A1M_uget_dac_daq_mode()				(TPDAS_A1M_get_dac_daq_mode()>>TPDAS_A1M_DAC_DAQ_MODE_BITSHIFT) 
#define TPDAS_A1M_uset_dac_daq_mode(d)				TPDAS_A1M_set_dac_daq_mode((d)<<TPDAS_A1M_DAC_DAQ_MODE_BITSHIFT)

		// unified macros with TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_..
#define TPDAS_A1M_uget_dac_async_daq_output_mode()	TPDAS_A1M_get_dac_async_daq_output_mode()	
#define TPDAS_A1M_uset_dac_async_daq_output_mode(d)	TPDAS_A1M_set_dac_async_daq_output_mode(d)

		// unified macros with ON/OFF
#define TPDAS_A1M_uget_adc_async_daq_fs_en()		(TPDAS_A1M_get_adc_async_daq_fs_en()>>TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_async_daq_fs_en(d)		TPDAS_A1M_set_adc_async_daq_fs_en((d)<<TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_BITSHIFT)

		// direct set/get macros
#define TPDAS_A1M_SET_ADC_DAQ_MODE_ASYNC			TPDAS_A1M_set_adc_daq_mode(TPDAS_A1M_ADC_DAQ_MODE_ASYNC)
#define TPDAS_A1M_SET_ADC_DAQ_MODE_SYNC_NO_FIFO		TPDAS_A1M_set_adc_daq_mode(TPDAS_A1M_ADC_DAQ_MODE_SYNC_NO_FIFO)
#define TPDAS_A1M_SET_ADC_DAQ_MODE_SYNC_FIFO_OPM	TPDAS_A1M_set_adc_daq_mode(TPDAS_A1M_ADC_DAQ_MODE_SYNC_FIFO_OPM)
#define TPDAS_A1M_SET_ADC_DAQ_MODE_SYNC_FIFO_PTM	TPDAS_A1M_set_adc_daq_mode(TPDAS_A1M_ADC_DAQ_MODE_SYNC_FIFO_PTM)

#define TPDAS_A1M_GET_ADC_DAQ_MODE					TPDAS_A1M_get_adc_daq_mode()
#define TPDAS_A1M_ADC_DAQ_MODE_IS_ASYNC				(TPDAS_A1M_get_adc_daq_mode()==TPDAS_A1M_ADC_DAQ_MODE_ASYNC)
#define TPDAS_A1M_ADC_DAQ_MODE_IS_SYNC_NO_FIFO		(TPDAS_A1M_get_adc_daq_mode()==TPDAS_A1M_ADC_DAQ_MODE_SYNC_NO_FIFO)
#define TPDAS_A1M_ADC_DAQ_MODE_IS_SYNC_FIFO_OPM		(TPDAS_A1M_get_adc_daq_mode()==TPDAS_A1M_ADC_DAQ_MODE_SYNC_FIFO_OPM)
#define TPDAS_A1M_ADC_DAQ_MODE_IS_SYNC_FIFO_PTM		(TPDAS_A1M_get_adc_daq_mode()==TPDAS_A1M_ADC_DAQ_MODE_SYNC_FIFO_PTM)

#define TPDAS_A1M_SET_DAC_DAQ_MODE_ASYNC			TPDAS_A1M_set_dac_daq_mode(TPDAS_A1M_DAC_DAQ_MODE_ASYNC)
#define TPDAS_A1M_SET_DAC_DAQ_MODE_SYNC_NO_FIFO		TPDAS_A1M_set_dac_daq_mode(TPDAS_A1M_DAC_DAQ_MODE_SYNC_NO_FIFO)
#define TPDAS_A1M_SET_DAC_DAQ_MODE_SYNC_FIFO_OPM	TPDAS_A1M_set_dac_daq_mode(TPDAS_A1M_DAC_DAQ_MODE_SYNC_FIFO_OPM)
#define TPDAS_A1M_SET_DAC_DAQ_MODE_SYNC_FIFO_PTM	TPDAS_A1M_set_dac_daq_mode(TPDAS_A1M_DAC_DAQ_MODE_SYNC_FIFO_PTM)

#define TPDAS_A1M_GET_DAC_DAQ_MODE					TPDAS_A1M_get_dac_daq_mode()
#define TPDAS_A1M_DAC_DAQ_MODE_IS_ASYNC				(TPDAS_A1M_get_dac_daq_mode()==TPDAS_A1M_DAC_DAQ_MODE_ASYNC)
#define TPDAS_A1M_DAC_DAQ_MODE_IS_SYNC_NO_FIFO		(TPDAS_A1M_get_dac_daq_mode()==TPDAS_A1M_DAC_DAQ_MODE_SYNC_NO_FIFO)
#define TPDAS_A1M_DAC_DAQ_MODE_IS_SYNC_FIFO_OPM		(TPDAS_A1M_get_dac_daq_mode()==TPDAS_A1M_DAC_DAQ_MODE_SYNC_FIFO_OPM)
#define TPDAS_A1M_DAC_DAQ_MODE_IS_SYNC_FIFO_PTM		(TPDAS_A1M_get_dac_daq_mode()==TPDAS_A1M_DAC_DAQ_MODE_SYNC_FIFO_PTM)

#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_FS_EN_OFF		TPDAS_A1M_set_adc_async_daq_fs_en(TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_OFF)
#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_FS_EN_ON		TPDAS_A1M_set_adc_async_daq_fs_en(TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_ON)

#define TPDAS_A1M_GET_ADC_ASYNC_DAQ_FS_EN			TPDAS_A1M_get_adc_async_daq_fs_en()
#define TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_IS_OFF		(TPDAS_A1M_get_adc_async_daq_fs_en()==TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_OFF)
#define TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_IS_ON			(TPDAS_A1M_get_adc_async_daq_fs_en()==TPDAS_A1M_ADC_ASYNC_DAQ_FS_EN_ON)

#define TPDAS_A1M_SET_DAC_ASYNC_DAQ_OUTPUT_MODE_TRANSPARENT	TPDAS_A1M_set_dac_async_daq_output_mode(TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_TRANSPARENT)
#define TPDAS_A1M_SET_DAC_ASYNC_DAQ_OUTPUT_MODE_SFT			TPDAS_A1M_set_dac_async_daq_output_mode(TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_SFT)
#define TPDAS_A1M_SET_DAC_ASYNC_DAQ_OUTPUT_MODE_FS			TPDAS_A1M_set_dac_async_daq_output_mode(TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_FS)
#define TPDAS_A1M_SET_DAC_ASYNC_DAQ_OUTPUT_MODE_IRT_END		TPDAS_A1M_set_dac_async_daq_output_mode(TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_IRT_END)

#define TPDAS_A1M_GET_DAC_ASYNC_DAQ_OUTPUT_MODE				TPDAS_A1M_get_dac_async_daq_output_mode()
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_IS_TRANSPARENT	(TPDAS_A1M_get_dac_async_daq_output_mode()==TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_TRANSPARENT)
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_IS_SFT			(TPDAS_A1M_get_dac_async_daq_output_mode()==TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_SFT)
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_IS_FS			(TPDAS_A1M_get_dac_async_daq_output_mode()==TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_FS)
#define TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_IS_IRT_END		(TPDAS_A1M_get_dac_async_daq_output_mode()==TPDAS_A1M_DAC_ASYNC_DAQ_OUTPUT_MODE_IRT_END)



	//--- CONTROL-2 register
#define TPDAS_A1M_get_daq_cntr2_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_CNTR2_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_daq_cntr2_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_CNTR2_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc_sync_daq_run()			(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_ADC_SYNC_DAQ_RUN_BITMASK)
#define TPDAS_A1M_set_adc_sync_daq_run(d)			TPDAS_A1M_set_daq_cntr2_rg(d & TPDAS_A1M_ADC_SYNC_DAQ_RUN_BITMASK)

#define TPDAS_A1M_get_dac_sync_daq_run()			(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_DAC_SYNC_DAQ_RUN_BITMASK)
#define TPDAS_A1M_set_dac_sync_daq_run(d)			TPDAS_A1M_set_daq_cntr2_rg(d & TPDAS_A1M_DAC_SYNC_DAQ_RUN_BITMASK)

#define TPDAS_A1M_set_adda_sync_daq_run(d)			TPDAS_A1M_set_daq_cntr2_rg(d & (TPDAS_A1M_DAC_SYNC_DAQ_RUN_BITMASK | TPDAS_A1M_ADC_SYNC_DAQ_RUN_BITMASK))

#define TPDAS_A1M_get_adc_rdy()						(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_ADC_RDY_BITMASK)

#define TPDAS_A1M_get_adc_sync_daq_eos()			(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_ADC_SYNC_DAQ_EOS_BITMASK)

#define TPDAS_A1M_set_adc_sync_daq_abort(d)			TPDAS_A1M_set_daq_cntr2_rg(d & TPDAS_A1M_ADC_SYNC_DAQ_ABORT_BITMASK)
#define TPDAS_A1M_set_dac_sync_daq_abort(d)			TPDAS_A1M_set_daq_cntr2_rg(d & TPDAS_A1M_DAC_SYNC_DAQ_ABORT_BITMASK)
#define TPDAS_A1M_set_adda_sync_daq_abort(d)		TPDAS_A1M_set_daq_cntr2_rg(d & (TPDAS_A1M_ADC_SYNC_DAQ_ABORT_BITMASK | TPDAS_A1M_DAC_SYNC_DAQ_ABORT_BITMASK))

#define TPDAS_A1M_get_adc_sync_daq_end()			(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_ADC_SYNC_DAQ_END_BITMASK)
#define TPDAS_A1M_get_dac_sync_daq_end()			(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_DAC_SYNC_DAQ_END_BITMASK)
#define TPDAS_A1M_get_adc_sync_daq_ssync()			(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_BITMASK)
#define TPDAS_A1M_get_dac_sync_daq_ssync()			(TPDAS_A1M_get_daq_cntr2_rg() & TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_BITMASK)

		// unified macros with ON/OFF (only read-back unified macros are provided) 
#define TPDAS_A1M_uget_adc_sync_daq_run()			TPDAS_A1M_get_adc_sync_daq_run()
#define TPDAS_A1M_uget_dac_sync_daq_run()			(TPDAS_A1M_get_dac_sync_daq_run() >> TPDAS_A1M_DAC_SYNC_DAQ_RUN_BITSHIFT)

#define TPDAS_A1M_uget_adc_rdy()					(TPDAS_A1M_get_adc_rdy()>>TPDAS_A1M_ADC_RDY_BITSHIFT)
#define TPDAS_A1M_uget_adc_sync_daq_eos()			(TPDAS_A1M_get_adc_sync_daq_eos()>>TPDAS_A1M_ADC_SYNC_DAQ_EOS_BITSHIFT)

#define TPDAS_A1M_uget_adc_sync_daq_end()			(TPDAS_A1M_get_adc_sync_daq_end() >> TPDAS_A1M_ADC_SYNC_DAQ_END_BITSHIFT)
#define TPDAS_A1M_uget_dac_sync_daq_end()			(TPDAS_A1M_get_dac_sync_daq_end() >> TPDAS_A1M_DAC_SYNC_DAQ_END_BITSHIFT)

#define TPDAS_A1M_uget_adc_sync_daq_ssync()			(TPDAS_A1M_get_adc_sync_daq_ssync()	>> TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_BITSHIFT)
#define TPDAS_A1M_uget_dac_sync_daq_ssync()			(TPDAS_A1M_get_dac_sync_daq_ssync()	>> TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_BITSHIFT)

		// direct set/get macros
#define TPDAS_A1M_SET_ADC_SYNC_DAQ_RUN_ON			TPDAS_A1M_set_adc_sync_daq_run(TPDAS_A1M_ADC_SYNC_DAQ_RUN_ON)
#define TPDAS_A1M_SET_DAC_SYNC_DAQ_RUN_ON			TPDAS_A1M_set_dac_sync_daq_run(TPDAS_A1M_DAC_SYNC_DAQ_RUN_ON)
#define TPDAS_A1M_SET_ADDA_SYNC_DAQ_RUN_ON			TPDAS_A1M_set_adda_sync_daq_run(TPDAS_A1M_ADC_SYNC_DAQ_RUN_ON | TPDAS_A1M_DAC_SYNC_DAQ_RUN_ON)			

#define TPDAS_A1M_GET_ADC_SYNC_DAQ_RUN				TPDAS_A1M_get_adc_sync_daq_run()
#define TPDAS_A1M_ADC_SYNC_DAQ_RUN_IS_OFF			(TPDAS_A1M_get_adc_sync_daq_run()==TPDAS_A1M_ADC_SYNC_DAQ_RUN_OFF)
#define TPDAS_A1M_ADC_SYNC_DAQ_RUN_IS_ON			(TPDAS_A1M_get_adc_sync_daq_run()==TPDAS_A1M_ADC_SYNC_DAQ_RUN_ON)

#define TPDAS_A1M_GET_DAC_SYNC_DAQ_RUN				TPDAS_A1M_get_dac_sync_daq_run()
#define TPDAS_A1M_DAC_SYNC_DAQ_RUN_IS_OFF			(TPDAS_A1M_get_dac_sync_daq_run()==TPDAS_A1M_DAC_SYNC_DAQ_RUN_OFF)
#define TPDAS_A1M_DAC_SYNC_DAQ_RUN_IS_ON			(TPDAS_A1M_get_dac_sync_daq_run()==TPDAS_A1M_DAC_SYNC_DAQ_RUN_ON)

#define TPDAS_A1M_GET_ADC_RDY						TPDAS_A1M_get_adc_rdy()									
#define TPDAS_A1M_ADC_RDY_IS_OFF					(TPDAS_A1M_get_adc_rdy()==TPDAS_A1M_ADC_RDY_OFF)
#define TPDAS_A1M_ADC_RDY_IS_ON						(TPDAS_A1M_get_adc_rdy()==TPDAS_A1M_ADC_RDY_ON)

#define TPDAS_A1M_GET_ADC_SYNC_DAQ_EOS				TPDAS_A1M_get_adc_sync_daq_eos()
#define TPDAS_A1M_ADC_SYNC_DAQ_EOS_IS_OFF			(TPDAS_A1M_get_adc_sync_daq_eos()==TPDAS_A1M_ADC_SYNC_DAQ_EOS_OFF)
#define TPDAS_A1M_ADC_SYNC_DAQ_EOS_IS_ON			(TPDAS_A1M_get_adc_sync_daq_eos()==TPDAS_A1M_ADC_SYNC_DAQ_EOS_ON)

#define TPDAS_A1M_SET_ADC_SYNC_DAQ_ABORT_ON			TPDAS_A1M_set_adc_sync_daq_abort(TPDAS_A1M_ADC_SYNC_DAQ_ABORT_ON)
#define TPDAS_A1M_SET_DAC_SYNC_DAQ_ABORT_ON			TPDAS_A1M_set_dac_sync_daq_abort(TPDAS_A1M_DAC_SYNC_DAQ_ABORT_ON)
#define TPDAS_A1M_SET_ADDA_SYNC_DAQ_ABORT_ON		TPDAS_A1M_set_adda_sync_daq_abort(TPDAS_A1M_ADC_SYNC_DAQ_ABORT_ON | TPDAS_A1M_DAC_SYNC_DAQ_ABORT_ON)

#define TPDAS_A1M_GET_ADC_SYNC_DAQ_END				TPDAS_A1M_get_adc_sync_daq_end()			
#define TPDAS_A1M_ADC_SYNC_DAQ_END_IS_OFF			(TPDAS_A1M_get_adc_sync_daq_end()==TPDAS_A1M_ADC_SYNC_DAQ_END_OFF)
#define TPDAS_A1M_ADC_SYNC_DAQ_END_IS_ON			(TPDAS_A1M_get_adc_sync_daq_end()==TPDAS_A1M_ADC_SYNC_DAQ_END_ON)

#define TPDAS_A1M_GET_DAC_SYNC_DAQ_END				TPDAS_A1M_get_dac_sync_daq_end()			
#define TPDAS_A1M_DAC_SYNC_DAQ_END_IS_OFF			(TPDAS_A1M_get_dac_sync_daq_end()==TPDAS_A1M_DAC_SYNC_DAQ_END_OFF)
#define TPDAS_A1M_DAC_SYNC_DAQ_END_IS_ON			(TPDAS_A1M_get_dac_sync_daq_end()==TPDAS_A1M_DAC_SYNC_DAQ_END_ON)

#define TPDAS_A1M_GET_ADC_SYNC_DAQ_SSYNC			TPDAS_A1M_get_adc_sync_daq_ssync()			
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_IS_OFF			(TPDAS_A1M_get_adc_sync_daq_ssync()==TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_OFF)
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_IS_ON			(TPDAS_A1M_get_adc_sync_daq_ssync()==TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_ON)

#define TPDAS_A1M_GET_DAC_SYNC_DAQ_SSYNC			TPDAS_A1M_get_dac_sync_daq_ssync()			
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_IS_OFF			(TPDAS_A1M_get_dac_sync_daq_ssync()==TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_OFF)
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_IS_ON			(TPDAS_A1M_get_dac_sync_daq_ssync()==TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_ON)

			// aliases
#define TPDAS_A1M_SET_ADC_SYNC_DAQ_RUN_OFF			TPDAS_A1M_SET_ADC_SYNC_DAQ_ABORT_ON			
#define TPDAS_A1M_SET_DAC_SYNC_DAQ_RUN_OFF			TPDAS_A1M_SET_DAC_SYNC_DAQ_ABORT_ON			
#define TPDAS_A1M_SET_ADDA_SYNC_DAQ_RUN_OFF			TPDAS_A1M_SET_ADDA_SYNC_DAQ_ABORT_ON		

#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_IS_OK			TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_IS_ON						
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_IS_OK			TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_IS_ON						



	//--- CONTROL-3 register
#define TPDAS_A1M_get_daq_cntr3_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_CNTR3_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)		// unified macros for AD/DA DAQ modes TPDAS_A1M_ADDA_DAQ_MODE_... 
#define TPDAS_A1M_set_daq_cntr3_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_CNTR3_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc_sync_fifo_daq_tf_sel()	(TPDAS_A1M_get_daq_cntr3_rg() & TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_BITMASK)
#define TPDAS_A1M_set_adc_sync_fifo_daq_tf_sel(d)	TPDAS_A1M_set_daq_cntr3_rg((TPDAS_A1M_get_daq_cntr3_rg() & ~TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_BITMASK) |  (d & TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_BITMASK))

#define TPDAS_A1M_get_dac_sync_fifo_daq_tf_sel()	(TPDAS_A1M_get_daq_cntr3_rg() & TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_BITMASK)
#define TPDAS_A1M_set_dac_sync_fifo_daq_tf_sel(d)	TPDAS_A1M_set_daq_cntr3_rg((TPDAS_A1M_get_daq_cntr3_rg() & ~TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_BITMASK) |  (d & TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_BITMASK))

#define TPDAS_A1M_get_adc_sync_fifo_irt_tf_sel()	(TPDAS_A1M_get_daq_cntr3_rg() & TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK)
#define TPDAS_A1M_set_adc_sync_fifo_irt_tf_sel(d)	TPDAS_A1M_set_daq_cntr3_rg((TPDAS_A1M_get_daq_cntr3_rg() & ~TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK) |  (d & TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK))

#define TPDAS_A1M_get_dac_sync_fifo_irt_tf_sel()	(TPDAS_A1M_get_daq_cntr3_rg() & TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK)
#define TPDAS_A1M_set_dac_sync_fifo_irt_tf_sel(d)	TPDAS_A1M_set_daq_cntr3_rg((TPDAS_A1M_get_daq_cntr3_rg() & ~TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK) |  (d & TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_BITMASK))

		// direct set/get macros
#define TPDAS_A1M_SET_ADC_SYNC_FIFO_DAQ_TF_SEL_FF	TPDAS_A1M_set_adc_sync_fifo_daq_tf_sel(TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_FF)
#define TPDAS_A1M_SET_ADC_SYNC_FIFO_DAQ_TF_SEL_PAF	TPDAS_A1M_set_adc_sync_fifo_daq_tf_sel(TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_PAF)

#define TPDAS_A1M_GET_ADC_SYNC_FIFO_DAQ_TF_SEL		TPDAS_A1M_get_adc_sync_fifo_daq_tf_sel()
#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_IS_FF	(TPDAS_A1M_get_adc_sync_fifo_daq_tf_sel()==TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_FF)
#define TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_IS_PAF	(TPDAS_A1M_get_adc_sync_fifo_daq_tf_sel()==TPDAS_A1M_ADC_SYNC_FIFO_DAQ_TF_SEL_PAF)

#define TPDAS_A1M_SET_DAC_SYNC_FIFO_DAQ_TF_SEL_EF	TPDAS_A1M_set_dac_sync_fifo_daq_tf_sel(TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_EF)
#define TPDAS_A1M_SET_DAC_SYNC_FIFO_DAQ_TF_SEL_PAE	TPDAS_A1M_set_dac_sync_fifo_daq_tf_sel(TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_PAE)

#define TPDAS_A1M_GET_DAC_SYNC_FIFO_DAQ_TF_SEL		TPDAS_A1M_get_dac_sync_fifo_daq_tf_sel()
#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_IS_EF	(TPDAS_A1M_get_dac_sync_fifo_daq_tf_sel()==TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_EF)
#define TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_IS_PAE	(TPDAS_A1M_get_dac_sync_fifo_daq_tf_sel()==TPDAS_A1M_DAC_SYNC_FIFO_DAQ_TF_SEL_PAE)

#define TPDAS_A1M_SET_ADC_SYNC_FIFO_IRT_TF_SEL_EF	TPDAS_A1M_set_adc_sync_fifo_irt_tf_sel(TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_EF)
#define TPDAS_A1M_SET_ADC_SYNC_FIFO_IRT_TF_SEL_PAE	TPDAS_A1M_set_adc_sync_fifo_irt_tf_sel(TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_PAE)

#define TPDAS_A1M_GET_ADC_SYNC_FIFO_IRT_TF_SEL		TPDAS_A1M_get_adc_sync_fifo_irt_tf_sel()	
#define TPDAS_A1M_ADC_SYNC_FIFO_IRT_TF_SEL_IS_EF	(TPDAS_A1M_get_adc_sync_fifo_irt_tf_sel()==TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_EF)
#define TPDAS_A1M_ADC_SYNC_FIFO_IRT_TF_SEL_IS_PAE	(TPDAS_A1M_get_adc_sync_fifo_irt_tf_sel()==TPDAS_A1M_ADC_SYNC_FIFO_DAQ_IRT_TF_SEL_PAE)

#define TPDAS_A1M_SET_DAC_SYNC_FIFO_IRT_TF_SEL_FF	TPDAS_A1M_set_dac_sync_fifo_irt_tf_sel(TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_FF)
#define TPDAS_A1M_SET_DAC_SYNC_FIFO_IRT_TF_SEL_PAF	TPDAS_A1M_set_dac_sync_fifo_irt_tf_sel(TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_PAF)

#define TPDAS_A1M_GET_DAC_SYNC_FIFO_IRT_TF_SEL		TPDAS_A1M_get_dac_sync_fifo_irt_tf_sel()	
#define TPDAS_A1M_DAC_SYNC_FIFO_IRT_TF_SEL_IS_FF	(TPDAS_A1M_get_dac_sync_fifo_irt_tf_sel()==TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_FF)
#define TPDAS_A1M_DAC_SYNC_FIFO_IRT_TF_SEL_IS_PAF	(TPDAS_A1M_get_dac_sync_fifo_irt_tf_sel()==TPDAS_A1M_DAC_SYNC_FIFO_DAQ_IRT_TF_SEL_PAF)



	//-- DAQ SYNC Register
#define TPDAS_A1M_get_daq_sync_rg()					((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_SYNC_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_daq_sync_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAQ_SYNC_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_fs_sel()						(TPDAS_A1M_get_daq_sync_rg() & TPDAS_A1M_FS_SEL_BITMASK)
#define TPDAS_A1M_set_fs_sel(d)						TPDAS_A1M_set_daq_sync_rg((TPDAS_A1M_get_daq_sync_rg() & ~TPDAS_A1M_FS_SEL_BITMASK) |  (d & TPDAS_A1M_FS_SEL_BITMASK))

#define TPDAS_A1M_get_adc_async_daq_acq_en()		(TPDAS_A1M_get_daq_sync_rg() & TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_BITMASK)
#define TPDAS_A1M_set_adc_async_daq_acq_en(d)		TPDAS_A1M_set_daq_sync_rg((TPDAS_A1M_get_daq_sync_rg() & ~TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_BITMASK) |  (d & TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_BITMASK))

#define TPDAS_A1M_get_adc_sync_no_fifo_daq_sccl_dly_en()		(TPDAS_A1M_get_daq_sync_rg() & TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_BITMASK)
#define TPDAS_A1M_set_adc_sync_no_fifo_daq_sccl_dly_en(d)		TPDAS_A1M_set_daq_sync_rg((TPDAS_A1M_get_daq_sync_rg() & ~TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_BITMASK))

#define TPDAS_A1M_get_adc_sync_daq_ssync_sel()		(TPDAS_A1M_get_daq_sync_rg() & TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_BITMASK)
#define TPDAS_A1M_set_adc_sync_daq_ssync_sel(d)		TPDAS_A1M_set_daq_sync_rg((TPDAS_A1M_get_daq_sync_rg() & ~TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_BITMASK) |  (d & TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_BITMASK))

#define TPDAS_A1M_get_dac_sync_daq_ssync_sel()		(TPDAS_A1M_get_daq_sync_rg() & TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_BITMASK)
#define TPDAS_A1M_set_dac_sync_daq_ssync_sel(d)		TPDAS_A1M_set_daq_sync_rg((TPDAS_A1M_get_daq_sync_rg() & ~TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_BITMASK) |  (d & TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_BITMASK))

#define TPDAS_A1M_get_adda_sync_daq_xsync_mode()	(TPDAS_A1M_get_daq_sync_rg() & TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_BITMASK)
#define TPDAS_A1M_set_adda_sync_daq_xsync_mode(d)	TPDAS_A1M_set_daq_sync_rg((TPDAS_A1M_get_daq_sync_rg() & ~TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_BITMASK) |  (d & TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_BITMASK))

#define TPDAS_A1M_get_adc_high_speed_en()			(TPDAS_A1M_get_daq_sync_rg() & TPDAS_A1M_ADC_HIGH_SPEED_EN_BITMASK)
#define TPDAS_A1M_set_adc_high_speed_en(d)			TPDAS_A1M_set_daq_sync_rg((TPDAS_A1M_get_daq_sync_rg() & ~TPDAS_A1M_ADC_HIGH_SPEED_EN_BITMASK) |  (d & TPDAS_A1M_ADC_HIGH_SPEED_EN_BITMASK))

		//direct set/get macros
#define TPDAS_A1M_SET_FS_SEL_PFG								TPDAS_A1M_set_fs_sel(TPDAS_A1M_FS_SEL_PFG)
#define TPDAS_A1M_SET_FS_SEL_XFS								TPDAS_A1M_set_fs_sel(TPDAS_A1M_FS_SEL_XFS)
#define TPDAS_A1M_SET_FS_SEL_PXTM0								TPDAS_A1M_set_fs_sel(TPDAS_A1M_FS_SEL_PXTM0)
#define TPDAS_A1M_SET_FS_SEL_PXTM1								TPDAS_A1M_set_fs_sel(TPDAS_A1M_FS_SEL_PXTM1)

#define TPDAS_A1M_GET_FS_SEL									TPDAS_A1M_get_fs_sel()						
#define TPDAS_A1M_FS_SEL_IS_PFG									(TPDAS_A1M_get_fs_sel()==TPDAS_A1M_FS_SEL_PFG)
#define TPDAS_A1M_FS_SEL_IS_XFS									(TPDAS_A1M_get_fs_sel()==TPDAS_A1M_FS_SEL_XFS)
#define TPDAS_A1M_FS_SEL_IS_PXTM0								(TPDAS_A1M_get_fs_sel()==TPDAS_A1M_FS_SEL_PXTM0)
#define TPDAS_A1M_FS_SEL_IS_PXTM1								(TPDAS_A1M_get_fs_sel()==TPDAS_A1M_FS_SEL_PXTM1)

#define TPDAS_A1M_SET_ADC_SYNC_DAQ_SSYNC_SEL_SFT				TPDAS_A1M_set_adc_sync_daq_ssync_sel(TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_SFT)
#define TPDAS_A1M_SET_ADC_SYNC_DAQ_SSYNC_SEL_XSYNC				TPDAS_A1M_set_adc_sync_daq_ssync_sel(TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_XSYNC)

#define TPDAS_A1M_GET_ADC_SYNC_DAQ_SSYNC_SEL					TPDAS_A1M_get_adc_sync_daq_ssync_sel()		
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_IS_SFT					(TPDAS_A1M_get_adc_sync_daq_ssync_sel()==TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_SFT)
#define TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_IS_XSYNC				(TPDAS_A1M_get_adc_sync_daq_ssync_sel()==TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_XSYNC)

#define TPDAS_A1M_SET_DAC_SYNC_DAQ_SSYNC_SEL_SFT				TPDAS_A1M_set_dac_sync_daq_ssync_sel(TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_SFT)
#define TPDAS_A1M_SET_DAC_SYNC_DAQ_SSYNC_SEL_XSYNC				TPDAS_A1M_set_dac_sync_daq_ssync_sel(TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_XSYNC)

#define TPDAS_A1M_GET_DAC_SYNC_DAQ_SSYNC_SEL					TPDAS_A1M_get_dac_sync_daq_ssync_sel()		
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_IS_SFT					(TPDAS_A1M_get_dac_sync_daq_ssync_sel()==TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_SFT)
#define TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_IS_XSYNC				(TPDAS_A1M_get_dac_sync_daq_ssync_sel()==TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_XSYNC)

#define TPDAS_A1M_SET_ADDA_SYNC_DAQ_XSYNC_MODE_LEVEL			TPDAS_A1M_set_adda_sync_daq_xsync_mode(TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_LEVEL)
#define TPDAS_A1M_SET_ADDA_SYNC_DAQ_XSYNC_MODE_EDGE				TPDAS_A1M_set_adda_sync_daq_xsync_mode(TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_EDGE)

#define TPDAS_A1M_GET_ADDA_SYNC_DAQ_XSYNC_MODE					TPDAS_A1M_get_adda_sync_daq_xsync_mode()	
#define TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_IS_LEVEL				(TPDAS_A1M_get_adda_sync_daq_xsync_mode()==TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_LEVEL)
#define TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_IS_EDGE				(TPDAS_A1M_get_adda_sync_daq_xsync_mode()==TPDAS_A1M_ADDA_SYNC_DAQ_XSYNC_MODE_EDGE)

#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_ACQ_EN_OFF					TPDAS_A1M_set_adc_async_daq_acq_en(TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_OFF)
#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_ACQ_EN_ON					TPDAS_A1M_set_adc_async_daq_acq_en(TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_ON)

#define TPDAS_A1M_GET_ADC_ASYNC_DAQ_ACQ_EN						TPDAS_A1M_get_adc_async_daq_acq_en()		
#define TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_IS_OFF					(TPDAS_A1M_get_adc_async_daq_acq_en()==TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_OFF)
#define TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_IS_ON					(TPDAS_A1M_get_adc_async_daq_acq_en()==TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_ON)

#define TPDAS_A1M_SET_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_OFF		TPDAS_A1M_set_adc_sync_no_fifo_daq_sccl_dly_en(TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_OFF)
#define TPDAS_A1M_SET_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_ON		TPDAS_A1M_set_adc_sync_no_fifo_daq_sccl_dly_en(TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_ON)

#define TPDAS_A1M_GET_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN			TPDAS_A1M_get_adc_sync_no_fifo_daq_sccl_dly_en()
#define TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_IS_OFF		(TPDAS_A1M_get_adc_sync_no_fifo_daq_sccl_dly_en()==TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_OFF)
#define TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_IS_ON		(TPDAS_A1M_get_adc_sync_no_fifo_daq_sccl_dly_en()==TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_ON)

#define TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_OFF						TPDAS_A1M_set_adc_high_speed_en(TPDAS_A1M_ADC_HIGH_SPEED_EN_OFF)
#define TPDAS_A1M_SET_ADC_HIGH_SPEED_EN_ON						TPDAS_A1M_set_adc_high_speed_en(TPDAS_A1M_ADC_HIGH_SPEED_EN_ON)

#define TPDAS_A1M_GET_ADC_HIGH_SPEED_EN							TPDAS_A1M_get_adc_high_speed_en()
#define TPDAS_A1M_ADC_HIGH_SPEED_EN_IS_OFF						(TPDAS_A1M_get_adc_high_speed_en()==TPDAS_A1M_ADC_HIGH_SPEED_EN_OFF)
#define TPDAS_A1M_ADC_HIGH_SPEED_EN_IS_ON						(TPDAS_A1M_get_adc_high_speed_en()==TPDAS_A1M_ADC_HIGH_SPEED_EN_ON)

		// unified macros with TPDAS_A1M_FS_SEL_...
#define TPDAS_A1M_uget_fs_sel()									TPDAS_A1M_get_fs_sel()
#define TPDAS_A1M_uset_fs_sel(d)								TPDAS_A1M_set_fs_sel(d)

		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_adc_async_daq_acq_en()					(TPDAS_A1M_get_adc_async_daq_acq_en()>>TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_async_daq_acq_en(d)					TPDAS_A1M_set_adc_async_daq_acq_en((d)<<TPDAS_A1M_ADC_ASYNC_DAQ_ACQ_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sync_no_fifo_daq_sccl_dly_en()		(TPDAS_A1M_get_adc_sync_no_fifo_daq_sccl_dly_en()>>TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sync_no_fifo_daq_sccl_dly_en(d)		TPDAS_A1M_set_adc_sync_no_fifo_daq_sccl_dly_en((d)<<TPDAS_A1M_ADC_SYNC_NO_FIFO_DAQ_SCCL_DLY_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_high_speed_en()						(TPDAS_A1M_get_adc_high_speed_en()>>TPDAS_A1M_ADC_HIGH_SPEED_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_high_speed_en(d)						TPDAS_A1M_set_adc_high_speed_en((d)<<TPDAS_A1M_ADC_HIGH_SPEED_EN_BITSHIFT)			

		// unified macros with TPDAS_A1M_ADDA_SYNC_DAQ_SSYNC_SEL_SFT/TPDAS_A1M_ADDA_SYNC_DAQ_SSYNC_SEL_XSYNC
#define TPDAS_A1M_uget_adc_sync_daq_ssync_sel()					(TPDAS_A1M_get_adc_sync_daq_ssync_sel()>>TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_BITSHIFT)
#define TPDAS_A1M_uset_adc_sync_daq_ssync_sel(d)				TPDAS_A1M_set_adc_sync_daq_ssync_sel((d)<<TPDAS_A1M_ADC_SYNC_DAQ_SSYNC_SEL_BITSHIFT)

#define TPDAS_A1M_uget_dac_sync_daq_ssync_sel()					(TPDAS_A1M_get_dac_sync_daq_ssync_sel()>>TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_BITSHIFT)
#define TPDAS_A1M_uset_dac_sync_daq_ssync_sel(d)				TPDAS_A1M_set_dac_sync_daq_ssync_sel((d)<<TPDAS_A1M_DAC_SYNC_DAQ_SSYNC_SEL_BITSHIFT)

	


	//-- ADC CNF Register
#define TPDAS_A1M_get_adc_cnf_rg()								((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_CNF_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_adc_cnf_rg(d)								(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_CNF_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc0_en()									(TPDAS_A1M_get_adc_cnf_rg() & TPDAS_A1M_ADC0_EN_BITMASK)
#define TPDAS_A1M_set_adc0_en(d)								TPDAS_A1M_set_adc_cnf_rg((TPDAS_A1M_get_adc_cnf_rg() & ~TPDAS_A1M_ADC0_EN_BITMASK) |  (d & TPDAS_A1M_ADC0_EN_BITMASK))

#define TPDAS_A1M_get_adc1_en()									(TPDAS_A1M_get_adc_cnf_rg() & TPDAS_A1M_ADC1_EN_BITMASK)
#define TPDAS_A1M_set_adc1_en(d)								TPDAS_A1M_set_adc_cnf_rg((TPDAS_A1M_get_adc_cnf_rg() & ~TPDAS_A1M_ADC1_EN_BITMASK) |  (d & TPDAS_A1M_ADC1_EN_BITMASK))

#define TPDAS_A1M_get_adc2_en()									(TPDAS_A1M_get_adc_cnf_rg() & TPDAS_A1M_ADC2_EN_BITMASK)
#define TPDAS_A1M_set_adc2_en(d)								TPDAS_A1M_set_adc_cnf_rg((TPDAS_A1M_get_adc_cnf_rg() & ~TPDAS_A1M_ADC2_EN_BITMASK) |  (d & TPDAS_A1M_ADC2_EN_BITMASK))

#define TPDAS_A1M_get_adc3_en()									(TPDAS_A1M_get_adc_cnf_rg() & TPDAS_A1M_ADC3_EN_BITMASK)
#define TPDAS_A1M_set_adc3_en(d)								TPDAS_A1M_set_adc_cnf_rg((TPDAS_A1M_get_adc_cnf_rg() & ~TPDAS_A1M_ADC3_EN_BITMASK) |  (d & TPDAS_A1M_ADC3_EN_BITMASK))

#define TPDAS_A1M_get_adc_pool_id()								(TPDAS_A1M_get_adc_cnf_rg() & TPDAS_A1M_ADC_POOL_ID_BITMASK)

		// direct set/get macros
#define TPDAS_A1M_SET_ADC0_EN_OFF								TPDAS_A1M_set_adc0_en(TPDAS_A1M_ADC0_EN_OFF)								
#define TPDAS_A1M_SET_ADC0_EN_ON								TPDAS_A1M_set_adc0_en(TPDAS_A1M_ADC0_EN_ON)								

#define TPDAS_A1M_GET_ADC0_EN									TPDAS_A1M_get_adc0_en()									
#define TPDAS_A1M_ADC0_EN_IS_OFF								(TPDAS_A1M_get_adc0_en()==TPDAS_A1M_ADC0_EN_OFF)
#define TPDAS_A1M_ADC0_EN_IS_ON									(TPDAS_A1M_get_adc0_en()==TPDAS_A1M_ADC0_EN_ON)

#define TPDAS_A1M_SET_ADC1_EN_OFF								TPDAS_A1M_set_adc1_en(TPDAS_A1M_ADC1_EN_OFF)								
#define TPDAS_A1M_SET_ADC1_EN_ON								TPDAS_A1M_set_adc1_en(TPDAS_A1M_ADC1_EN_ON)								

#define TPDAS_A1M_GET_ADC1_EN									TPDAS_A1M_get_adc1_en()									
#define TPDAS_A1M_ADC1_EN_IS_OFF								(TPDAS_A1M_get_adc1_en()==TPDAS_A1M_ADC1_EN_OFF)
#define TPDAS_A1M_ADC1_EN_IS_ON									(TPDAS_A1M_get_adc1_en()==TPDAS_A1M_ADC1_EN_ON)

#define TPDAS_A1M_SET_ADC2_EN_OFF								TPDAS_A1M_set_adc2_en(TPDAS_A1M_ADC2_EN_OFF)								
#define TPDAS_A1M_SET_ADC2_EN_ON								TPDAS_A1M_set_adc2_en(TPDAS_A1M_ADC2_EN_ON)								

#define TPDAS_A1M_GET_ADC2_EN									TPDAS_A1M_get_adc2_en()									
#define TPDAS_A1M_ADC2_EN_IS_OFF								(TPDAS_A1M_get_adc2_en()==TPDAS_A1M_ADC2_EN_OFF)
#define TPDAS_A1M_ADC2_EN_IS_ON									(TPDAS_A1M_get_adc2_en()==TPDAS_A1M_ADC2_EN_ON)

#define TPDAS_A1M_SET_ADC3_EN_OFF								TPDAS_A1M_set_adc3_en(TPDAS_A1M_ADC3_EN_OFF)								
#define TPDAS_A1M_SET_ADC3_EN_ON								TPDAS_A1M_set_adc3_en(TPDAS_A1M_ADC3_EN_ON)	

#define TPDAS_A1M_GET_ADC3_EN									TPDAS_A1M_get_adc3_en()									
#define TPDAS_A1M_ADC3_EN_IS_OFF								(TPDAS_A1M_get_adc3_en()==TPDAS_A1M_ADC3_EN_OFF)
#define TPDAS_A1M_ADC3_EN_IS_ON									(TPDAS_A1M_get_adc3_en()==TPDAS_A1M_ADC3_EN_ON)

#define TPDAS_A1M_GET_ADC_POOL_ID								TPDAS_A1M_get_adc_pool_id()
#define TPDAS_A1M_ADC_POOL_ID_IS_X2								(TPDAS_A1M_get_adc_pool_id()==TPDAS_A1M_ADC_POOL_ID_X2)
#define TPDAS_A1M_ADC_POOL_ID_IS_X4								(TPDAS_A1M_get_adc_pool_id()==TPDAS_A1M_ADC_POOL_ID_X4)

		// combined macros
#define TPDAS_A1M_SET_ADC_ALL_EN_OFF							TPDAS_A1M_set_adc_cnf_rg(0)
#define TPDAS_A1M_SET_ADC_ALL_EN_ON								TPDAS_A1M_set_adc_cnf_rg(0xf)


		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_adc0_en()								TPDAS_A1M_get_adc0_en()
#define TPDAS_A1M_uset_adc0_en(d)								TPDAS_A1M_set_adc0_en(d)

#define TPDAS_A1M_uget_adc1_en()								(TPDAS_A1M_get_adc1_en()>>TPDAS_A1M_ADC1_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc1_en(d)								TPDAS_A1M_set_adc1_en((d)<<TPDAS_A1M_ADC1_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc2_en()								(TPDAS_A1M_get_adc2_en()>>TPDAS_A1M_ADC2_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc2_en(d)								TPDAS_A1M_set_adc2_en((d)<<TPDAS_A1M_ADC2_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc3_en()								(TPDAS_A1M_get_adc3_en()>>TPDAS_A1M_ADC3_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc3_en(d)								TPDAS_A1M_set_adc3_en((d)<<TPDAS_A1M_ADC3_EN_BITSHIFT)


	//-- DAC CNF Register
#define TPDAS_A1M_get_dac_cnf_rg()								((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC_CNF_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_dac_cnf_rg(d)								(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC_CNF_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_dac0_en()									(TPDAS_A1M_get_dac_cnf_rg() & TPDAS_A1M_DAC0_EN_BITMASK)
#define TPDAS_A1M_set_dac0_en(d)								TPDAS_A1M_set_dac_cnf_rg((TPDAS_A1M_get_dac_cnf_rg() & ~TPDAS_A1M_DAC0_EN_BITMASK) |  (d & TPDAS_A1M_DAC0_EN_BITMASK))

#define TPDAS_A1M_get_dac1_en()									(TPDAS_A1M_get_dac_cnf_rg() & TPDAS_A1M_DAC1_EN_BITMASK)
#define TPDAS_A1M_set_dac1_en(d)								TPDAS_A1M_set_dac_cnf_rg((TPDAS_A1M_get_dac_cnf_rg() & ~TPDAS_A1M_DAC1_EN_BITMASK) |  (d & TPDAS_A1M_DAC1_EN_BITMASK))

#define TPDAS_A1M_get_dac2_en()									(TPDAS_A1M_get_dac_cnf_rg() & TPDAS_A1M_DAC2_EN_BITMASK)
#define TPDAS_A1M_set_dac2_en(d)								TPDAS_A1M_set_dac_cnf_rg((TPDAS_A1M_get_dac_cnf_rg() & ~TPDAS_A1M_DAC2_EN_BITMASK) |  (d & TPDAS_A1M_DAC2_EN_BITMASK))

#define TPDAS_A1M_get_dac3_en()									(TPDAS_A1M_get_dac_cnf_rg() & TPDAS_A1M_DAC3_EN_BITMASK)
#define TPDAS_A1M_set_dac3_en(d)								TPDAS_A1M_set_dac_cnf_rg((TPDAS_A1M_get_dac_cnf_rg() & ~TPDAS_A1M_DAC3_EN_BITMASK) |  (d & TPDAS_A1M_DAC3_EN_BITMASK))

#define TPDAS_A1M_get_dac_pool_id()								(TPDAS_A1M_get_dac_cnf_rg() & TPDAS_A1M_DAC_POOL_ID_BITMASK)

		// direct set/get macros
#define TPDAS_A1M_SET_DAC0_EN_OFF								TPDAS_A1M_set_dac0_en(TPDAS_A1M_DAC0_EN_OFF)								
#define TPDAS_A1M_SET_DAC0_EN_ON								TPDAS_A1M_set_dac0_en(TPDAS_A1M_DAC0_EN_ON)								

#define TPDAS_A1M_GET_DAC0_EN									TPDAS_A1M_get_dac0_en()									
#define TPDAS_A1M_DAC0_EN_IS_OFF								(TPDAS_A1M_get_dac0_en()==TPDAS_A1M_DAC0_EN_OFF)
#define TPDAS_A1M_DAC0_EN_IS_ON									(TPDAS_A1M_get_dac0_en()==TPDAS_A1M_DAC0_EN_ON)

#define TPDAS_A1M_SET_DAC1_EN_OFF								TPDAS_A1M_set_dac1_en(TPDAS_A1M_DAC1_EN_OFF)								
#define TPDAS_A1M_SET_DAC1_EN_ON								TPDAS_A1M_set_dac1_en(TPDAS_A1M_DAC1_EN_ON)								

#define TPDAS_A1M_GET_DAC1_EN									TPDAS_A1M_get_dac1_en()									
#define TPDAS_A1M_DAC1_EN_IS_OFF								(TPDAS_A1M_get_dac1_en()==TPDAS_A1M_DAC1_EN_OFF)
#define TPDAS_A1M_DAC1_EN_IS_ON									(TPDAS_A1M_get_dac1_en()==TPDAS_A1M_DAC1_EN_ON)

#define TPDAS_A1M_SET_DAC2_EN_OFF								TPDAS_A1M_set_dac2_en(TPDAS_A1M_DAC2_EN_OFF)								
#define TPDAS_A1M_SET_DAC2_EN_ON								TPDAS_A1M_set_dac2_en(TPDAS_A1M_DAC2_EN_ON)								

#define TPDAS_A1M_GET_DAC2_EN									TPDAS_A1M_get_dac2_en()									
#define TPDAS_A1M_DAC2_EN_IS_OFF								(TPDAS_A1M_get_dac2_en()==TPDAS_A1M_DAC2_EN_OFF)
#define TPDAS_A1M_DAC2_EN_IS_ON									(TPDAS_A1M_get_dac2_en()==TPDAS_A1M_DAC2_EN_ON)

#define TPDAS_A1M_SET_DAC3_EN_OFF								TPDAS_A1M_set_dac3_en(TPDAS_A1M_DAC3_EN_OFF)								
#define TPDAS_A1M_SET_DAC3_EN_ON								TPDAS_A1M_set_dac3_en(TPDAS_A1M_DAC3_EN_ON)	

#define TPDAS_A1M_GET_DAC3_EN									TPDAS_A1M_get_dac3_en()									
#define TPDAS_A1M_DAC3_EN_IS_OFF								(TPDAS_A1M_get_dac3_en()==TPDAS_A1M_DAC3_EN_OFF)
#define TPDAS_A1M_DAC3_EN_IS_ON									(TPDAS_A1M_get_dac3_en()==TPDAS_A1M_DAC3_EN_ON)

#define TPDAS_A1M_GET_DAC_POOL_ID								TPDAS_A1M_get_dac_pool_id()
#define TPDAS_A1M_DAC_POOL_ID_IS_X2								(TPDAS_A1M_get_dac_pool_id()==TPDAS_A1M_DAC_POOL_ID_X2)
#define TPDAS_A1M_DAC_POOL_ID_IS_X4								(TPDAS_A1M_get_dac_pool_id()==TPDAS_A1M_DAC_POOL_ID_X4)

		// combined macros
#define TPDAS_A1M_SET_DAC_ALL_EN_OFF							TPDAS_A1M_set_dac_cnf_rg(0)
#define TPDAS_A1M_SET_DAC_ALL_EN_ON								TPDAS_A1M_set_dac_cnf_rg(0xf)


		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_dac0_en()								TPDAS_A1M_get_dac0_en()
#define TPDAS_A1M_uset_dac0_en(d)								TPDAS_A1M_set_dac0_en(d)

#define TPDAS_A1M_uget_dac1_en()								(TPDAS_A1M_get_dac1_en()>>TPDAS_A1M_DAC1_EN_BITSHIFT)
#define TPDAS_A1M_uset_dac1_en(d)								TPDAS_A1M_set_dac1_en((d)<<TPDAS_A1M_DAC1_EN_BITSHIFT)

#define TPDAS_A1M_uget_dac2_en()								(TPDAS_A1M_get_dac2_en()>>TPDAS_A1M_DAC2_EN_BITSHIFT)
#define TPDAS_A1M_uset_dac2_en(d)								TPDAS_A1M_set_dac2_en((d)<<TPDAS_A1M_DAC2_EN_BITSHIFT)

#define TPDAS_A1M_uget_dac3_en()								(TPDAS_A1M_get_dac3_en()>>TPDAS_A1M_DAC3_EN_BITSHIFT)
#define TPDAS_A1M_uset_dac3_en(d)								TPDAS_A1M_set_dac3_en((d)<<TPDAS_A1M_DAC3_EN_BITSHIFT)


	//-- ADC IMUX/SCCL CNF-#1/#2 Registers
#define TPDAS_A1M_get_adc_imux_cnf1_rg()						((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_IMUX_CNF1_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_adc_imux_cnf1_rg(d)						(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_IMUX_CNF1_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_get_adc_imux_cnf2_rg()						((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_IMUX_CNF2_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_adc_imux_cnf2_rg(d)						(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_IMUX_CNF2_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc_sccl0_imux_ch_sel()					(TPDAS_A1M_get_adc_imux_cnf1_rg() & TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_BITMASK)
#define TPDAS_A1M_set_adc_sccl0_imux_ch_sel(d)					TPDAS_A1M_set_adc_imux_cnf1_rg((TPDAS_A1M_get_adc_imux_cnf1_rg() & ~TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_BITMASK))

#define TPDAS_A1M_get_adc_sccl0_imux_diff_en()					(TPDAS_A1M_get_adc_imux_cnf1_rg() & TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl0_imux_diff_en(d)					TPDAS_A1M_set_adc_imux_cnf1_rg((TPDAS_A1M_get_adc_imux_cnf1_rg() & ~TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITMASK))

#define TPDAS_A1M_get_adc_sccl0_eos_en()						(TPDAS_A1M_get_adc_imux_cnf1_rg() & TPDAS_A1M_ADC_SCCL0_EOS_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl0_eos_en(d)						TPDAS_A1M_set_adc_imux_cnf1_rg((TPDAS_A1M_get_adc_imux_cnf1_rg() & ~TPDAS_A1M_ADC_SCCL0_EOS_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL0_EOS_EN_BITMASK))

#define TPDAS_A1M_get_adc_sccl1_imux_ch_sel()					(TPDAS_A1M_get_adc_imux_cnf1_rg() & TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_BITMASK)
#define TPDAS_A1M_set_adc_sccl1_imux_ch_sel(d)					TPDAS_A1M_set_adc_imux_cnf1_rg((TPDAS_A1M_get_adc_imux_cnf1_rg() & ~TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_BITMASK))

#define TPDAS_A1M_get_adc_sccl1_imux_diff_en()					(TPDAS_A1M_get_adc_imux_cnf1_rg() & TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl1_imux_diff_en(d)					TPDAS_A1M_set_adc_imux_cnf1_rg((TPDAS_A1M_get_adc_imux_cnf1_rg() & ~TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_BITMASK))

#define TPDAS_A1M_get_adc_sccl1_eos_en()						(TPDAS_A1M_get_adc_imux_cnf1_rg() & TPDAS_A1M_ADC_SCCL1_EOS_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl1_eos_en(d)						TPDAS_A1M_set_adc_imux_cnf1_rg((TPDAS_A1M_get_adc_imux_cnf1_rg() & ~TPDAS_A1M_ADC_SCCL1_EOS_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL1_EOS_EN_BITMASK))

#define TPDAS_A1M_get_adc_sccl2_imux_ch_sel()					(TPDAS_A1M_get_adc_imux_cnf2_rg() & TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_BITMASK)
#define TPDAS_A1M_set_adc_sccl2_imux_ch_sel(d)					TPDAS_A1M_set_adc_imux_cnf2_rg((TPDAS_A1M_get_adc_imux_cnf2_rg() & ~TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_BITMASK))

#define TPDAS_A1M_get_adc_sccl2_imux_diff_en()					(TPDAS_A1M_get_adc_imux_cnf2_rg() & TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl2_imux_diff_en(d)					TPDAS_A1M_set_adc_imux_cnf2_rg((TPDAS_A1M_get_adc_imux_cnf2_rg() & ~TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_BITMASK))

#define TPDAS_A1M_get_adc_sccl2_eos_en()						(TPDAS_A1M_get_adc_imux_cnf2_rg() & TPDAS_A1M_ADC_SCCL2_EOS_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl2_eos_en(d)						TPDAS_A1M_set_adc_imux_cnf2_rg((TPDAS_A1M_get_adc_imux_cnf2_rg() & ~TPDAS_A1M_ADC_SCCL2_EOS_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL2_EOS_EN_BITMASK))

#define TPDAS_A1M_get_adc_sccl3_imux_ch_sel()					(TPDAS_A1M_get_adc_imux_cnf2_rg() & TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_BITMASK)
#define TPDAS_A1M_set_adc_sccl3_imux_ch_sel(d)					TPDAS_A1M_set_adc_imux_cnf2_rg((TPDAS_A1M_get_adc_imux_cnf2_rg() & ~TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_BITMASK))

#define TPDAS_A1M_get_adc_sccl3_imux_diff_en()					(TPDAS_A1M_get_adc_imux_cnf2_rg() & TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl3_imux_diff_en(d)					TPDAS_A1M_set_adc_imux_cnf2_rg((TPDAS_A1M_get_adc_imux_cnf2_rg() & ~TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_BITMASK))

#define TPDAS_A1M_get_adc_sccl3_eos_en()						(TPDAS_A1M_get_adc_imux_cnf2_rg() & TPDAS_A1M_ADC_SCCL3_EOS_EN_BITMASK)
#define TPDAS_A1M_set_adc_sccl3_eos_en(d)						TPDAS_A1M_set_adc_imux_cnf2_rg((TPDAS_A1M_get_adc_imux_cnf2_rg() & ~TPDAS_A1M_ADC_SCCL3_EOS_EN_BITMASK) |  (d & TPDAS_A1M_ADC_SCCL3_EOS_EN_BITMASK))

		// aliases for ADC IMUX control in ADC ASYNC MODE
#define TPDAS_A1M_get_adc_async_daq_imux_cnf_rg()				TPDAS_A1M_get_adc_imux_cnf1_rg()
#define TPDAS_A1M_set_adc_async_daq_imux_cnf_rg(d)				TPDAS_A1M_set_adc_imux_cnf1_rg(d)

#define TPDAS_A1M_get_adc_async_daq_imux_ch_sel()				TPDAS_A1M_get_adc_sccl0_imux_ch_sel()
#define TPDAS_A1M_set_adc_async_daq_imux_ch_sel(d)				TPDAS_A1M_set_adc_sccl0_imux_ch_sel(d)

#define TPDAS_A1M_get_adc_async_daq_imux_diff_en()				TPDAS_A1M_get_adc_sccl0_imux_diff_en()
#define TPDAS_A1M_set_adc_async_daq_imux_diff_en(d)				TPDAS_A1M_set_adc_sccl0_imux_diff_en(d)

		// direct set/get macros
#define TPDAS_A1M_SET_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12			TPDAS_A1M_set_adc_sccl0_imux_ch_sel(TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12)					
#define TPDAS_A1M_SET_ADC_SCCL0_IMUX_CH_SEL_1_5_9_13			TPDAS_A1M_set_adc_sccl0_imux_ch_sel(TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_1_5_9_13)					
#define TPDAS_A1M_SET_ADC_SCCL0_IMUX_CH_SEL_2_6_10_14			TPDAS_A1M_set_adc_sccl0_imux_ch_sel(TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_2_6_10_14)					
#define TPDAS_A1M_SET_ADC_SCCL0_IMUX_CH_SEL_3_7_11_15			TPDAS_A1M_set_adc_sccl0_imux_ch_sel(TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_3_7_11_15)					

#define TPDAS_A1M_GET_ADC_SCCL0_IMUX_CH_SEL						TPDAS_A1M_get_adc_sccl0_imux_ch_sel()
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_IS_0_4_8_12				(TPDAS_A1M_get_adc_sccl0_imux_ch_sel()==TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_0_4_8_12)
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_IS_1_5_9_13				(TPDAS_A1M_get_adc_sccl0_imux_ch_sel()==TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_1_5_9_13)
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_IS_2_6_10_14			(TPDAS_A1M_get_adc_sccl0_imux_ch_sel()==TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_2_6_10_14)
#define TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_IS_3_7_11_15			(TPDAS_A1M_get_adc_sccl0_imux_ch_sel()==TPDAS_A1M_ADC_SCCL0_IMUX_CH_SEL_3_7_11_15)

#define TPDAS_A1M_SET_ADC_SCCL0_IMUX_DIFF_EN_OFF				TPDAS_A1M_set_adc_sccl0_imux_diff_en(TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_OFF)					
#define TPDAS_A1M_SET_ADC_SCCL0_IMUX_DIFF_EN_ON					TPDAS_A1M_set_adc_sccl0_imux_diff_en(TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_ON)					

#define TPDAS_A1M_GET_ADC_SCCL0_IMUX_DIFF_EN					TPDAS_A1M_get_adc_sccl0_imux_diff_en()
#define TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_IS_OFF					(TPDAS_A1M_get_adc_sccl0_imux_diff_en()==TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_OFF)
#define TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_IS_ON					(TPDAS_A1M_get_adc_sccl0_imux_diff_en()==TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_ON)

#define TPDAS_A1M_SET_ADC_SCCL0_EOS_EN_OFF						TPDAS_A1M_set_adc_sccl0_eos_en(TPDAS_A1M_ADC_SCCL0_EOS_EN_OFF)
#define TPDAS_A1M_SET_ADC_SCCL0_EOS_EN_ON						TPDAS_A1M_set_adc_sccl0_eos_en(TPDAS_A1M_ADC_SCCL0_EOS_EN_ON)

#define TPDAS_A1M_GET_ADC_SCCL0_EOS_EN							TPDAS_A1M_get_adc_sccl0_eos_en()
#define TPDAS_A1M_ADC_SCCL0_EOS_EN_IS_OFF						(TPDAS_A1M_get_adc_sccl0_eos_en()==TPDAS_A1M_ADC_SCCL0_EOS_EN_OFF)
#define TPDAS_A1M_ADC_SCCL0_EOS_EN_IS_ON						(TPDAS_A1M_get_adc_sccl0_eos_en()==TPDAS_A1M_ADC_SCCL0_EOS_EN_ON)

#define TPDAS_A1M_SET_ADC_SCCL1_IMUX_CH_SEL_0_4_8_12			TPDAS_A1M_set_adc_sccl1_imux_ch_sel(TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_0_4_8_12)					
#define TPDAS_A1M_SET_ADC_SCCL1_IMUX_CH_SEL_1_5_9_13			TPDAS_A1M_set_adc_sccl1_imux_ch_sel(TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_1_5_9_13)					
#define TPDAS_A1M_SET_ADC_SCCL1_IMUX_CH_SEL_2_6_10_14			TPDAS_A1M_set_adc_sccl1_imux_ch_sel(TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_2_6_10_14)					
#define TPDAS_A1M_SET_ADC_SCCL1_IMUX_CH_SEL_3_7_11_15			TPDAS_A1M_set_adc_sccl1_imux_ch_sel(TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_3_7_11_15)					

#define TPDAS_A1M_GET_ADC_SCCL1_IMUX_CH_SEL						TPDAS_A1M_get_adc_sccl1_imux_ch_sel()
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_IS_0_4_8_12				(TPDAS_A1M_get_adc_sccl1_imux_ch_sel()==TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_0_4_8_12)
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_IS_1_5_9_13				(TPDAS_A1M_get_adc_sccl1_imux_ch_sel()==TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_1_5_9_13)
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_IS_2_6_10_14			(TPDAS_A1M_get_adc_sccl1_imux_ch_sel()==TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_2_6_10_14)
#define TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_IS_3_7_11_15			(TPDAS_A1M_get_adc_sccl1_imux_ch_sel()==TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_3_7_11_15)

#define TPDAS_A1M_SET_ADC_SCCL1_IMUX_DIFF_EN_OFF				TPDAS_A1M_set_adc_sccl1_imux_diff_en(TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_OFF)					
#define TPDAS_A1M_SET_ADC_SCCL1_IMUX_DIFF_EN_ON					TPDAS_A1M_set_adc_sccl1_imux_diff_en(TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_ON)					

#define TPDAS_A1M_GET_ADC_SCCL1_IMUX_DIFF_EN					TPDAS_A1M_get_adc_sccl1_imux_diff_en()
#define TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_IS_OFF					(TPDAS_A1M_get_adc_sccl1_imux_diff_en()==TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_OFF)
#define TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_IS_ON					(TPDAS_A1M_get_adc_sccl1_imux_diff_en()==TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_ON)

#define TPDAS_A1M_SET_ADC_SCCL1_EOS_EN_OFF						TPDAS_A1M_set_adc_sccl1_eos_en(TPDAS_A1M_ADC_SCCL1_EOS_EN_OFF)
#define TPDAS_A1M_SET_ADC_SCCL1_EOS_EN_ON						TPDAS_A1M_set_adc_sccl1_eos_en(TPDAS_A1M_ADC_SCCL1_EOS_EN_ON)

#define TPDAS_A1M_GET_ADC_SCCL1_EOS_EN							TPDAS_A1M_get_adc_sccl1_eos_en()
#define TPDAS_A1M_ADC_SCCL1_EOS_EN_IS_OFF						(TPDAS_A1M_get_adc_sccl1_eos_en()==TPDAS_A1M_ADC_SCCL1_EOS_EN_OFF)
#define TPDAS_A1M_ADC_SCCL1_EOS_EN_IS_ON						(TPDAS_A1M_get_adc_sccl1_eos_en()==TPDAS_A1M_ADC_SCCL1_EOS_EN_ON)

#define TPDAS_A1M_SET_ADC_SCCL2_IMUX_CH_SEL_0_4_8_12			TPDAS_A1M_set_adc_sccl2_imux_ch_sel(TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_0_4_8_12)					
#define TPDAS_A1M_SET_ADC_SCCL2_IMUX_CH_SEL_1_5_9_13			TPDAS_A1M_set_adc_sccl2_imux_ch_sel(TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_1_5_9_13)					
#define TPDAS_A1M_SET_ADC_SCCL2_IMUX_CH_SEL_2_6_10_14			TPDAS_A1M_set_adc_sccl2_imux_ch_sel(TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_2_6_10_14)					
#define TPDAS_A1M_SET_ADC_SCCL2_IMUX_CH_SEL_3_7_11_15			TPDAS_A1M_set_adc_sccl2_imux_ch_sel(TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_3_7_11_15)					

#define TPDAS_A1M_GET_ADC_SCCL2_IMUX_CH_SEL						TPDAS_A1M_get_adc_sccl2_imux_ch_sel()
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_IS_0_4_8_12				(TPDAS_A1M_get_adc_sccl2_imux_ch_sel()==TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_0_4_8_12)
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_IS_1_5_9_13				(TPDAS_A1M_get_adc_sccl2_imux_ch_sel()==TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_1_5_9_13)
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_IS_2_6_10_14			(TPDAS_A1M_get_adc_sccl2_imux_ch_sel()==TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_2_6_10_14)
#define TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_IS_3_7_11_15			(TPDAS_A1M_get_adc_sccl2_imux_ch_sel()==TPDAS_A1M_ADC_SCCL2_IMUX_CH_SEL_3_7_11_15)

#define TPDAS_A1M_SET_ADC_SCCL2_IMUX_DIFF_EN_OFF				TPDAS_A1M_set_adc_sccl2_imux_diff_en(TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_OFF)					
#define TPDAS_A1M_SET_ADC_SCCL2_IMUX_DIFF_EN_ON					TPDAS_A1M_set_adc_sccl2_imux_diff_en(TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_ON)					

#define TPDAS_A1M_GET_ADC_SCCL2_IMUX_DIFF_EN					TPDAS_A1M_get_adc_sccl2_imux_diff_en()
#define TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_IS_OFF					(TPDAS_A1M_get_adc_sccl2_imux_diff_en()==TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_OFF)
#define TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_IS_ON					(TPDAS_A1M_get_adc_sccl2_imux_diff_en()==TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_ON)

#define TPDAS_A1M_SET_ADC_SCCL2_EOS_EN_OFF						TPDAS_A1M_set_adc_sccl2_eos_en(TPDAS_A1M_ADC_SCCL2_EOS_EN_OFF)
#define TPDAS_A1M_SET_ADC_SCCL2_EOS_EN_ON						TPDAS_A1M_set_adc_sccl2_eos_en(TPDAS_A1M_ADC_SCCL2_EOS_EN_ON)

#define TPDAS_A1M_GET_ADC_SCCL2_EOS_EN							TPDAS_A1M_get_adc_sccl2_eos_en()
#define TPDAS_A1M_ADC_SCCL2_EOS_EN_IS_OFF						(TPDAS_A1M_get_adc_sccl2_eos_en()==TPDAS_A1M_ADC_SCCL2_EOS_EN_OFF)
#define TPDAS_A1M_ADC_SCCL2_EOS_EN_IS_ON						(TPDAS_A1M_get_adc_sccl2_eos_en()==TPDAS_A1M_ADC_SCCL2_EOS_EN_ON)

#define TPDAS_A1M_SET_ADC_SCCL3_IMUX_CH_SEL_0_4_8_12			TPDAS_A1M_set_adc_sccl3_imux_ch_sel(TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_0_4_8_12)					
#define TPDAS_A1M_SET_ADC_SCCL3_IMUX_CH_SEL_1_5_9_13			TPDAS_A1M_set_adc_sccl3_imux_ch_sel(TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_1_5_9_13)					
#define TPDAS_A1M_SET_ADC_SCCL3_IMUX_CH_SEL_2_6_10_14			TPDAS_A1M_set_adc_sccl3_imux_ch_sel(TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_2_6_10_14)					
#define TPDAS_A1M_SET_ADC_SCCL3_IMUX_CH_SEL_3_7_11_15			TPDAS_A1M_set_adc_sccl3_imux_ch_sel(TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_3_7_11_15)					

#define TPDAS_A1M_GET_ADC_SCCL3_IMUX_CH_SEL						TPDAS_A1M_get_adc_sccl3_imux_ch_sel()
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_IS_0_4_8_12				(TPDAS_A1M_get_adc_sccl3_imux_ch_sel()==TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_0_4_8_12)
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_IS_1_5_9_13				(TPDAS_A1M_get_adc_sccl3_imux_ch_sel()==TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_1_5_9_13)
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_IS_2_6_10_14			(TPDAS_A1M_get_adc_sccl3_imux_ch_sel()==TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_2_6_10_14)
#define TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_IS_3_7_11_15			(TPDAS_A1M_get_adc_sccl3_imux_ch_sel()==TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_3_7_11_15)

#define TPDAS_A1M_SET_ADC_SCCL3_IMUX_DIFF_EN_OFF				TPDAS_A1M_set_adc_sccl3_imux_diff_en(TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_OFF)					
#define TPDAS_A1M_SET_ADC_SCCL3_IMUX_DIFF_EN_ON					TPDAS_A1M_set_adc_sccl3_imux_diff_en(TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_ON)					

#define TPDAS_A1M_GET_ADC_SCCL3_IMUX_DIFF_EN					TPDAS_A1M_get_adc_sccl3_imux_diff_en()
#define TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_IS_OFF					(TPDAS_A1M_get_adc_sccl3_imux_diff_en()==TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_OFF)
#define TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_IS_ON					(TPDAS_A1M_get_adc_sccl3_imux_diff_en()==TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_ON)

#define TPDAS_A1M_SET_ADC_SCCL3_EOS_EN_OFF						TPDAS_A1M_set_adc_sccl3_eos_en(TPDAS_A1M_ADC_SCCL3_EOS_EN_OFF)
#define TPDAS_A1M_SET_ADC_SCCL3_EOS_EN_ON						TPDAS_A1M_set_adc_sccl3_eos_en(TPDAS_A1M_ADC_SCCL3_EOS_EN_ON)

#define TPDAS_A1M_GET_ADC_SCCL3_EOS_EN							TPDAS_A1M_get_adc_sccl3_eos_en()
#define TPDAS_A1M_ADC_SCCL3_EOS_EN_IS_OFF						(TPDAS_A1M_get_adc_sccl3_eos_en()==TPDAS_A1M_ADC_SCCL3_EOS_EN_OFF)
#define TPDAS_A1M_ADC_SCCL3_EOS_EN_IS_ON						(TPDAS_A1M_get_adc_sccl3_eos_en()==TPDAS_A1M_ADC_SCCL3_EOS_EN_ON)

			// for ADC IMUX control in ADC ASYNC MODE
#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_0_4_8_12		TPDAS_A1M_set_adc_async_daq_imux_ch_sel(TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_0_4_8_12)					
#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_1_5_9_13		TPDAS_A1M_set_adc_async_daq_imux_ch_sel(TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_1_5_9_13)					
#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_2_6_10_14		TPDAS_A1M_set_adc_async_daq_imux_ch_sel(TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_2_6_10_14)					
#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_CH_SEL_3_7_11_15		TPDAS_A1M_set_adc_async_daq_imux_ch_sel(TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_3_7_11_15)					

#define TPDAS_A1M_GET_ADC_ASYNC_DAQ_IMUX_CH_SEL					TPDAS_A1M_get_adc_async_daq_imux_ch_sel()
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_IS_0_4_8_12			(TPDAS_A1M_get_adc_async_daq_imux_ch_sel()==TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_0_4_8_12)
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_IS_1_5_9_13			(TPDAS_A1M_get_adc_async_daq_imux_ch_sel()==TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_1_5_9_13)
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_IS_2_6_10_14		(TPDAS_A1M_get_adc_async_daq_imux_ch_sel()==TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_2_6_10_14)
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_IS_3_7_11_15		(TPDAS_A1M_get_adc_async_daq_imux_ch_sel()==TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_CH_SEL_3_7_11_15)

#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_DIFF_EN_OFF			TPDAS_A1M_set_adc_async_daq_imux_diff_en(TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_OFF)					
#define TPDAS_A1M_SET_ADC_ASYNC_DAQ_IMUX_DIFF_EN_ON				TPDAS_A1M_set_adc_async_daq_imux_diff_en(TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_ON)					

#define TPDAS_A1M_GET_ADC_ASYNC_DAQ_IMUX_DIFF_EN				TPDAS_A1M_get_adc_async_daq_imux_diff_en()
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_IS_OFF				(TPDAS_A1M_get_adc_async_daq_imux_diff_en()==TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_OFF)
#define TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_IS_ON				(TPDAS_A1M_get_adc_async_daq_imux_diff_en()==TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_ON)

		// unified macros with TPDAS_A1M_ADC_SCCL_IMUX_CH_SEL_0_4_8_12/_1_5_9_13/... 
#define TPDAS_A1M_uget_adc_sccl0_imux_ch_sel()					TPDAS_A1M_get_adc_sccl0_imux_ch_sel()
#define TPDAS_A1M_uset_adc_sccl0_imux_ch_sel(d)					TPDAS_A1M_set_adc_sccl0_imux_ch_sel(d)

#define TPDAS_A1M_uget_adc_sccl1_imux_ch_sel()					(TPDAS_A1M_get_adc_sccl1_imux_ch_sel()>>TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl1_imux_ch_sel(d)					TPDAS_A1M_set_adc_sccl1_imux_ch_sel((d)<<TPDAS_A1M_ADC_SCCL1_IMUX_CH_SEL_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl2_imux_ch_sel()					TPDAS_A1M_get_adc_sccl2_imux_ch_sel()
#define TPDAS_A1M_uset_adc_sccl2_imux_ch_sel(d)					TPDAS_A1M_set_adc_sccl2_imux_ch_sel(d)

#define TPDAS_A1M_uget_adc_sccl3_imux_ch_sel()					(TPDAS_A1M_get_adc_sccl3_imux_ch_sel()>>TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl3_imux_ch_sel(d)					TPDAS_A1M_set_adc_sccl3_imux_ch_sel((d)<<TPDAS_A1M_ADC_SCCL3_IMUX_CH_SEL_BITSHIFT)
			// for ADC IMUX control in ADC ASYNC MODE
#define TPDAS_A1M_uget_adc_async_daq_imux_ch_sel()				TPDAS_A1M_get_adc_async_daq_imux_ch_sel()
#define TPDAS_A1M_uset_adc_async_daq_imux_ch_sel(d)				TPDAS_A1M_set_adc_async_daq_imux_ch_sel(d)

		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_adc_sccl0_imux_diff_en()					(TPDAS_A1M_get_adc_sccl0_imux_diff_en()>>TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl0_imux_diff_en(d)				TPDAS_A1M_set_adc_sccl0_imux_diff_en((d)<<TPDAS_A1M_ADC_SCCL0_IMUX_DIFF_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl0_eos_en()						(TPDAS_A1M_get_adc_sccl0_eos_en()>>TPDAS_A1M_ADC_SCCL0_EOS_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl0_eos_en(d)						TPDAS_A1M_set_adc_sccl0_eos_en((d)<<TPDAS_A1M_ADC_SCCL0_EOS_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl1_imux_diff_en()					(TPDAS_A1M_get_adc_sccl1_imux_diff_en()>>TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl1_imux_diff_en(d)				TPDAS_A1M_set_adc_sccl1_imux_diff_en((d)<<TPDAS_A1M_ADC_SCCL1_IMUX_DIFF_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl1_eos_en()						(TPDAS_A1M_get_adc_sccl1_eos_en()>>TPDAS_A1M_ADC_SCCL1_EOS_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl1_eos_en(d)						TPDAS_A1M_set_adc_sccl1_eos_en((d)<<TPDAS_A1M_ADC_SCCL1_EOS_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl2_imux_diff_en()					(TPDAS_A1M_get_adc_sccl2_imux_diff_en()>>TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl2_imux_diff_en(d)				TPDAS_A1M_set_adc_sccl2_imux_diff_en((d)<<TPDAS_A1M_ADC_SCCL2_IMUX_DIFF_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl2_eos_en()						(TPDAS_A1M_get_adc_sccl2_eos_en()>>TPDAS_A1M_ADC_SCCL2_EOS_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl2_eos_en(d)						TPDAS_A1M_set_adc_sccl2_eos_en((d)<<TPDAS_A1M_ADC_SCCL2_EOS_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl3_imux_diff_en()					(TPDAS_A1M_get_adc_sccl3_imux_diff_en()>>TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl3_imux_diff_en(d)				TPDAS_A1M_set_adc_sccl3_imux_diff_en((d)<<TPDAS_A1M_ADC_SCCL3_IMUX_DIFF_EN_BITSHIFT)

#define TPDAS_A1M_uget_adc_sccl3_eos_en()						(TPDAS_A1M_get_adc_sccl3_eos_en()>>TPDAS_A1M_ADC_SCCL3_EOS_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_sccl3_eos_en(d)						TPDAS_A1M_set_adc_sccl3_eos_en((d)<<TPDAS_A1M_ADC_SCCL3_EOS_EN_BITSHIFT)
			// for ADC IMUX control in ADC ASYNC MODE
#define TPDAS_A1M_uget_adc_async_daq_imux_diff_en()				(TPDAS_A1M_get_adc_async_daq_imux_diff_en()>>TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_BITSHIFT)
#define TPDAS_A1M_uset_adc_async_daq_imux_diff_en(d)			TPDAS_A1M_set_adc_async_daq_imux_diff_en((d)<<TPDAS_A1M_ADC_ASYNC_DAQ_IMUX_DIFF_EN_BITSHIFT)



	//-- ADC/DAC FIFO status Register (read-only)
#define TPDAS_A1M_get_fifo_stat_rg()							((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_FIFO_STAT_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)

#define TPDAS_A1M_get_adc_fifo_ff()								(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_ADC_FIFO_FF_BITMASK)
#define TPDAS_A1M_get_adc_fifo_paf()							(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_ADC_FIFO_PAF_BITMASK)
#define TPDAS_A1M_get_adc_fifo_ef()								(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_ADC_FIFO_EF_BITMASK)
#define TPDAS_A1M_get_adc_fifo_pae()							(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_ADC_FIFO_PAE_BITMASK)
#define TPDAS_A1M_get_dac_fifo_ff()								(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_DAC_FIFO_FF_BITMASK)
#define TPDAS_A1M_get_dac_fifo_paf()							(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_DAC_FIFO_PAF_BITMASK)
#define TPDAS_A1M_get_dac_fifo_ef()								(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_DAC_FIFO_EF_BITMASK)
#define TPDAS_A1M_get_dac_fifo_pae()							(TPDAS_A1M_get_fifo_stat_rg() & TPDAS_A1M_DAC_FIFO_PAE_BITMASK)

		// direct get macros
#define TPDAS_A1M_GET_ADC_FIFO_FF								TPDAS_A1M_get_adc_fifo_ff()
#define TPDAS_A1M_ADC_FIFO_FF_IS_OFF							(TPDAS_A1M_get_adc_fifo_ff()==TPDAS_A1M_ADC_FIFO_FF_OFF)
#define TPDAS_A1M_ADC_FIFO_FF_IS_ON								(TPDAS_A1M_get_adc_fifo_ff()==TPDAS_A1M_ADC_FIFO_FF_ON)

#define TPDAS_A1M_GET_ADC_FIFO_PAF								TPDAS_A1M_get_adc_fifo_paf()
#define TPDAS_A1M_ADC_FIFO_PAF_IS_OFF							(TPDAS_A1M_get_adc_fifo_paf()==TPDAS_A1M_ADC_FIFO_PAF_OFF)
#define TPDAS_A1M_ADC_FIFO_PAF_IS_ON							(TPDAS_A1M_get_adc_fifo_paf()==TPDAS_A1M_ADC_FIFO_PAF_ON)

#define TPDAS_A1M_GET_ADC_FIFO_EF								TPDAS_A1M_get_adc_fifo_ef()
#define TPDAS_A1M_ADC_FIFO_EF_IS_OFF							(TPDAS_A1M_get_adc_fifo_ef()==TPDAS_A1M_ADC_FIFO_EF_OFF)
#define TPDAS_A1M_ADC_FIFO_EF_IS_ON								(TPDAS_A1M_get_adc_fifo_ef()==TPDAS_A1M_ADC_FIFO_EF_ON)

#define TPDAS_A1M_GET_ADC_FIFO_PAE								TPDAS_A1M_get_adc_fifo_pae()
#define TPDAS_A1M_ADC_FIFO_PAE_IS_OFF							(TPDAS_A1M_get_adc_fifo_pae()==TPDAS_A1M_ADC_FIFO_PAE_OFF)
#define TPDAS_A1M_ADC_FIFO_PAE_IS_ON							(TPDAS_A1M_get_adc_fifo_pae()==TPDAS_A1M_ADC_FIFO_PAE_ON)

#define TPDAS_A1M_GET_DAC_FIFO_FF								TPDAS_A1M_get_dac_fifo_ff()
#define TPDAS_A1M_DAC_FIFO_FF_IS_OFF							(TPDAS_A1M_get_dac_fifo_ff()==TPDAS_A1M_DAC_FIFO_FF_OFF)
#define TPDAS_A1M_DAC_FIFO_FF_IS_ON								(TPDAS_A1M_get_dac_fifo_ff()==TPDAS_A1M_DAC_FIFO_FF_ON)

#define TPDAS_A1M_GET_DAC_FIFO_PAF								TPDAS_A1M_get_dac_fifo_paf()
#define TPDAS_A1M_DAC_FIFO_PAF_IS_OFF							(TPDAS_A1M_get_dac_fifo_paf()==TPDAS_A1M_DAC_FIFO_PAF_OFF)
#define TPDAS_A1M_DAC_FIFO_PAF_IS_ON							(TPDAS_A1M_get_dac_fifo_paf()==TPDAS_A1M_DAC_FIFO_PAF_ON)

#define TPDAS_A1M_GET_DAC_FIFO_EF								TPDAS_A1M_get_dac_fifo_ef()
#define TPDAS_A1M_DAC_FIFO_EF_IS_OFF							(TPDAS_A1M_get_dac_fifo_ef()==TPDAS_A1M_DAC_FIFO_EF_OFF)
#define TPDAS_A1M_DAC_FIFO_EF_IS_ON								(TPDAS_A1M_get_dac_fifo_ef()==TPDAS_A1M_DAC_FIFO_EF_ON)

#define TPDAS_A1M_GET_DAC_FIFO_PAE								TPDAS_A1M_get_dac_fifo_pae()
#define TPDAS_A1M_DAC_FIFO_PAE_IS_OFF							(TPDAS_A1M_get_dac_fifo_pae()==TPDAS_A1M_DAC_FIFO_PAE_OFF)
#define TPDAS_A1M_DAC_FIFO_PAE_IS_ON							(TPDAS_A1M_get_dac_fifo_pae()==TPDAS_A1M_DAC_FIFO_PAE_ON)

		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_adc_fifo_ff()							TPDAS_A1M_get_adc_fifo_ff()
#define TPDAS_A1M_uget_adc_fifo_paf()							(TPDAS_A1M_get_adc_fifo_paf()>>TPDAS_A1M_ADC_FIFO_PAF_SHIFT)
#define TPDAS_A1M_uget_adc_fifo_ef()							(TPDAS_A1M_get_adc_fifo_ef()>>TPDAS_A1M_ADC_FIFO_EF_SHIFT)
#define TPDAS_A1M_uget_adc_fifo_pae()							(TPDAS_A1M_get_adc_fifo_pae()>>TPDAS_A1M_ADC_FIFO_PAE_SHIFT)
#define TPDAS_A1M_uget_dac_fifo_ff()							(TPDAS_A1M_get_dac_fifo_ff()>>TPDAS_A1M_DAC_FIFO_FF_SHIFT)
#define TPDAS_A1M_uget_dac_fifo_paf()							(TPDAS_A1M_get_dac_fifo_paf()>>TPDAS_A1M_DAC_FIFO_PAF_SHIFT)
#define TPDAS_A1M_uget_dac_fifo_ef()							(TPDAS_A1M_get_dac_fifo_ef()>>TPDAS_A1M_DAC_FIFO_EF_SHIFT)
#define TPDAS_A1M_uget_dac_fifo_pae()							(TPDAS_A1M_get_dac_fifo_pae()>>TPDAS_A1M_DAC_FIFO_PAE_SHIFT)



	//-- ERROR status/clear Registers
#define TPDAS_A1M_get_err_stat_rg()								((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ERR_STAT_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_err_clr_rg(d)								(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ERR_CLR_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc_fifo_ovf()							(TPDAS_A1M_get_err_stat_rg() & TPDAS_A1M_ADC_FIFO_OVF_BITMASK)
#define TPDAS_A1M_get_adc_fifo_unf()							(TPDAS_A1M_get_err_stat_rg() & TPDAS_A1M_ADC_FIFO_UNF_BITMASK)
#define TPDAS_A1M_get_dac_fifo_ovf()							(TPDAS_A1M_get_err_stat_rg() & TPDAS_A1M_DAC_FIFO_OVF_BITMASK)
#define TPDAS_A1M_get_dac_fifo_unf()							(TPDAS_A1M_get_err_stat_rg() & TPDAS_A1M_DAC_FIFO_UNF_BITMASK)
#define TPDAS_A1M_get_adc_irt_err()								(TPDAS_A1M_get_err_stat_rg() & TPDAS_A1M_ADC_IRT_ERR_BITMASK)
#define TPDAS_A1M_get_dac_irt_err()								(TPDAS_A1M_get_err_stat_rg() & TPDAS_A1M_DAC_IRT_ERR_BITMASK)

#define TPDAS_A1M_clr_adc_fifo_ovf()							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_ADC_FIFO_OVF_CLR)
#define TPDAS_A1M_clr_adc_fifo_unf()							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_ADC_FIFO_UNF_CLR)
#define TPDAS_A1M_clr_dac_fifo_ovf()							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_DAC_FIFO_OVF_CLR)
#define TPDAS_A1M_clr_dac_fifo_unf()							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_DAC_FIFO_UNF_CLR)
#define TPDAS_A1M_clr_adc_irt_err()								TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_ADC_IRT_ERR_CLR)
#define TPDAS_A1M_clr_dac_irt_err()								TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_DAC_IRT_ERR_CLR)

		// direct clear/get macros
#define TPDAS_A1M_CLR_ADC_FIFO_OVF								TPDAS_A1M_clr_adc_fifo_ovf()
#define TPDAS_A1M_CLR_ADC_FIFO_UNF								TPDAS_A1M_clr_adc_fifo_unf()
#define TPDAS_A1M_CLR_DAC_FIFO_OVF								TPDAS_A1M_clr_dac_fifo_ovf()
#define TPDAS_A1M_CLR_DAC_FIFO_UNF								TPDAS_A1M_clr_dac_fifo_unf()
#define TPDAS_A1M_CLR_ADC_IRT_ERR								TPDAS_A1M_clr_adc_irt_err()
#define TPDAS_A1M_CLR_DAC_IRT_ERR								TPDAS_A1M_clr_dac_irt_err()
			// combined CLEAR macros
#define TPDAS_A1M_CLR_ERR_ALL									TPDAS_A1M_set_err_clr_rg(0xff)
#define TPDAS_A1M_CLR_ADC_FIFO_ERR_ALL							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_ADC_FIFO_OVF_CLR | TPDAS_A1M_ADC_FIFO_UNF_CLR)
#define TPDAS_A1M_CLR_DAC_FIFO_ERR_ALL							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_DAC_FIFO_OVF_CLR | TPDAS_A1M_DAC_FIFO_UNF_CLR)
#define TPDAS_A1M_CLR_ADDA_FIFO_ERR_ALL							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_ADC_FIFO_OVF_CLR | TPDAS_A1M_ADC_FIFO_UNF_CLR | TPDAS_A1M_DAC_FIFO_OVF_CLR | TPDAS_A1M_DAC_FIFO_UNF_CLR)
#define TPDAS_A1M_CLR_ADDA_IRT_ERR_ALL							TPDAS_A1M_set_err_clr_rg(TPDAS_A1M_ADC_IRT_ERR_CLR | TPDAS_A1M_DAC_IRT_ERR_CLR)


#define TPDAS_A1M_GET_ADC_FIFO_OVF								TPDAS_A1M_get_adc_fifo_ovf()
#define TPDAS_A1M_ADC_FIFO_OVF_IS_OFF							(TPDAS_A1M_get_adc_fifo_ovf()==TPDAS_A1M_ADC_FIFO_OVF_OFF)
#define TPDAS_A1M_ADC_FIFO_OVF_IS_ON							(TPDAS_A1M_get_adc_fifo_ovf()==TPDAS_A1M_ADC_FIFO_OVF_ON)

#define TPDAS_A1M_GET_ADC_FIFO_UNF								TPDAS_A1M_get_adc_fifo_unf()
#define TPDAS_A1M_ADC_FIFO_UNF_IS_OFF							(TPDAS_A1M_get_adc_fifo_unf()==TPDAS_A1M_ADC_FIFO_UNF_OFF)
#define TPDAS_A1M_ADC_FIFO_UNF_IS_ON							(TPDAS_A1M_get_adc_fifo_unf()==TPDAS_A1M_ADC_FIFO_UNF_ON)

#define TPDAS_A1M_GET_DAC_FIFO_OVF								TPDAS_A1M_get_dac_fifo_ovf()
#define TPDAS_A1M_DAC_FIFO_OVF_IS_OFF							(TPDAS_A1M_get_dac_fifo_ovf()==TPDAS_A1M_DAC_FIFO_OVF_OFF)
#define TPDAS_A1M_DAC_FIFO_OVF_IS_ON							(TPDAS_A1M_get_dac_fifo_ovf()==TPDAS_A1M_DAC_FIFO_OVF_ON)

#define TPDAS_A1M_GET_DAC_FIFO_UNF								TPDAS_A1M_get_dac_fifo_unf()
#define TPDAS_A1M_DAC_FIFO_UNF_IS_OFF							(TPDAS_A1M_get_dac_fifo_unf()==TPDAS_A1M_DAC_FIFO_UNF_OFF)
#define TPDAS_A1M_DAC_FIFO_UNF_IS_ON							(TPDAS_A1M_get_dac_fifo_unf()==TPDAS_A1M_DAC_FIFO_UNF_ON)

#define TPDAS_A1M_GET_ADC_IRT_ERR								TPDAS_A1M_get_adc_irt_err()
#define TPDAS_A1M_ADC_IRT_ERR_IS_OFF							(TPDAS_A1M_get_adc_irt_err()==TPDAS_A1M_ADC_IRT_ERR_OFF)
#define TPDAS_A1M_ADC_IRT_ERR_IS_ON								(TPDAS_A1M_get_adc_irt_err()==TPDAS_A1M_ADC_IRT_ERR_ON)

#define TPDAS_A1M_GET_DAC_IRT_ERR								TPDAS_A1M_get_dac_irt_err()
#define TPDAS_A1M_DAC_IRT_ERR_IS_OFF							(TPDAS_A1M_get_dac_irt_err()==TPDAS_A1M_DAC_IRT_ERR_OFF)
#define TPDAS_A1M_DAC_IRT_ERR_IS_ON								(TPDAS_A1M_get_dac_irt_err()==TPDAS_A1M_DAC_IRT_ERR_ON)

		// combined error status
#define TPDAS_A1M_GET_ERR_ALL									(TPDAS_A1M_get_err_stat_rg())
#define TPDAS_A1M_GET_ADDA_IRT_ERR_ALL							(TPDAS_A1M_get_err_stat_rg() & (TPDAS_A1M_ADC_IRT_ERR_BITMASK | TPDAS_A1M_DAC_IRT_ERR_BITMASK))
#define TPDAS_A1M_GET_ADC_FIFO_ERR_ALL							(TPDAS_A1M_get_err_stat_rg() & (TPDAS_A1M_ADC_FIFO_OVF_BITMASK | TPDAS_A1M_ADC_FIFO_UNF_BITMASK))
#define TPDAS_A1M_GET_ADC_FIFO_IRT_ERR_ALL						(TPDAS_A1M_get_err_stat_rg() & (TPDAS_A1M_ADC_FIFO_OVF_BITMASK | TPDAS_A1M_ADC_FIFO_UNF_BITMASK | TPDAS_A1M_ADC_IRT_ERR_BITMASK))
#define TPDAS_A1M_GET_DAC_FIFO_ERR_ALL							(TPDAS_A1M_get_err_stat_rg() & (TPDAS_A1M_DAC_FIFO_OVF_BITMASK | TPDAS_A1M_DAC_FIFO_UNF_BITMASK))
#define TPDAS_A1M_GET_DAC_FIFO_IRT_ERR_ALL						(TPDAS_A1M_get_err_stat_rg() & (TPDAS_A1M_DAC_FIFO_OVF_BITMASK | TPDAS_A1M_DAC_FIFO_UNF_BITMASK | TPDAS_A1M_DAC_IRT_ERR_BITMASK))
#define TPDAS_A1M_GET_ADDA_FIFO_ERR_ALL							(TPDAS_A1M_get_err_stat_rg() & (TPDAS_A1M_ADC_FIFO_OVF_BITMASK | TPDAS_A1M_ADC_FIFO_UNF_BITMASK | TPDAS_A1M_DAC_FIFO_OVF_BITMASK | TPDAS_A1M_DAC_FIFO_UNF_BITMASK))


		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_adc_fifo_ovf()							TPDAS_A1M_get_adc_fifo_ovf()
#define TPDAS_A1M_uget_adc_fifo_unf()							(TPDAS_A1M_get_adc_fifo_unf()>>TPDAS_A1M_ADC_FIFO_UNF_SHIFT)
#define TPDAS_A1M_uget_dac_fifo_ovf()							(TPDAS_A1M_get_dac_fifo_ovf()>>TPDAS_A1M_DAC_FIFO_OVF_SHIFT)
#define TPDAS_A1M_uget_dac_fifo_unf()							(TPDAS_A1M_get_dac_fifo_unf()>>TPDAS_A1M_DAC_FIFO_UNF_SHIFT)
#define TPDAS_A1M_uget_adc_irt_err()							(TPDAS_A1M_get_adc_irt_err()>>TPDAS_A1M_ADC_IRT_ERR_SHIFT)
#define TPDAS_A1M_uget_dac_irt_err()							(TPDAS_A1M_get_dac_irt_err()>>TPDAS_A1M_DAC_IRT_ERR_SHIFT)

   

	//-- HIRQ-0 Interrupt control Register
#define TPDAS_A1M_get_hirq0_sel_rg()							((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ0_SEL_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_hirq0_sel_rg(d)							(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ0_SEL_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_hirq0_sel()								(TPDAS_A1M_get_hirq0_sel_rg() & TPDAS_A1M_HIRQ_SEL_BITMASK)
#define TPDAS_A1M_set_hirq0_sel(d)								TPDAS_A1M_set_hirq0_sel_rg((TPDAS_A1M_get_hirq0_sel_rg() & ~TPDAS_A1M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A1M_HIRQ_SEL_BITMASK))
#define TPDAS_A1M_get_hirq0_en()								(TPDAS_A1M_get_hirq0_sel_rg() & TPDAS_A1M_HIRQ_EN_BITMASK)
#define TPDAS_A1M_set_hirq0_en(d)								TPDAS_A1M_set_hirq0_sel_rg((TPDAS_A1M_get_hirq0_sel_rg() & ~TPDAS_A1M_HIRQ_EN_BITMASK) |  (d & TPDAS_A1M_HIRQ_EN_BITMASK))
#define TPDAS_A1M_get_hirq0_pol()								(TPDAS_A1M_get_hirq0_sel_rg() & TPDAS_A1M_HIRQ_POL_BITMASK)
#define TPDAS_A1M_set_hirq0_pol(d)								TPDAS_A1M_set_hirq0_sel_rg((TPDAS_A1M_get_hirq0_sel_rg() & ~TPDAS_A1M_HIRQ_POL_BITMASK) |  (d & TPDAS_A1M_HIRQ_POL_BITMASK))

		// direct set macros
#define TPDAS_A1M_SET_HIRQ0_OFF									TPDAS_A1M_set_hirq0_sel_rg(TPDAS_A1M_HIRQ_SEL_OFF | TPDAS_A1M_HIRQ_EN_OFF)

#define TPDAS_A1M_SET_HIRQ0_EN_OFF								TPDAS_A1M_set_hirq0_en(TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_SET_HIRQ0_EN_ON								TPDAS_A1M_set_hirq0_en(TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_SET_HIRQ0_POL_POS								TPDAS_A1M_set_hirq0_pol(TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_SET_HIRQ0_POL_NEG								TPDAS_A1M_set_hirq0_pol(TPDAS_A1M_HIRQ_POL_NEG)

#define TPDAS_A1M_SET_HIRQ0_SEL_OFF								TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_SYNC_DAQ_END				TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_SYNC_DAQ_END				TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_IRT							TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_IRT							TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_SET_HIRQ0_SEL_GPIO0							TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_SET_HIRQ0_SEL_GPIO1							TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_SET_HIRQ0_SEL_ERR								TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_SET_HIRQ0_SEL_FS								TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_RDY							TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_RDY)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_IRT_END						TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_IRT_END)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_IRT_END						TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_IRT_END)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_FIFO_FF						TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_FF)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_FIFO_PAF					TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAF)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_FIFO_EF						TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_EF)
#define TPDAS_A1M_SET_HIRQ0_SEL_ADC_FIFO_PAE					TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAE)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_FIFO_FF						TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_FF)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_FIFO_PAF					TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAF)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_FIFO_EF						TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_EF)
#define TPDAS_A1M_SET_HIRQ0_SEL_DAC_FIFO_PAE					TPDAS_A1M_set_hirq0_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAE)


		// direct get macros
#define TPDAS_A1M_GET_HIRQ0_SEL									TPDAS_A1M_get_hirq0_sel()
#define TPDAS_A1M_HIRQ0_SEL_IS_OFF								(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_HIRQ0_SEL_IS_ADC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ0_SEL_IS_DAC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ0_SEL_IS_ADC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ0_SEL_IS_DAC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ0_SEL_IS_ADC_IRT							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_HIRQ0_SEL_IS_DAC_IRT							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_HIRQ0_SEL_IS_GPIO0							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_HIRQ0_SEL_IS_GPIO1							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_HIRQ0_SEL_IS_ERR								(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_HIRQ0_SEL_IS_FS								(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_HIRQ0_SEL_ADC_RDY								(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_RDY)
#define TPDAS_A1M_HIRQ0_SEL_ADC_IRT_END							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_IRT_END)
#define TPDAS_A1M_HIRQ0_SEL_DAC_IRT_END							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_IRT_END)
#define TPDAS_A1M_HIRQ0_SEL_ADC_FIFO_FF							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_FF)
#define TPDAS_A1M_HIRQ0_SEL_ADC_FIFO_PAF						(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAF)
#define TPDAS_A1M_HIRQ0_SEL_ADC_FIFO_EF							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_EF)
#define TPDAS_A1M_HIRQ0_SEL_ADC_FIFO_PAE						(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAE)
#define TPDAS_A1M_HIRQ0_SEL_DAC_FIFO_FF							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_FF)
#define TPDAS_A1M_HIRQ0_SEL_DAC_FIFO_PAF						(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAF)
#define TPDAS_A1M_HIRQ0_SEL_DAC_FIFO_EF							(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_EF)
#define TPDAS_A1M_HIRQ0_SEL_DAC_FIFO_PAE						(TPDAS_A1M_get_hirq0_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAE)

#define TPDAS_A1M_GET_HIRQ0_EN									TPDAS_A1M_get_hirq0_en()
#define TPDAS_A1M_HIRQ0_EN_IS_OFF								(TPDAS_A1M_get_hirq0_en()==TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_HIRQ0_EN_IS_ON								(TPDAS_A1M_get_hirq0_en()==TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_GET_HIRQ0_POL									TPDAS_A1M_get_hirq0_pol()
#define TPDAS_A1M_HIRQ0_POL_IS_POS								(TPDAS_A1M_get_hirq0_pol()==TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_HIRQ0_POL_IS_NEG								(TPDAS_A1M_get_hirq0_pol()==TPDAS_A1M_HIRQ_POL_NEG)


		// unified macros with TPDAS_A1M_HIRQ_SEL_...
#define TPDAS_A1M_uget_hirq0_sel()								TPDAS_A1M_get_hirq0_sel()
#define TPDAS_A1M_uset_hirq0_sel(d)								TPDAS_A1M_set_hirq0_sel(d)

		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_hirq0_en()								(TPDAS_A1M_get_hirq0_en()>>TPDAS_A1M_HIRQ_EN_SHIFT)
#define TPDAS_A1M_uset_hirq0_en(d)								TPDAS_A1M_set_hirq0_en((d)<<TPDAS_A1M_HIRQ_EN_SHIFT)

		// unified macros with TPDAS_HIRQ_POL_.. 
#define TPDAS_A1M_uget_hirq0_pol()								TPDAS_A1M_get_hirq0_pol()
#define TPDAS_A1M_uset_hirq0_pol(d)								TPDAS_A1M_set_hirq0_pol(d)



	//-- HIRQ-1 Interrupt control Register
#define TPDAS_A1M_get_hirq1_sel_rg()							((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ1_SEL_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_hirq1_sel_rg(d)							(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ1_SEL_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_hirq1_sel()								(TPDAS_A1M_get_hirq1_sel_rg() & TPDAS_A1M_HIRQ_SEL_BITMASK)
#define TPDAS_A1M_set_hirq1_sel(d)								TPDAS_A1M_set_hirq1_sel_rg((TPDAS_A1M_get_hirq1_sel_rg() & ~TPDAS_A1M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A1M_HIRQ_SEL_BITMASK))
#define TPDAS_A1M_get_hirq1_en()								(TPDAS_A1M_get_hirq1_sel_rg() & TPDAS_A1M_HIRQ_EN_BITMASK)
#define TPDAS_A1M_set_hirq1_en(d)								TPDAS_A1M_set_hirq1_sel_rg((TPDAS_A1M_get_hirq1_sel_rg() & ~TPDAS_A1M_HIRQ_EN_BITMASK) |  (d & TPDAS_A1M_HIRQ_EN_BITMASK))
#define TPDAS_A1M_get_hirq1_pol()								(TPDAS_A1M_get_hirq1_sel_rg() & TPDAS_A1M_HIRQ_POL_BITMASK)
#define TPDAS_A1M_set_hirq1_pol(d)								TPDAS_A1M_set_hirq1_sel_rg((TPDAS_A1M_get_hirq1_sel_rg() & ~TPDAS_A1M_HIRQ_POL_BITMASK) |  (d & TPDAS_A1M_HIRQ_POL_BITMASK))

		// direct set macros
#define TPDAS_A1M_SET_HIRQ1_OFF									TPDAS_A1M_set_hirq1_sel_rg(TPDAS_A1M_HIRQ_SEL_OFF | TPDAS_A1M_HIRQ_EN_OFF)

#define TPDAS_A1M_SET_HIRQ1_EN_OFF								TPDAS_A1M_set_hirq1_en(TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_SET_HIRQ1_EN_ON								TPDAS_A1M_set_hirq1_en(TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_SET_HIRQ1_POL_POS								TPDAS_A1M_set_hirq1_pol(TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_SET_HIRQ1_POL_NEG								TPDAS_A1M_set_hirq1_pol(TPDAS_A1M_HIRQ_POL_NEG)

#define TPDAS_A1M_SET_HIRQ1_SEL_OFF								TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_SYNC_DAQ_END				TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_SYNC_DAQ_END				TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_IRT							TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_IRT							TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_SET_HIRQ1_SEL_GPIO0							TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_SET_HIRQ1_SEL_GPIO1							TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_SET_HIRQ1_SEL_ERR								TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_SET_HIRQ1_SEL_FS								TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_RDY							TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_RDY)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_IRT_END						TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_IRT_END)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_IRT_END						TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_IRT_END)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_FIFO_FF						TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_FF)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_FIFO_PAF					TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAF)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_FIFO_EF						TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_EF)
#define TPDAS_A1M_SET_HIRQ1_SEL_ADC_FIFO_PAE					TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAE)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_FIFO_FF						TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_FF)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_FIFO_PAF					TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAF)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_FIFO_EF						TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_EF)
#define TPDAS_A1M_SET_HIRQ1_SEL_DAC_FIFO_PAE					TPDAS_A1M_set_hirq1_sel(TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAE)


		// direct get macros
#define TPDAS_A1M_GET_HIRQ1_SEL									TPDAS_A1M_get_hirq1_sel()
#define TPDAS_A1M_HIRQ1_SEL_IS_OFF								(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_HIRQ1_SEL_IS_ADC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ1_SEL_IS_DAC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ1_SEL_IS_ADC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ1_SEL_IS_DAC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ1_SEL_IS_ADC_IRT							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_HIRQ1_SEL_IS_DAC_IRT							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_HIRQ1_SEL_IS_GPIO0							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_HIRQ1_SEL_IS_GPIO1							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_HIRQ1_SEL_IS_ERR								(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_HIRQ1_SEL_IS_FS								(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_HIRQ1_SEL_ADC_RDY								(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_RDY)
#define TPDAS_A1M_HIRQ1_SEL_ADC_IRT_END							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_IRT_END)
#define TPDAS_A1M_HIRQ1_SEL_DAC_IRT_END							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_IRT_END)
#define TPDAS_A1M_HIRQ1_SEL_ADC_FIFO_FF							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_FF)
#define TPDAS_A1M_HIRQ1_SEL_ADC_FIFO_PAF						(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAF)
#define TPDAS_A1M_HIRQ1_SEL_ADC_FIFO_EF							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_EF)
#define TPDAS_A1M_HIRQ1_SEL_ADC_FIFO_PAE						(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_ADC_FIFO_PAE)
#define TPDAS_A1M_HIRQ1_SEL_DAC_FIFO_FF							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_FF)
#define TPDAS_A1M_HIRQ1_SEL_DAC_FIFO_PAF						(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAF)
#define TPDAS_A1M_HIRQ1_SEL_DAC_FIFO_EF							(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_EF)
#define TPDAS_A1M_HIRQ1_SEL_DAC_FIFO_PAE						(TPDAS_A1M_get_hirq1_sel()==TPDAS_A1M_HIRQ_SEL_DAC_FIFO_PAE)

#define TPDAS_A1M_GET_HIRQ1_EN									TPDAS_A1M_get_hirq1_en()
#define TPDAS_A1M_HIRQ1_EN_IS_OFF								(TPDAS_A1M_get_hirq1_en()==TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_HIRQ1_EN_IS_ON								(TPDAS_A1M_get_hirq1_en()==TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_GET_HIRQ1_POL									TPDAS_A1M_get_hirq1_pol()
#define TPDAS_A1M_HIRQ1_POL_IS_POS								(TPDAS_A1M_get_hirq1_pol()==TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_HIRQ1_POL_IS_NEG								(TPDAS_A1M_get_hirq1_pol()==TPDAS_A1M_HIRQ_POL_NEG)


		// unified macros with TPDAS_A1M_HIRQ_SEL_...
#define TPDAS_A1M_uget_hirq1_sel()								TPDAS_A1M_get_hirq1_sel()
#define TPDAS_A1M_uset_hirq1_sel(d)								TPDAS_A1M_set_hirq1_sel(d)

		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_hirq1_en()								(TPDAS_A1M_get_hirq1_en()>>TPDAS_A1M_HIRQ_EN_SHIFT)
#define TPDAS_A1M_uset_hirq1_en(d)								TPDAS_A1M_set_hirq1_en((d)<<TPDAS_A1M_HIRQ_EN_SHIFT)

		// unified macros with TPDAS_HIRQ_POL_.. 
#define TPDAS_A1M_uget_hirq1_pol()								TPDAS_A1M_get_hirq1_pol()
#define TPDAS_A1M_uset_hirq1_pol(d)								TPDAS_A1M_set_hirq1_pol(d)



	//-- HIRQ-2 Interrupt control Register
#define TPDAS_A1M_get_hirq2_sel_rg()							((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ2_SEL_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_hirq2_sel_rg(d)							(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ2_SEL_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_hirq2_sel()								(TPDAS_A1M_get_hirq2_sel_rg() & TPDAS_A1M_HIRQ_SEL_BITMASK)
#define TPDAS_A1M_set_hirq2_sel(d)								TPDAS_A1M_set_hirq2_sel_rg((TPDAS_A1M_get_hirq2_sel_rg() & ~TPDAS_A1M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A1M_HIRQ_SEL_BITMASK))
#define TPDAS_A1M_get_hirq2_en()								(TPDAS_A1M_get_hirq2_sel_rg() & TPDAS_A1M_HIRQ_EN_BITMASK)
#define TPDAS_A1M_set_hirq2_en(d)								TPDAS_A1M_set_hirq2_sel_rg((TPDAS_A1M_get_hirq2_sel_rg() & ~TPDAS_A1M_HIRQ_EN_BITMASK) |  (d & TPDAS_A1M_HIRQ_EN_BITMASK))
#define TPDAS_A1M_get_hirq2_pol()								(TPDAS_A1M_get_hirq2_sel_rg() & TPDAS_A1M_HIRQ_POL_BITMASK)
#define TPDAS_A1M_set_hirq2_pol(d)								TPDAS_A1M_set_hirq2_sel_rg((TPDAS_A1M_get_hirq2_sel_rg() & ~TPDAS_A1M_HIRQ_POL_BITMASK) |  (d & TPDAS_A1M_HIRQ_POL_BITMASK))

		// direct set macros
#define TPDAS_A1M_SET_HIRQ2_OFF									TPDAS_A1M_set_hirq2_sel_rg(TPDAS_A1M_HIRQ_SEL_OFF | TPDAS_A1M_HIRQ_EN_OFF)

#define TPDAS_A1M_SET_HIRQ2_EN_OFF								TPDAS_A1M_set_hirq2_en(TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_SET_HIRQ2_EN_ON								TPDAS_A1M_set_hirq2_en(TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_SET_HIRQ2_POL_POS								TPDAS_A1M_set_hirq2_pol(TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_SET_HIRQ2_POL_NEG								TPDAS_A1M_set_hirq2_pol(TPDAS_A1M_HIRQ_POL_NEG)

#define TPDAS_A1M_SET_HIRQ2_SEL_OFF								TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_SET_HIRQ2_SEL_ADC_SYNC_DAQ_END				TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ2_SEL_DAC_SYNC_DAQ_END				TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ2_SEL_ADC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ2_SEL_DAC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ2_SEL_ADC_IRT							TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_SET_HIRQ2_SEL_DAC_IRT							TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_SET_HIRQ2_SEL_GPIO0							TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_SET_HIRQ2_SEL_GPIO1							TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_SET_HIRQ2_SEL_ERR								TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_SET_HIRQ2_SEL_FS								TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_SET_HIRQ2_SEL_ADC_RDY							TPDAS_A1M_set_hirq2_sel(TPDAS_A1M_HIRQ_SEL_ADC_RDY)


		// direct get macros
#define TPDAS_A1M_GET_HIRQ2_SEL									TPDAS_A1M_get_hirq2_sel()
#define TPDAS_A1M_HIRQ2_SEL_IS_OFF								(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_HIRQ2_SEL_IS_ADC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ2_SEL_IS_DAC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ2_SEL_IS_ADC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ2_SEL_IS_DAC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ2_SEL_IS_ADC_IRT							(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_HIRQ2_SEL_IS_DAC_IRT							(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_HIRQ2_SEL_IS_GPIO0							(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_HIRQ2_SEL_IS_GPIO1							(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_HIRQ2_SEL_IS_ERR								(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_HIRQ2_SEL_IS_FS								(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_HIRQ2_SEL_ADC_RDY								(TPDAS_A1M_get_hirq2_sel()==TPDAS_A1M_HIRQ_SEL_ADC_RDY)

#define TPDAS_A1M_GET_HIRQ2_EN									TPDAS_A1M_get_hirq2_en()
#define TPDAS_A1M_HIRQ2_EN_IS_OFF								(TPDAS_A1M_get_hirq2_en()==TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_HIRQ2_EN_IS_ON								(TPDAS_A1M_get_hirq2_en()==TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_GET_HIRQ2_POL									TPDAS_A1M_get_hirq2_pol()
#define TPDAS_A1M_HIRQ2_POL_IS_POS								(TPDAS_A1M_get_hirq2_pol()==TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_HIRQ2_POL_IS_NEG								(TPDAS_A1M_get_hirq2_pol()==TPDAS_A1M_HIRQ_POL_NEG)


		// unified macros with TPDAS_A1M_HIRQ_SEL_...
#define TPDAS_A1M_uget_hirq2_sel()								TPDAS_A1M_get_hirq2_sel()
#define TPDAS_A1M_uset_hirq2_sel(d)								TPDAS_A1M_set_hirq2_sel(d)

		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_hirq2_en()								(TPDAS_A1M_get_hirq2_en()>>TPDAS_A1M_HIRQ_EN_SHIFT)
#define TPDAS_A1M_uset_hirq2_en(d)								TPDAS_A1M_set_hirq2_en((d)<<TPDAS_A1M_HIRQ_EN_SHIFT)

		// unified macros with TPDAS_HIRQ_POL_.. 
#define TPDAS_A1M_uget_hirq2_pol()								TPDAS_A1M_get_hirq2_pol()
#define TPDAS_A1M_uset_hirq2_pol(d)								TPDAS_A1M_set_hirq2_pol(d)



	//-- HIRQ-3 Interrupt control Register
#define TPDAS_A1M_get_hirq3_sel_rg()							((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ3_SEL_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_hirq3_sel_rg(d)							(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_HIRQ3_SEL_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_hirq3_sel()								(TPDAS_A1M_get_hirq3_sel_rg() & TPDAS_A1M_HIRQ_SEL_BITMASK)
#define TPDAS_A1M_set_hirq3_sel(d)								TPDAS_A1M_set_hirq3_sel_rg((TPDAS_A1M_get_hirq3_sel_rg() & ~TPDAS_A1M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A1M_HIRQ_SEL_BITMASK))
#define TPDAS_A1M_get_hirq3_en()								(TPDAS_A1M_get_hirq3_sel_rg() & TPDAS_A1M_HIRQ_EN_BITMASK)
#define TPDAS_A1M_set_hirq3_en(d)								TPDAS_A1M_set_hirq3_sel_rg((TPDAS_A1M_get_hirq3_sel_rg() & ~TPDAS_A1M_HIRQ_EN_BITMASK) |  (d & TPDAS_A1M_HIRQ_EN_BITMASK))
#define TPDAS_A1M_get_hirq3_pol()								(TPDAS_A1M_get_hirq3_sel_rg() & TPDAS_A1M_HIRQ_POL_BITMASK)
#define TPDAS_A1M_set_hirq3_pol(d)								TPDAS_A1M_set_hirq3_sel_rg((TPDAS_A1M_get_hirq3_sel_rg() & ~TPDAS_A1M_HIRQ_POL_BITMASK) |  (d & TPDAS_A1M_HIRQ_POL_BITMASK))

		// direct set macros
#define TPDAS_A1M_SET_HIRQ3_OFF									TPDAS_A1M_set_hirq3_sel_rg(TPDAS_A1M_HIRQ_SEL_OFF | TPDAS_A1M_HIRQ_EN_OFF)

#define TPDAS_A1M_SET_HIRQ3_EN_OFF								TPDAS_A1M_set_hirq3_en(TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_SET_HIRQ3_EN_ON								TPDAS_A1M_set_hirq3_en(TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_SET_HIRQ3_POL_POS								TPDAS_A1M_set_hirq3_pol(TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_SET_HIRQ3_POL_NEG								TPDAS_A1M_set_hirq3_pol(TPDAS_A1M_HIRQ_POL_NEG)

#define TPDAS_A1M_SET_HIRQ3_SEL_OFF								TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_SET_HIRQ3_SEL_ADC_SYNC_DAQ_END				TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ3_SEL_DAC_SYNC_DAQ_END				TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_SET_HIRQ3_SEL_ADC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ3_SEL_DAC_SYNC_DAQ_SSYNC_OK			TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_SET_HIRQ3_SEL_ADC_IRT							TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_SET_HIRQ3_SEL_DAC_IRT							TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_SET_HIRQ3_SEL_GPIO0							TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_SET_HIRQ3_SEL_GPIO1							TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_SET_HIRQ3_SEL_ERR								TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_SET_HIRQ3_SEL_FS								TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_SET_HIRQ3_SEL_ADC_RDY							TPDAS_A1M_set_hirq3_sel(TPDAS_A1M_HIRQ_SEL_ADC_RDY)


		// direct get macros
#define TPDAS_A1M_GET_HIRQ3_SEL									TPDAS_A1M_get_hirq3_sel()
#define TPDAS_A1M_HIRQ3_SEL_IS_OFF								(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_OFF)
#define TPDAS_A1M_HIRQ3_SEL_IS_ADC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ3_SEL_IS_DAC_SYNC_DAQ_END					(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_END)
#define TPDAS_A1M_HIRQ3_SEL_IS_ADC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_ADC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ3_SEL_IS_DAC_SYNC_DAQ_SSYNC_OK			(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_DAC_SYNC_DAQ_SSYNC_OK)
#define TPDAS_A1M_HIRQ3_SEL_IS_ADC_IRT							(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_ADC_IRT)
#define TPDAS_A1M_HIRQ3_SEL_IS_DAC_IRT							(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_DAC_IRT)
#define TPDAS_A1M_HIRQ3_SEL_IS_GPIO0							(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_GPIO0)
#define TPDAS_A1M_HIRQ3_SEL_IS_GPIO1							(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_GPIO1)
#define TPDAS_A1M_HIRQ3_SEL_IS_ERR								(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_ERR)
#define TPDAS_A1M_HIRQ3_SEL_IS_FS								(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_FS)
#define TPDAS_A1M_HIRQ3_SEL_ADC_RDY								(TPDAS_A1M_get_hirq3_sel()==TPDAS_A1M_HIRQ_SEL_ADC_RDY)

#define TPDAS_A1M_GET_HIRQ3_EN									TPDAS_A1M_get_hirq3_en()
#define TPDAS_A1M_HIRQ3_EN_IS_OFF								(TPDAS_A1M_get_hirq3_en()==TPDAS_A1M_HIRQ_EN_OFF)
#define TPDAS_A1M_HIRQ3_EN_IS_ON								(TPDAS_A1M_get_hirq3_en()==TPDAS_A1M_HIRQ_EN_ON)

#define TPDAS_A1M_GET_HIRQ3_POL									TPDAS_A1M_get_hirq3_pol()
#define TPDAS_A1M_HIRQ3_POL_IS_POS								(TPDAS_A1M_get_hirq3_pol()==TPDAS_A1M_HIRQ_POL_POS)
#define TPDAS_A1M_HIRQ3_POL_IS_NEG								(TPDAS_A1M_get_hirq3_pol()==TPDAS_A1M_HIRQ_POL_NEG)


		// unified macros with TPDAS_A1M_HIRQ_SEL_...
#define TPDAS_A1M_uget_hirq3_sel()								TPDAS_A1M_get_hirq3_sel()
#define TPDAS_A1M_uset_hirq3_sel(d)								TPDAS_A1M_set_hirq3_sel(d)

		// unified macros with ON/OFF 
#define TPDAS_A1M_uget_hirq3_en()								(TPDAS_A1M_get_hirq3_en()>>TPDAS_A1M_HIRQ_EN_SHIFT)
#define TPDAS_A1M_uset_hirq3_en(d)								TPDAS_A1M_set_hirq3_en((d)<<TPDAS_A1M_HIRQ_EN_SHIFT)

		// unified macros with TPDAS_HIRQ_POL_.. 
#define TPDAS_A1M_uget_hirq3_pol()								TPDAS_A1M_get_hirq3_pol()
#define TPDAS_A1M_uset_hirq3_pol(d)								TPDAS_A1M_set_hirq3_pol(d)




	//-- HIRQ ERR Aux Interrupt Mask (XIM) Register
#define TPDAS_A1M_get_xim_err_rg()								((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_XIM_ERR_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_xim_err_rg(d)								(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_XIM_ERR_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_xim_adc_fifo_ovf()						(TPDAS_A1M_get_xim_err_rg() & TPDAS_A1M_XIM_ADC_FIFO_OVF_BITMASK)
#define TPDAS_A1M_set_xim_adc_fifo_ovf(d)						TPDAS_A1M_set_xim_err_rg((TPDAS_A1M_get_xim_err_rg() & ~TPDAS_A1M_XIM_ADC_FIFO_OVF_BITMASK) |  (d & TPDAS_A1M_XIM_ADC_FIFO_OVF_BITMASK))

#define TPDAS_A1M_get_xim_adc_fifo_unf()						(TPDAS_A1M_get_xim_err_rg() & TPDAS_A1M_XIM_ADC_FIFO_UNF_BITMASK)
#define TPDAS_A1M_set_xim_adc_fifo_unf(d)						TPDAS_A1M_set_xim_err_rg((TPDAS_A1M_get_xim_err_rg() & ~TPDAS_A1M_XIM_ADC_FIFO_UNF_BITMASK) |  (d & TPDAS_A1M_XIM_ADC_FIFO_UNF_BITMASK))

#define TPDAS_A1M_get_xim_dac_fifo_ovf()						(TPDAS_A1M_get_xim_err_rg() & TPDAS_A1M_XIM_DAC_FIFO_OVF_BITMASK)
#define TPDAS_A1M_set_xim_dac_fifo_ovf(d)						TPDAS_A1M_set_xim_err_rg((TPDAS_A1M_get_xim_err_rg() & ~TPDAS_A1M_XIM_DAC_FIFO_OVF_BITMASK) |  (d & TPDAS_A1M_XIM_DAC_FIFO_OVF_BITMASK))

#define TPDAS_A1M_get_xim_dac_fifo_unf()						(TPDAS_A1M_get_xim_err_rg() & TPDAS_A1M_XIM_DAC_FIFO_UNF_BITMASK)
#define TPDAS_A1M_set_xim_dac_fifo_unf(d)						TPDAS_A1M_set_xim_err_rg((TPDAS_A1M_get_xim_err_rg() & ~TPDAS_A1M_XIM_DAC_FIFO_UNF_BITMASK) |  (d & TPDAS_A1M_XIM_DAC_FIFO_UNF_BITMASK))

#define TPDAS_A1M_get_xim_adc_irt_err()							(TPDAS_A1M_get_xim_err_rg() & TPDAS_A1M_XIM_ADC_IRT_ERR_BITMASK)
#define TPDAS_A1M_set_xim_adc_irt_err(d)						TPDAS_A1M_set_xim_err_rg((TPDAS_A1M_get_xim_err_rg() & ~TPDAS_A1M_XIM_ADC_IRT_ERR_BITMASK) |  (d & TPDAS_A1M_XIM_ADC_IRT_ERR_BITMASK))

#define TPDAS_A1M_get_xim_dac_irt_err()							(TPDAS_A1M_get_xim_err_rg() & TPDAS_A1M_XIM_DAC_IRT_ERR_BITMASK)
#define TPDAS_A1M_set_xim_dac_irt_err(d)						TPDAS_A1M_set_xim_err_rg((TPDAS_A1M_get_xim_err_rg() & ~TPDAS_A1M_XIM_DAC_IRT_ERR_BITMASK) |  (d & TPDAS_A1M_XIM_DAC_IRT_ERR_BITMASK))

		// direct set/get macros
#define TPDAS_A1M_SET_XIM_ADC_FIFO_OVF_OFF						TPDAS_A1M_set_xim_adc_fifo_ovf(TPDAS_A1M_XIM_ADC_FIFO_OVF_OFF)
#define TPDAS_A1M_SET_XIM_ADC_FIFO_OVF_ON						TPDAS_A1M_set_xim_adc_fifo_ovf(TPDAS_A1M_XIM_ADC_FIFO_OVF_ON)

#define TPDAS_A1M_SET_XIM_ADC_FIFO_UNF_OFF						TPDAS_A1M_set_xim_adc_fifo_unf(TPDAS_A1M_XIM_ADC_FIFO_UNF_OFF)
#define TPDAS_A1M_SET_XIM_ADC_FIFO_UNF_ON						TPDAS_A1M_set_xim_adc_fifo_unf(TPDAS_A1M_XIM_ADC_FIFO_UNF_ON)

#define TPDAS_A1M_SET_XIM_DAC_FIFO_OVF_OFF						TPDAS_A1M_set_xim_dac_fifo_ovf(TPDAS_A1M_XIM_DAC_FIFO_OVF_OFF)
#define TPDAS_A1M_SET_XIM_DAC_FIFO_OVF_ON						TPDAS_A1M_set_xim_dac_fifo_ovf(TPDAS_A1M_XIM_DAC_FIFO_OVF_ON)

#define TPDAS_A1M_SET_XIM_DAC_FIFO_UNF_OFF						TPDAS_A1M_set_xim_dac_fifo_unf(TPDAS_A1M_XIM_DAC_FIFO_UNF_OFF)
#define TPDAS_A1M_SET_XIM_DAC_FIFO_UNF_ON						TPDAS_A1M_set_xim_dac_fifo_unf(TPDAS_A1M_XIM_DAC_FIFO_UNF_ON)

#define TPDAS_A1M_SET_XIM_ADC_IRT_ERR_OFF						TPDAS_A1M_set_xim_adc_irt_err(TPDAS_A1M_XIM_ADC_IRT_ERR_OFF)
#define TPDAS_A1M_SET_XIM_ADC_IRT_ERR_ON						TPDAS_A1M_set_xim_adc_irt_err(TPDAS_A1M_XIM_ADC_IRT_ERR_ON)

#define TPDAS_A1M_SET_XIM_DAC_IRT_ERR_OFF						TPDAS_A1M_set_xim_adc_irt_err(TPDAS_A1M_XIM_DAC_IRT_ERR_OFF)
#define TPDAS_A1M_SET_XIM_DAC_IRT_ERR_ON						TPDAS_A1M_set_xim_adc_irt_err(TPDAS_A1M_XIM_DAC_IRT_ERR_ON)

#define TPDAS_A1M_GET_XIM_ADC_FIFO_OVF							TPDAS_A1M_get_xim_adc_fifo_ovf()	
#define TPDAS_A1M_XIM_ADC_FIFO_OVF_IS_OFF						(TPDAS_A1M_get_xim_adc_fifo_ovf()==TPDAS_A1M_XIM_ADC_FIFO_OVF_OFF)
#define TPDAS_A1M_XIM_ADC_FIFO_OVF_IS_ON						(TPDAS_A1M_get_xim_adc_fifo_ovf()==TPDAS_A1M_XIM_ADC_FIFO_OVF_ON)

#define TPDAS_A1M_GET_XIM_ADC_FIFO_UNF							TPDAS_A1M_get_xim_adc_fifo_unf()	
#define TPDAS_A1M_XIM_ADC_FIFO_UNF_IS_OFF						(TPDAS_A1M_get_xim_adc_fifo_unf()==TPDAS_A1M_XIM_ADC_FIFO_UNF_OFF)
#define TPDAS_A1M_XIM_ADC_FIFO_UNF_IS_ON						(TPDAS_A1M_get_xim_adc_fifo_unf()==TPDAS_A1M_XIM_ADC_FIFO_UNF_ON)

#define TPDAS_A1M_GET_XIM_DAC_FIFO_OVF							TPDAS_A1M_get_xim_dac_fifo_ovf()	
#define TPDAS_A1M_XIM_DAC_FIFO_OVF_IS_OFF						(TPDAS_A1M_get_xim_dac_fifo_ovf()==TPDAS_A1M_XIM_DAC_FIFO_OVF_OFF)
#define TPDAS_A1M_XIM_DAC_FIFO_OVF_IS_ON						(TPDAS_A1M_get_xim_dac_fifo_ovf()==TPDAS_A1M_XIM_DAC_FIFO_OVF_ON)

#define TPDAS_A1M_GET_XIM_DAC_FIFO_UNF							TPDAS_A1M_get_xim_dac_fifo_unf()	
#define TPDAS_A1M_XIM_DAC_FIFO_UNF_IS_OFF						(TPDAS_A1M_get_xim_dac_fifo_unf()==TPDAS_A1M_XIM_DAC_FIFO_UNF_OFF)
#define TPDAS_A1M_XIM_DAC_FIFO_UNF_IS_ON						(TPDAS_A1M_get_xim_dac_fifo_unf()==TPDAS_A1M_XIM_DAC_FIFO_UNF_ON)

#define TPDAS_A1M_GET_XIM_ADC_IRT_ERR							TPDAS_A1M_get_xim_adc_irt_err()
#define TPDAS_A1M_XIM_ADC_IRT_ERR_IS_OFF						(TPDAS_A1M_get_xim_adc_irt_err()==TPDAS_A1M_XIM_ADC_IRT_ERR_OFF)
#define TPDAS_A1M_XIM_ADC_IRT_ERR_IS_ON							(TPDAS_A1M_get_xim_adc_irt_err()==TPDAS_A1M_XIM_ADC_IRT_ERR_ON)

#define TPDAS_A1M_GET_XIM_DAC_IRT_ERR							TPDAS_A1M_get_xim_dac_irt_err()
#define TPDAS_A1M_XIM_DAC_IRT_ERR_IS_OFF						(TPDAS_A1M_get_xim_dac_irt_err()==TPDAS_A1M_XIM_DAC_IRT_ERR_OFF)
#define TPDAS_A1M_XIM_DAC_IRT_ERR_IS_ON							(TPDAS_A1M_get_xim_dac_irt_err()==TPDAS_A1M_XIM_DAC_IRT_ERR_ON)

		// unified macros witgh ON/OFF
#define TPDAS_A1M_uget_xim_adc_fifo_ovf()						TPDAS_A1M_get_xim_adc_fifo_ovf()
#define TPDAS_A1M_uset_xim_adc_fifo_ovf(d)						TPDAS_A1M_set_xim_adc_fifo_ovf(d)

#define TPDAS_A1M_uget_xim_adc_fifo_unf()						(TPDAS_A1M_get_xim_adc_fifo_unf()>>TPDAS_A1M_XIM_ADC_FIFO_UNF_BITSHIFT)
#define TPDAS_A1M_uset_xim_adc_fifo_unf(d)						TPDAS_A1M_set_xim_adc_fifo_unf((d)<<TPDAS_A1M_XIM_ADC_FIFO_UNF_BITSHIFT)

#define TPDAS_A1M_uget_xim_dac_fifo_ovf()						(TPDAS_A1M_get_xim_dac_fifo_ovf()>>TPDAS_A1M_XIM_DAC_FIFO_OVF_BITSHIFT)
#define TPDAS_A1M_uset_xim_dac_fifo_ovf(d)						TPDAS_A1M_set_xim_dac_fifo_ovf((d)<<TPDAS_A1M_XIM_DAC_FIFO_OVF_BITSHIFT)

#define TPDAS_A1M_uget_xim_dac_fifo_unf()						(TPDAS_A1M_get_xim_dac_fifo_unf()>>TPDAS_A1M_XIM_DAC_FIFO_UNF_BITSHIFT)
#define TPDAS_A1M_uset_xim_dac_fifo_unf(d)						TPDAS_A1M_set_xim_dac_fifo_unf((d)<<TPDAS_A1M_XIM_DAC_FIFO_UNF_BITSHIFT)

#define TPDAS_A1M_uget_xim_adc_irt_err()						(TPDAS_A1M_get_xim_adc_irt_err()>>TPDAS_A1M_XIM_ADC_IRT_ERR_BITSHIFT)
#define TPDAS_A1M_uset_xim_adc_irt_err(d)						TPDAS_A1M_set_xim_adc_irt_err((d)<<TPDAS_A1M_XIM_ADC_IRT_ERR_BITSHIFT)

#define TPDAS_A1M_uget_xim_dac_irt_err()						(TPDAS_A1M_get_xim_dac_irt_err()>>TPDAS_A1M_XIM_DAC_IRT_ERR_BITSHIFT)
#define TPDAS_A1M_uset_xim_dac_irt_err(d)						TPDAS_A1M_set_xim_dac_irt_err((d)<<TPDAS_A1M_XIM_DAC_IRT_ERR_BITSHIFT)



	//--- GPIO control registers
#define TPDAS_A1M_get_gpio_dir_rg()					((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_GPIO_DIR_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_gpio_dir_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_GPIO_DIR_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_get_gpio_data_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_GPIO_DATA_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_gpio_data_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_GPIO_DATA_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_gpio0_dir()					(TPDAS_A1M_get_gpio_dir_rg() & TPDAS_A1M_GPIO0_DIR_BITMASK)
#define TPDAS_A1M_set_gpio0_dir(d)					TPDAS_A1M_set_gpio_dir_rg((TPDAS_A1M_get_gpio_dir_rg() & ~TPDAS_A1M_GPIO0_DIR_BITMASK) |  (d & TPDAS_A1M_GPIO0_DIR_BITMASK))
#define TPDAS_A1M_get_gpio1_dir()					(TPDAS_A1M_get_gpio_dir_rg() & TPDAS_A1M_GPIO1_DIR_BITMASK)
#define TPDAS_A1M_set_gpio1_dir(d)					TPDAS_A1M_set_gpio_dir_rg((TPDAS_A1M_get_gpio_dir_rg() & ~TPDAS_A1M_GPIO1_DIR_BITMASK) |  (d & TPDAS_A1M_GPIO1_DIR_BITMASK))
#define TPDAS_A1M_get_gpio2_dir()					(TPDAS_A1M_get_gpio_dir_rg() & TPDAS_A1M_GPIO2_DIR_BITMASK)
#define TPDAS_A1M_set_gpio2_dir(d)					TPDAS_A1M_set_gpio_dir_rg((TPDAS_A1M_get_gpio_dir_rg() & ~TPDAS_A1M_GPIO2_DIR_BITMASK) |  (d & TPDAS_A1M_GPIO2_DIR_BITMASK))
#define TPDAS_A1M_get_gpio3_dir()					(TPDAS_A1M_get_gpio_dir_rg() & TPDAS_A1M_GPIO3_DIR_BITMASK)
#define TPDAS_A1M_set_gpio3_dir(d)					TPDAS_A1M_set_gpio_dir_rg((TPDAS_A1M_get_gpio_dir_rg() & ~TPDAS_A1M_GPIO3_DIR_BITMASK) |  (d & TPDAS_A1M_GPIO3_DIR_BITMASK))

#define TPDAS_A1M_get_gpio0_data()					(TPDAS_A1M_get_gpio_data_rg() & TPDAS_A1M_GPIO0_DATA_BITMASK)
#define TPDAS_A1M_set_gpio0_data(d)					TPDAS_A1M_set_gpio_data_rg((TPDAS_A1M_get_gpio_data_rg() & ~TPDAS_A1M_GPIO0_DATA_BITMASK) |  (d & TPDAS_A1M_GPIO0_DATA_BITMASK))
#define TPDAS_A1M_get_gpio1_data()					(TPDAS_A1M_get_gpio_data_rg() & TPDAS_A1M_GPIO1_DATA_BITMASK)
#define TPDAS_A1M_set_gpio1_data(d)					TPDAS_A1M_set_gpio_data_rg((TPDAS_A1M_get_gpio_data_rg() & ~TPDAS_A1M_GPIO1_DATA_BITMASK) |  (d & TPDAS_A1M_GPIO1_DATA_BITMASK))
#define TPDAS_A1M_get_gpio2_data()					(TPDAS_A1M_get_gpio_data_rg() & TPDAS_A1M_GPIO2_DATA_BITMASK)
#define TPDAS_A1M_set_gpio2_data(d)					TPDAS_A1M_set_gpio_data_rg((TPDAS_A1M_get_gpio_data_rg() & ~TPDAS_A1M_GPIO2_DATA_BITMASK) |  (d & TPDAS_A1M_GPIO2_DATA_BITMASK))
#define TPDAS_A1M_get_gpio3_data()					(TPDAS_A1M_get_gpio_data_rg() & TPDAS_A1M_GPIO3_DATA_BITMASK)
#define TPDAS_A1M_set_gpio3_data(d)					TPDAS_A1M_set_gpio_data_rg((TPDAS_A1M_get_gpio_data_rg() & ~TPDAS_A1M_GPIO3_DATA_BITMASK) |  (d & TPDAS_A1M_GPIO3_DATA_BITMASK))

		// direct set/get macros
#define TPDAS_A1M_SET_GPIO0_DIR_INPUT				TPDAS_A1M_set_gpio0_dir(TPDAS_A1M_GPIO0_DIR_INPUT)
#define TPDAS_A1M_SET_GPIO0_DIR_OUTPUT				TPDAS_A1M_set_gpio0_dir(TPDAS_A1M_GPIO0_DIR_OUTPUT)
#define TPDAS_A1M_SET_GPIO1_DIR_INPUT				TPDAS_A1M_set_gpio1_dir(TPDAS_A1M_GPIO1_DIR_INPUT)
#define TPDAS_A1M_SET_GPIO1_DIR_OUTPUT				TPDAS_A1M_set_gpio1_dir(TPDAS_A1M_GPIO1_DIR_OUTPUT)
#define TPDAS_A1M_SET_GPIO2_DIR_INPUT				TPDAS_A1M_set_gpio2_dir(TPDAS_A1M_GPIO2_DIR_INPUT)
#define TPDAS_A1M_SET_GPIO2_DIR_OUTPUT				TPDAS_A1M_set_gpio2_dir(TPDAS_A1M_GPIO2_DIR_OUTPUT)
#define TPDAS_A1M_SET_GPIO3_DIR_INPUT				TPDAS_A1M_set_gpio3_dir(TPDAS_A1M_GPIO3_DIR_INPUT)
#define TPDAS_A1M_SET_GPIO3_DIR_OUTPUT				TPDAS_A1M_set_gpio3_dir(TPDAS_A1M_GPIO3_DIR_OUTPUT)

#define TPDAS_A1M_SET_GPIO0_DATA_OFF				TPDAS_A1M_set_gpio0_data(TPDAS_A1M_GPIO0_DATA_OFF)
#define TPDAS_A1M_SET_GPIO0_DATA_ON					TPDAS_A1M_set_gpio0_data(TPDAS_A1M_GPIO0_DATA_ON)
#define TPDAS_A1M_SET_GPIO1_DATA_OFF				TPDAS_A1M_set_gpio1_data(TPDAS_A1M_GPIO1_DATA_OFF)
#define TPDAS_A1M_SET_GPIO1_DATA_ON					TPDAS_A1M_set_gpio1_data(TPDAS_A1M_GPIO1_DATA_ON)
#define TPDAS_A1M_SET_GPIO2_DATA_OFF				TPDAS_A1M_set_gpio2_data(TPDAS_A1M_GPIO2_DATA_OFF)
#define TPDAS_A1M_SET_GPIO2_DATA_ON					TPDAS_A1M_set_gpio2_data(TPDAS_A1M_GPIO2_DATA_ON)
#define TPDAS_A1M_SET_GPIO3_DATA_OFF				TPDAS_A1M_set_gpio3_data(TPDAS_A1M_GPIO3_DATA_OFF)
#define TPDAS_A1M_SET_GPIO3_DATA_ON					TPDAS_A1M_set_gpio3_data(TPDAS_A1M_GPIO3_DATA_ON)

#define TPDAS_A1M_GET_GPIO0_DIR						TPDAS_A1M_get_gpio0_dir()
#define TPDAS_A1M_GPIO0_DIR_IS_INPUT				(TPDAS_A1M_get_gpio0_dir()==TPDAS_A1M_GPIO0_DIR_INPUT)
#define TPDAS_A1M_GPIO0_DIR_IS_OUTPUT				(TPDAS_A1M_get_gpio0_dir()==TPDAS_A1M_GPIO0_DIR_OUTPUT)

#define TPDAS_A1M_GET_GPIO1_DIR						TPDAS_A1M_get_gpio1_dir()
#define TPDAS_A1M_GPIO1_DIR_IS_INPUT				(TPDAS_A1M_get_gpio1_dir()==TPDAS_A1M_GPIO1_DIR_INPUT)
#define TPDAS_A1M_GPIO1_DIR_IS_OUTPUT				(TPDAS_A1M_get_gpio1_dir()==TPDAS_A1M_GPIO1_DIR_OUTPUT)

#define TPDAS_A1M_GET_GPIO2_DIR						TPDAS_A1M_get_gpio2_dir()
#define TPDAS_A1M_GPIO2_DIR_IS_INPUT				(TPDAS_A1M_get_gpio2_dir()==TPDAS_A1M_GPIO2_DIR_INPUT)
#define TPDAS_A1M_GPIO2_DIR_IS_OUTPUT				(TPDAS_A1M_get_gpio2_dir()==TPDAS_A1M_GPIO2_DIR_OUTPUT)

#define TPDAS_A1M_GET_GPIO3_DIR						TPDAS_A1M_get_gpio3_dir()
#define TPDAS_A1M_GPIO3_DIR_IS_INPUT				(TPDAS_A1M_get_gpio3_dir()==TPDAS_A1M_GPIO3_DIR_INPUT)
#define TPDAS_A1M_GPIO3_DIR_IS_OUTPUT				(TPDAS_A1M_get_gpio3_dir()==TPDAS_A1M_GPIO3_DIR_OUTPUT)

#define TPDAS_A1M_GET_GPIO0_DATA					TPDAS_A1M_get_gpio0_data()
#define TPDAS_A1M_GPIO0_DATA_IS_OFF					(TPDAS_A1M_get_gpio0_data()==TPDAS_A1M_GPIO0_DATA_OFF)
#define TPDAS_A1M_GPIO0_DATA_IS_ON					(TPDAS_A1M_get_gpio0_data()==TPDAS_A1M_GPIO0_DATA_ON)

#define TPDAS_A1M_GET_GPIO1_DATA					TPDAS_A1M_get_gpio1_data()
#define TPDAS_A1M_GPIO1_DATA_IS_OFF					(TPDAS_A1M_get_gpio1_data()==TPDAS_A1M_GPIO1_DATA_OFF)
#define TPDAS_A1M_GPIO1_DATA_IS_ON					(TPDAS_A1M_get_gpio1_data()==TPDAS_A1M_GPIO1_DATA_ON)

#define TPDAS_A1M_GET_GPIO2_DATA					TPDAS_A1M_get_gpio2_data()
#define TPDAS_A1M_GPIO2_DATA_IS_OFF					(TPDAS_A1M_get_gpio2_data()==TPDAS_A1M_GPIO2_DATA_OFF)
#define TPDAS_A1M_GPIO2_DATA_IS_ON					(TPDAS_A1M_get_gpio2_data()==TPDAS_A1M_GPIO2_DATA_ON)

#define TPDAS_A1M_GET_GPIO3_DATA					TPDAS_A1M_get_gpio3_data()
#define TPDAS_A1M_GPIO3_DATA_IS_OFF					(TPDAS_A1M_get_gpio3_data()==TPDAS_A1M_GPIO3_DATA_OFF)
#define TPDAS_A1M_GPIO3_DATA_IS_ON					(TPDAS_A1M_get_gpio3_data()==TPDAS_A1M_GPIO3_DATA_ON)

		// unified GPIO DIR macros with TPDAS_A1M_GPIO_DIR_INPUT/_OUTPUT
#define TPDAS_A1M_uget_gpio0_dir()					TPDAS_A1M_get_gpio0_dir()
#define TPDAS_A1M_uset_gpio0_dir(d)					TPDAS_A1M_set_gpio0_dir(d)
#define TPDAS_A1M_uget_gpio1_dir()					(TPDAS_A1M_get_gpio1_dir()>>TPDAS_A1M_GPIO1_DIR_BITSHIFT)
#define TPDAS_A1M_uset_gpio1_dir(d)					TPDAS_A1M_set_gpio1_dir((d)<<TPDAS_A1M_GPIO1_DIR_BITSHIFT)
#define TPDAS_A1M_uget_gpio2_dir()					(TPDAS_A1M_get_gpio2_dir()>>TPDAS_A1M_GPIO2_DIR_BITSHIFT)
#define TPDAS_A1M_uset_gpio2_dir(d)					TPDAS_A1M_set_gpio2_dir((d)<<TPDAS_A1M_GPIO2_DIR_BITSHIFT)
#define TPDAS_A1M_uget_gpio3_dir()					(TPDAS_A1M_get_gpio3_dir()>>TPDAS_A1M_GPIO3_DIR_BITSHIFT)
#define TPDAS_A1M_uset_gpio3_dir(d)					TPDAS_A1M_set_gpio3_dir((d)<<TPDAS_A1M_GPIO3_DIR_BITSHIFT)

		// unified GPIO DATA macros with ON/OFF
#define TPDAS_A1M_uget_gpio0_data()					TPDAS_A1M_get_gpio0_data()
#define TPDAS_A1M_uset_gpio0_data(d)				TPDAS_A1M_set_gpio0_data(d)
#define TPDAS_A1M_uget_gpio1_data()					(TPDAS_A1M_get_gpio1_data()>>TPDAS_A1M_GPIO1_DATA_BITSHIFT)
#define TPDAS_A1M_uset_gpio1_data(d)				TPDAS_A1M_set_gpio1_data((d)<<TPDAS_A1M_GPIO1_DATA_BITSHIFT)
#define TPDAS_A1M_uget_gpio2_data()					(TPDAS_A1M_get_gpio2_data()>>TPDAS_A1M_GPIO2_DATA_BITSHIFT)
#define TPDAS_A1M_uset_gpio2_data(d)				TPDAS_A1M_set_gpio2_data((d)<<TPDAS_A1M_GPIO2_DATA_BITSHIFT)
#define TPDAS_A1M_uget_gpio3_data()					(TPDAS_A1M_get_gpio3_data()>>TPDAS_A1M_GPIO3_DATA_BITSHIFT)
#define TPDAS_A1M_uset_gpio3_data(d)				TPDAS_A1M_set_gpio3_data((d)<<TPDAS_A1M_GPIO3_DATA_BITSHIFT)



	//--- PFG CNTR#1..3 registers
#define TPDAS_A1M_get_pfg_cntr1_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_PFG_CNTR1_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_pfg_cntr1_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_PFG_CNTR1_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_get_pfg_cntr2_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_PFG_CNTR2_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_pfg_cntr2_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_PFG_CNTR2_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_get_pfg_cntr3_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_PFG_CNTR3_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_pfg_cntr3_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_PFG_CNTR3_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

		// for convinience, GET/SET functions for numeric 'V' ans 'R' fields are combined to the 
		// corresponding number of digits and are aligned at bit #0.
#define TPDAS_A1M_get_pfg_v()						((TPDAS_A1M_get_pfg_cntr1_rg() & TPDAS_A1M_PFG_V07_BITMASK) | ((TPDAS_A1M_get_pfg_cntr2_rg() & TPDAS_A1M_PFG_V8_BITMASK)<<8))
#define TPDAS_A1M_set_pfg_v(d)						{TPDAS_A1M_set_pfg_cntr1_rg(d & 0xff); TPDAS_A1M_set_pfg_cntr2_rg((TPDAS_A1M_get_pfg_cntr2_rg() & ~TPDAS_A1M_PFG_V8_BITMASK) |  ((d>>8) & TPDAS_A1M_PFG_V8_BITMASK));}
#define TPDAS_A1M_get_pfg_r()						((TPDAS_A1M_get_pfg_cntr2_rg() & TPDAS_A1M_PFG_R_BITMASK)>>TPDAS_A1M_PFG_R_BITSHIFT)
#define TPDAS_A1M_set_pfg_r(d)						TPDAS_A1M_set_pfg_cntr2_rg((TPDAS_A1M_get_pfg_cntr2_rg() & ~TPDAS_A1M_PFG_R_BITMASK) |  (((d)<<TPDAS_A1M_PFG_R_BITSHIFT) & TPDAS_A1M_PFG_R_BITMASK))
#define TPDAS_A1M_get_pfg_s()						(TPDAS_A1M_get_pfg_cntr3_rg() & TPDAS_A1M_PFG_S_BITMASK)
#define TPDAS_A1M_set_pfg_s(d)						TPDAS_A1M_set_pfg_cntr3_rg((TPDAS_A1M_get_pfg_cntr3_rg() & ~TPDAS_A1M_PFG_S_BITMASK) |  (d & TPDAS_A1M_PFG_S_BITMASK))
#define TPDAS_A1M_get_pfg_x()						(TPDAS_A1M_get_pfg_cntr3_rg() & TPDAS_A1M_PFG_X_BITMASK)
#define TPDAS_A1M_set_pfg_x(d)						TPDAS_A1M_set_pfg_cntr3_rg((TPDAS_A1M_get_pfg_cntr3_rg() & ~TPDAS_A1M_PFG_X_BITMASK) |  (d & TPDAS_A1M_PFG_X_BITMASK))

		// direct set/get macros
#define TPDAS_A1M_GET_PFG_V							TPDAS_A1M_get_pfg_v()
#define TPDAS_A1M_SET_PFG_V(d)						TPDAS_A1M_set_pfg_v(d)

#define TPDAS_A1M_GET_PFG_R							TPDAS_A1M_get_pfg_r()
#define TPDAS_A1M_SET_PFG_R(d)						TPDAS_A1M_set_pfg_r(d)

#define TPDAS_A1M_SET_PFG_X(d)						TPDAS_A1M_set_pfg_x(d)
#define TPDAS_A1M_SET_PFG_X_DIV1					TPDAS_A1M_set_pfg_x(TPDAS_A1M_PFG_X_DIV1)
#define TPDAS_A1M_SET_PFG_X_DIV32					TPDAS_A1M_set_pfg_x(TPDAS_A1M_PFG_X_DIV32)
#define TPDAS_A1M_SET_PFG_X_DIV1024					TPDAS_A1M_set_pfg_x(TPDAS_A1M_PFG_X_DIV1024)

#define TPDAS_A1M_GET_PFG_X							TPDAS_A1M_get_pfg_x()
#define TPDAS_A1M_PFG_X_IS_DIV1						(TPDAS_A1M_get_pfg_x()==TPDAS_A1M_PFG_X_DIV1)
#define TPDAS_A1M_PFG_X_IS_DIV32					(TPDAS_A1M_get_pfg_x()==TPDAS_A1M_PFG_X_DIV32)
#define TPDAS_A1M_PFG_X_IS_DIV1024					(TPDAS_A1M_get_pfg_x()==TPDAS_A1M_PFG_X_DIV1024)

#define TPDAS_A1M_SET_PFG_S(d)						TPDAS_A1M_set_pfg_s(d)
#define TPDAS_A1M_SET_PFG_S_DIV2					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV2)
#define TPDAS_A1M_SET_PFG_S_DIV4					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV4)
#define TPDAS_A1M_SET_PFG_S_DIV5					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV5)
#define TPDAS_A1M_SET_PFG_S_DIV6					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV6)
#define TPDAS_A1M_SET_PFG_S_DIV7					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV7)
#define TPDAS_A1M_SET_PFG_S_DIV8					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV8)
#define TPDAS_A1M_SET_PFG_S_DIV9					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV9)
#define TPDAS_A1M_SET_PFG_S_DIV10					TPDAS_A1M_set_pfg_s(TPDAS_A1M_PFG_S_DIV10)

#define TPDAS_A1M_GET_PFG_S							TPDAS_A1M_get_pfg_s()
#define TPDAS_A1M_PFG_S_IS_DIV2						(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV2)
#define TPDAS_A1M_PFG_S_IS_DIV4						(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV4)
#define TPDAS_A1M_PFG_S_IS_DIV5						(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV5)
#define TPDAS_A1M_PFG_S_IS_DIV6						(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV6)
#define TPDAS_A1M_PFG_S_IS_DIV7						(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV7)
#define TPDAS_A1M_PFG_S_IS_DIV8						(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV8)
#define TPDAS_A1M_PFG_S_IS_DIV9						(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV9)
#define TPDAS_A1M_PFG_S_IS_DIV10					(TPDAS_A1M_get_pfg_s()==TPDAS_A1M_PFG_S_DIV10)

		// combined PFG programmaing macros
			// sets PFG field values
#define TPDAS_A1M_SET_PFG(v,r,s,x)					{TPDAS_A1M_SET_PFG_V(v);TPDAS_A1M_SET_PFG_R(r);TPDAS_A1M_SET_PFG_S(s);TPDAS_A1M_SET_PFG_X(x);}
			// macros to set PFG V/R/S/X fields, which are updated by PFG build in +8 and +2 shifters for V and R fields
#define TPDAS_A1M_SET_PFG_EXACTLY(v,r,s,x)			{TPDAS_A1M_SET_PFG_V(v-8);TPDAS_A1M_SET_PFG_R(r-2);TPDAS_A1M_SET_PFG_S(s);TPDAS_A1M_SET_PFG_X(x);}

		// unified macros with numeric values
#define TPDAS_A1M_uget_pfg_v()						TPDAS_A1M_get_pfg_v()
#define TPDAS_A1M_uset_pfg_v(d)						TPDAS_A1M_set_pfg_v(d)
#define TPDAS_A1M_uget_pfg_r()						TPDAS_A1M_get_pfg_r()
#define TPDAS_A1M_uset_pfg_r(d)						TPDAS_A1M_set_pfg_r(d)
#define TPDAS_A1M_uget_pfg_s()						TPDAS_A1M_get_pfg_s()
#define TPDAS_A1M_uset_pfg_s(d)						TPDAS_A1M_set_pfg_s(d)
#define TPDAS_A1M_uget_pfg_x()						TPDAS_A1M_get_pfg_x()
#define TPDAS_A1M_uset_pfg_x(d)						TPDAS_A1M_set_pfg_x(d)

//==============================================================================================


//=============== macros for accessing ADC/DAC direct and FIFO data  ===========================

	//~~~ programming ADC/DAC FIFO PAF/PAE flags in FIFO-CNF mode (ADC/DAC ASYNC DAQ mode)
	//    (only bit D0 is valid when writing to FIFO SIN register)
#define TPDAS_A1M_set_adc_fifo_sin_rg(d)			(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_FIFO_SIN_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_set_dac_fifo_sin_rg(d)			(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC_FIFO_SIN_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_set_adc_fifo_sin(d)				TPDAS_A1M_set_adc_fifo_sin_rg(d)			
#define TPDAS_A1M_set_dac_fifo_sin(d)				TPDAS_A1M_set_dac_fifo_sin_rg(d)			

	//~~~ ADC/DAC FIFO data access macros
#define TPDAS_A1M_get_adc_fifo_data_rg()			((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_FIFO_DATA_RG_ADDR) & TPDAS_A1M_ADDA_FIFO_DATA_BITMASK)
#define TPDAS_A1M_get_adc_fifo_data_msw_rg()		((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_FIFO_DATA_MSW_RG_ADDR) & TPDAS_A1M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A1M_set_dac_fifo_data_rg(d)			(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC_FIFO_DATA_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc_fifo_data()				TPDAS_A1M_get_adc_fifo_data_rg()			
#define TPDAS_A1M_get_adc_fifo_data_eos()			(TPDAS_A1M_get_adc_fifo_data_msw_rg() & TPDAS_A1M_ADC_FIFO_MSW_EOS_BITMASK)

#define TPDAS_A1M_set_dac_fifo_data(d)				TPDAS_A1M_set_dac_fifo_data_rg(d)			

	//~~~ ADC/DAC direct access via PIOX
#define TPDAS_A1M_get_adc0_data_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC0_DATA_RG_ADDR) & TPDAS_A1M_ADDA_DATA_BITMASK)
#define TPDAS_A1M_get_adc1_data_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC1_DATA_RG_ADDR) & TPDAS_A1M_ADDA_DATA_BITMASK)
#define TPDAS_A1M_get_adc2_data_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC2_DATA_RG_ADDR) & TPDAS_A1M_ADDA_DATA_BITMASK)
#define TPDAS_A1M_get_adc3_data_rg()				((*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC3_DATA_RG_ADDR) & TPDAS_A1M_ADDA_DATA_BITMASK)

#define TPDAS_A1M_set_dac0_data_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC0_DATA_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_set_dac1_data_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC1_DATA_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_set_dac2_data_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC2_DATA_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))
#define TPDAS_A1M_set_dac3_data_rg(d)				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC3_DATA_RG_ADDR) = ((__TPDAS_A1M_DATA_TYPE__)(d))

#define TPDAS_A1M_get_adc0_data()					TPDAS_A1M_get_adc0_data_rg()				
#define TPDAS_A1M_get_adc1_data()					TPDAS_A1M_get_adc1_data_rg()				
#define TPDAS_A1M_get_adc2_data()					TPDAS_A1M_get_adc2_data_rg()				
#define TPDAS_A1M_get_adc3_data()					TPDAS_A1M_get_adc3_data_rg()				

#define TPDAS_A1M_set_dac0_data(d)					TPDAS_A1M_set_dac0_data_rg(d)
#define TPDAS_A1M_set_dac1_data(d)					TPDAS_A1M_set_dac1_data_rg(d)
#define TPDAS_A1M_set_dac2_data(d)					TPDAS_A1M_set_dac2_data_rg(d)
#define TPDAS_A1M_set_dac3_data(d)					TPDAS_A1M_set_dac3_data_rg(d)

	// direct set/get macros
	// GET macros for ADC/ADC-FIFO data 
#define TPDAS_A1M_GET_ADC_FIFO_DATA					TPDAS_A1M_get_adc_fifo_data_rg()
#define TPDAS_A1M_SET_DAC_FIFO_DATA(d)				TPDAS_A1M_set_dac_fifo_data(d)

#define TPDAS_A1M_GET_ADC0_DATA						TPDAS_A1M_get_adc0_data_rg()					
#define TPDAS_A1M_GET_ADC1_DATA						TPDAS_A1M_get_adc1_data_rg()					
#define TPDAS_A1M_GET_ADC2_DATA						TPDAS_A1M_get_adc2_data_rg()					
#define TPDAS_A1M_GET_ADC3_DATA						TPDAS_A1M_get_adc3_data_rg()					

#define TPDAS_A1M_SET_DAC0_DATA(d)					TPDAS_A1M_set_dac0_data_rg(d)
#define TPDAS_A1M_SET_DAC1_DATA(d)					TPDAS_A1M_set_dac1_data_rg(d)
#define TPDAS_A1M_SET_DAC2_DATA(d)					TPDAS_A1M_set_dac2_data_rg(d)
#define TPDAS_A1M_SET_DAC3_DATA(d)					TPDAS_A1M_set_dac3_data_rg(d)

#define TPDAS_A1M_GET_ADC_FIFO_DATA_EOS				(TPDAS_A1M_get_adc_fifo_data_eos() & TPDAS_A1M_ADC_FIFO_MSW_EOS_BITMASK)
#define TPDAS_A1M_ADC_FIFO_DATA_EOS_IS_OFF			(TPDAS_A1M_GET_ADC_FIFO_DATA_EOS==TPDAS_A1M_ADC_FIFO_MSW_EOS_OFF)				
#define TPDAS_A1M_ADC_FIFO_DATA_EOS_IS_ON			(TPDAS_A1M_GET_ADC_FIFO_DATA_EOS==TPDAS_A1M_ADC_FIFO_MSW_EOS_OFF)				

	// sign/unsign-extended 32/16-bit macros for direct usage in expressions
#define TPDAS_A1M_CONV_ADC_TO_INT32(d)				((((int)d)<<16)>>16)
#define TPDAS_A1M_CONV_ADC_TO_UINT32(d)				((unsigned)(d))
#define TPDAS_A1M_CONV_ADC_TO_INT16(d)				((int short)(d))
#define TPDAS_A1M_CONV_ADC_TO_UINT16(d)				((unsiged short)(d))

#define TPDAS_A1M_CONV_INT32_TO_DAC(d)				(((__TPDAS_A1M_DATA_TYPE__)(d)))
#define TPDAS_A1M_CONV_UINT32_TO_DAC(d)				(((__TPDAS_A1M_DATA_TYPE__)(d)))
#define TPDAS_A1M_CONV_INT16_TO_DAC(d)				((__TPDAS_A1M_DATA_TYPE__) ((unsigned short)d))
#define TPDAS_A1M_CONV_UINT16_TO_DAC(d)				((__TPDAS_A1M_DATA_TYPE__) ((unsigned short)d))


	//  ADC START and DAC LOAD commands for ADC/DAC ASYNC DAQ modes
#define TPDAS_A1M_set_async_adc_start_rg()				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_START_RG_ADDR) = 0
#define TPDAS_A1M_set_async_dac_output_rg()				(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_DAC_OUTPUT_RG_ADDR) = 0
#define TPDAS_A1M_set_async_adc_start_dac_output_rg()	(*(__TPDAS_A1M_DATA_TYPE__ *)TPDAS_A1M_ADC_START_DAC_OUTPUT_RG_ADDR) = 0

#define TPDAS_A1M_SET_ASYNC_ADC_START				TPDAS_A1M_set_async_adc_start_rg()		
#define TPDAS_A1M_SET_ASYNC_DAC_OUTPUT				TPDAS_A1M_set_async_dac_output_rg()
#define TPDAS_A1M_SET_ASYNC_ADC_START_DAC_OUTPUT	TPDAS_A1M_set_async_adc_start_dac_output_rg()
//==============================================================================================



//====================== optional/extension functions ==========================================
/*
 	Optional/Extended API functions bodies can be either included or excluded into user source code
 	during compilation of user application. Inclusion/exclusion is controlled via run-time
	compiler keys (definitions). This is required in order to reduce size of DSP code (is extended
	API functions are not required), and in order to exclude mutiple function body appearance.

	__TPDAS_A1M_EXCLUDE_FIFO_CONTROL_API__
		- if defined in user code prior inclusion of this header file, 
		  then the bodies for ADC/DAC FIFO OFFSET programming functions will be 
		  excluded from the user source code. Default is INCLUDED. Note, that in case 
		  user application comprises of several .C linked together modules, then this 
		  run-time compiler key must be included in all modules except for the one module in
		  order to exclude multiple function body appearance.
*/




int TPDAS_A1M_set_adc_fifo_offset(unsigned pae, unsigned paf);
int TPDAS_A1M_set_dac_fifo_offset(unsigned pae, unsigned paf);


#ifndef __TPDAS_A1M_EXCLUDE_FIFO_CONTROL_API__
//--------------- ADC/DAC FIFO OFFSET programming API functions ------------------------------
// These functions perform programming of ADC/DAC FIFO PAF/PAE flags.
// ADC/DAC FIFO PAE/PAF params values are assumed to be the values
// origined to the FIFO address #0.
// Note different PAE/PAF flags adjustment for ADC and DAC FIFOs in order
// to write/read exactly specified number of FIFO samples.

int TPDAS_A1M_set_adc_fifo_offset(unsigned pae, unsigned paf)
{ int i;
  unsigned d;
  unsigned initial_adc_daq_mode;

        	//-- validate input params
	if(paf>=TPDAS_A1M_FIFO_LEN  || pae>=TPDAS_A1M_FIFO_LEN) return(-1);

		// save initial ADC DAQ mode
	initial_adc_daq_mode = TPDAS_A1M_GET_ADC_DAQ_MODE;

		// set ADC FIFO CNF mode and perform FIFO reset
    TPDAS_A1M_SET_ADC_DAQ_MODE_ASYNC;	
	TPDAS_A1M_RESET_ADC_DAQ;

		// load 18-bit PAE data starting fm LSB
		// Note!!!
		//   Offset value for the PAE flag, which has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO address #0, so input PAE function parameter
		//   is OK to be transmitted directly to FIFO.
		//   Output FIFO PAE flag will be set active during
		//   writing of PAE samples to FIFO and will be set inactive
		//   after writing the (PAE+1)-th FIFO sample (in case
		//   not FIFO reads is perfromed). Output FIFO PAE flag
		//   is updated on FIFO RCLK. 

    d = pae;
	for(i=0;i<18; i++)
    {
    	TPDAS_A1M_set_adc_fifo_sin_rg(d) ;
	   	d >>= 1;
	}


		// load 18-bit PAF data starting fm LSB
		// Note!!!
		//   FIFO chip requires that PAF offset value has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO LEN, and not to the FIFO address #0. So,
		//   input PAF function parameter has to be converted
		//   to (FIFO_LEN-PAF) prior transmision to FIFO.

	d = TPDAS_A1M_FIFO_LEN-paf;
	for(i=0;i<18; i++)
 	{
    	TPDAS_A1M_set_adc_fifo_sin_rg(d) ;
	   	d >>= 1;
	}

		// restore initial ADC DAQ mode
	TPDAS_A1M_set_adc_daq_mode(initial_adc_daq_mode);

		// exit OK
	return(0);
}



int TPDAS_A1M_set_dac_fifo_offset(unsigned pae, unsigned paf)
{ int i;
  unsigned d;
  unsigned initial_dac_daq_mode;

        	//-- validate input params
	if(paf>=TPDAS_A1M_FIFO_LEN  || pae>=TPDAS_A1M_FIFO_LEN) return(-1);

		// save initial DAC DAQ mode
	initial_dac_daq_mode = TPDAS_A1M_GET_DAC_DAQ_MODE;

		// set DAC FIFO CNF mode and perform FIFO reset
	TPDAS_A1M_SET_DAC_DAQ_MODE_ASYNC;	
	TPDAS_A1M_RESET_DAC_DAQ;

		// load 18-bit PAE data starting fm LSB
		// Note!!!
		//   FIFO chip requires that PAE offset value must be origined to the
		//   FIFO address #0, so input PAE function parameter
		//   is OK to be transmitted directly to FIFO.
		//   Output FIFO PAE flag will be set active during
		//   writing of PAE samples to FIFO and will be set inactive
		//   after writing the (PAE+1)-th FIFO sample (in case
		//   not FIFO reads is perfromed). 

	d = pae;
	for(i=0;i<18; i++)
    {
    	TPDAS_A1M_set_dac_fifo_sin_rg(d) ;
	   	d >>= 1;
	}


		// load 18-bit PAF data starting fm LSB
		// Note!!!
		//   FIFO chip requires that PAF offset value has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO LEN, and not to the FIFO address #0. So,
		//   input PAF function parameter has to be converted
		//   to (FIFO_LEN-PAF) prior transmision to FIFO.
		//   The output FIFO PAF flag will be set active after writing
		//   (FIFO_LEN-PAF) FIFO samples and is updated on DAC FIFO WCLK.

	d = TPDAS_A1M_FIFO_LEN-paf;
	for(i=0;i<18; i++)
	{
    	TPDAS_A1M_set_dac_fifo_sin_rg(d) ;
	   	d >>= 1;
	}

		// restore initial DAC DAQ mode
	TPDAS_A1M_set_dac_daq_mode(initial_dac_daq_mode);

		// exit OK
	return(0);
}


//------------------------------------------------------------------------------------------

#endif


//==============================================================================================

#endif
